//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoAppProvorCycleDecoder 
// ROL : This class manage the life cycle of the processing
//		 - to read a given file containing one cycle of one profiler
//		 - to read the argos messages in it
//		 - to decod the provor frames
//		 - write the decoded records in an ASCII output file
//
// CRE : 26/08/1999
// AUT : ST
// VER : @(#)1.38
//       @(#)04/03/02
//
//****************************************************************************
// 
// HST : 26/08/1999 ST  Creation
//		 22/09/1999 ST  FAE 004 : Append the missing '\n'
//		 22/09/1999 ST  FAE 005 : Tests for the end of life
//		 24/09/1999 ST  FAE 006 : Send status messages to the standard output
//		 24/09/1999 ST  FAE 013 : Add SurfDriftFrame only if it exists !!
//		 15/10/1999 ST  FAE 014 : Manage relative dating modulo
//		 15/10/1999 ST  FAE 021 : Add a filter flag to FilterProvorFrames
//		 15/10/1999 ST  FAE 023 : Pos QC of the descent profile
//		 15/10/1999 ST  FAE 024 : Descending profile start time
//		 18/07/2000 ST  Use EV delay to compute the descending profile start time
//		 30/11/2000 ST  FAE 070 : Use EV delay to compute immersion drift profile time
//		 07/09/2001 ST  FAE 127 : Profiler info are no more profiler specific
//		 26/09/2001 ST  FAE 122 : Psal instead of Cndc in Metocean profilers
//		 27/09/2001 ST  FAE 126 : Apply clock-delta to immersion drift records
//		 03/10/2001 ST  FAE 124 : Provor 3.61 new fields
//		 18/10/2001 ST  FAE 139 : Do not forget to count bad types frames
//		 04/12/2001 ST  FAE 096 : Ascending profiles uses descent prsep !
//		 04/12/2001 ST  FAE 097 : Compute the clock drift using the emitting date-time
//		 05/12/2001 ST  FAE 167 : Do not use ascent start time to compute the ascent time
//		 27/12/2001 ST  Now using the XML report
//		 04/01/2002 ST  CoDatProfile -> CoDatImmProfile
//		 11/02/2002 ST  Do not use CoStcTPtrSlist any more
//		 22/02/2002 ST  Use CoStcTools instead of CoStcTime
//		 26/02/2002 ST  Use CoDatPosition instead of CoStcLocalization
//		 12/08/2002 ST  Now use CoDatRecord
//		 13/08/2002 ST  FAE 205 : Manage double in Argos messages
//		 11/10/2002 ST  Rewriting for future Coriolis V2 compatibility
//		 02/12/2002 ST  Coriolis V2 compatibility
//		 06/12/2002 ST  Remove an unused include (CoDatPreofilerInfo)
//		 24/01/2003 ST  FDI 114 : Provor 3.81
//		 05/02/2003 ST  FRA 167 : Pressure & date/time modulo in descending profile
//		 24/02/2003 ST  FRA 179 : If ASCFC_Start_time is defined, use it
//		 25/02/2003 ST  FRA 180 : Bug in FillImmersionDrift when there is no record
//		 22/04/2003 ST  FRA 207 : In FillImmersionDrift, do not apply _timeCorrection twice !
//		 14/10/2003 ST  FDI 145 : Provor 4.1
//		 02/03/2004 ST  Manage Provor 4.11
//
//****************************************************************************
//
// Copyright, IFREMER, 1999
//
//****************************************************************************
static const char CoAppProvorCycleDecoder_IDENT[] = "@(#) CORIOLIS : CoAppProvorCycleDecoder.cpp 1.38 04/03/02";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoAppProvorCycleDecoder_IN_
#include <CoAppProvorCycleDecoder.h>
#undef  _CoAppProvorCycleDecoder_IN_
#include <CoDfaArgosFile.h>
#include <CoDbaConnection.h>
#include <CoDbaPlatformAccessor.h>
#include <CoDbaTrajectoryAccessor.h>
#include <CoDfaDecodedCycleFile.h>
#include <CoDatArgosMessage.h>
#include <CoDatAttribute.h>
#include <CoDatLocatedRecord.h>
#include <CoDatMeasure.h>
#include <CoDatPhysicalParameterCodes.h>
#include <CoDatProvorFrame.h>
#include <CoStcCatManager.h>
#include <CoStcError.h>
#include <CoStcFileConf.h>
#include <CoStcReport.h>
#include <CoStcTools.h>
#include <stdlib.h>



//----------------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------------
const double CoApp_k_profileOptions_4_0[18][2][6] =
{
	 0.0,   0.0,  0.0,    0.0,  0.0,    0.0,		 2.0, 200.0,  5.0, 1500.0, 10.0, 2000.0,
	 0.0,   0.0,  0.0,    0.0,  0.0,    0.0,		 2.0, 100.0, 10.0, 1200.0, 20.0, 2000.0,
	 0.0,   0.0,  0.0,    0.0,  0.0,    0.0,		 5.0, 200.0, 10.0, 1200.0, 20.0, 2000.0,
	 0.0,   0.0,  0.0,    0.0,  0.0,    0.0,		 5.0, 100.0, 10.0,  400.0, 20.0, 2000.0,
	 0.0,   0.0,  0.0,    0.0,  0.0,    0.0,		10.0, 100.0, 20.0, 1500.0, 50.0, 2000.0,

	 2.0,  20.0, 10.0,  220.0, 20.0, 1020.0,		 2.0,  20.0, 10.0,  220.0, 20.0, 1020.0,
	 5.0,  50.0, 25.0, 1050.0,  0.0, 1050.0,		 2.0,  20.0, 10.0,  220.0, 20.0, 1020.0,
	10.0, 100.0, 50.0, 1100.0,  0.0, 1100.0,		 2.0,  20.0, 10.0,  220.0, 20.0, 1020.0,
	 5.0,  50.0, 25.0, 1050.0,  0.0, 1050.0,		 5.0,  50.0, 25.0, 1050.0,  0.0, 1050.0,
	10.0, 100.0, 50.0, 1100.0,  0.0, 1100.0,		 5.0,  50.0, 25.0, 1050.0,  0.0, 1050.0,
	10.0, 100.0, 50.0, 1100.0,  0.0, 1100.0,		10.0, 100.0, 50.0, 1100.0,  0.0, 1100.0,

	10.0, 100.0, 50.0, 1100.0,  0.0, 1100.0,		 2.0, 100.0, 10.0, 1200.0, 20.0, 2000.0,
	 5.0,  50.0, 25.0, 1050.0,  0.0, 1050.0,		 5.0, 200.0, 10.0, 1200.0, 20.0, 2000.0,
	10.0, 100.0, 50.0, 1100.0,  0.0, 1100.0,		 5.0, 200.0, 10.0, 1200.0, 20.0, 2000.0,
	 5.0,  50.0, 25.0, 1050.0,  0.0, 1050.0,		 5.0, 100.0, 10.0,  400.0, 20.0, 2000.0,
	10.0, 100.0, 50.0, 1100.0,  0.0, 1100.0,		 5.0, 100.0, 10.0,  400.0, 20.0, 2000.0,
	 5.0,  50.0, 25.0, 1050.0,  0.0, 1050.0,		10.0, 100.0, 20.0, 1500.0, 50.0, 2000.0,
	10.0, 100.0, 50.0, 1100.0,  0.0, 1100.0,		10.0, 100.0, 20.0, 1500.0, 50.0, 2000.0
};




//****************************************************************************
//
// SPG : CoAppProvorCycleDecoder
// ROL : Constructor
//
//****************************************************************************
CoAppProvorCycleDecoder::
CoAppProvorCycleDecoder(
    )
	: CoAppProcess()
	, _ignoreCrcErrors(FALSE)
	, _inputFileName("")
	, _outputFileName("")
	, _q_dbConnection((CoDbaConnection*)0)
	, _q_platformAccessor((CoDbaPlatformAccessor*)0)
	, _q_trajectoryAccessor((CoDbaTrajectoryAccessor*)0)
	, _argosMessages()
	, _platform()
	, _version(0.0)
	, _technicalStatus()
	, _timeCorrection(CoDat_k_undefinedInt)
	, _technicalFrames()
	, _descentFrames()
	, _driftFrames()
	, _ascentFrames()
	, _endOfLife(FALSE)
	, _descentProfile()
	, _immersionDrift()
	, _ascentProfile()
	, _surfaceDrift()
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Check the parent instance validity
		//********************************************************************

		if (_isBad == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 166", 201);
			CO_THROW(CoStc_k_error);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 177", 202);
		_isBad = TRUE;
	}
}




//****************************************************************************
//
// SPG : ~CoAppProvorCycleDecoder
// ROL : Destructor
//
//****************************************************************************
CoAppProvorCycleDecoder::
~CoAppProvorCycleDecoder(
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : Run
// ROL : Run the processing
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
Run(
    int p_argc,								// I : Arguments count
    char** qqp_argv							// I : Arguments array
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Check the number of parameters and decode them
		//********************************************************************

		switch (p_argc)
		{
			case 3 :
			{
				_ignoreCrcErrors = FALSE;
				_inputFileName = qqp_argv[1];
				_outputFileName = qqp_argv[2];
			}
			break;

			case 4 :
			{
				if (RWCString(qqp_argv[1]) == "-i")
				{
					_ignoreCrcErrors = TRUE;
					_inputFileName = qqp_argv[2];
					_outputFileName = qqp_argv[3];
				}
				else
				{
					CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 259", 203);
					CO_THROW(CoStc_k_error);
				}
			}
			break;

			default :
			{
				CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 267", 203);
				CO_THROW(CoStc_k_error);
			}
		}

		//********************************************************************
		// Run the life cycle
		//********************************************************************

		CoAppProcess::Run();
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 285", 204);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Initialize
// ROL : Initialize the processing
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
Initialize(
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Create the database connection and the database accessors.
		//********************************************************************

		_q_dbConnection = new CoDbaConnection();
		if (_q_dbConnection->IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 321", 205);
			CO_THROW(CoStc_k_error);
		}
		_q_platformAccessor = new CoDbaPlatformAccessor(_q_dbConnection);
		if (_q_platformAccessor->IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 327", 206);
			CO_THROW(CoStc_k_error);
		}
		_q_trajectoryAccessor = new CoDbaTrajectoryAccessor(_q_dbConnection);
		if (_q_trajectoryAccessor->IsBad() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 333", 207);
			CO_THROW(CoStc_k_error);
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 344", 208);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Compute
// ROL : Realize the processing
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
Compute(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Extract the argos messages from the input file
		//********************************************************************

		ExtractArgosMessages();

		//********************************************************************
		// Retrieve informations about the current profiler
		//********************************************************************

		RetrieveProfilerInfo();

		//********************************************************************
		// Compute the cycle number
		//********************************************************************

		ComputeCycleNumber();

		//********************************************************************
		// Mark the Argos messages in double
		//********************************************************************

		MarkDoubleArgosMessages();

		//********************************************************************
		// Extract the provor's data from the argos messages and instanciate
		// the corresponding frame objects
		//********************************************************************

		ExtractProvorFrames();

		//********************************************************************
		// Compute and update the transmission statistics and the Argos
		// statistics
		//********************************************************************

		UpdateTransmissionStatistics();
		UpdateArgosStatistics();

		//********************************************************************
		// Sort and filter the lists of frames
		//********************************************************************

		FilterTechnicalFrames();
		FilterDescentFrames();
		FilterDriftFrames();
		FilterAscentFrames();

		//********************************************************************
		// Check for the end of life of the float
		//********************************************************************

		CheckEndOfLife();

		//********************************************************************
		// Fill the profiles
		//********************************************************************

		if (_technicalStatus.HasAttribute("CYCLE_NUMBER") == TRUE)
		{
			FillDescentProfile();
			FillImmersionDrift();
			FillAscentProfile();
		}

		//********************************************************************
		// Compute the parameteers measurements informations for each profile
		//********************************************************************

		if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
			(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
			(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
			(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
		{
			ComputeParametersMeasurementsInformations();
		}
		else if (_version == 4.0)
		{
			ComputeParametersMeasurementsInformationsWithoutPrsep();
		}

		//********************************************************************
		// Write the decoded cycle file
		//********************************************************************

		WriteDecodedCycleFile();
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 467", 209);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Finish
// ROL : Terminate the processing
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
Finish(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Destroy database objects
		//********************************************************************

		if (_q_platformAccessor != (CoDbaPlatformAccessor*)0)
		{
			delete _q_platformAccessor;
			_q_platformAccessor = (CoDbaPlatformAccessor*)0;
		}
		if (_q_trajectoryAccessor != (CoDbaTrajectoryAccessor*)0)
		{
			delete _q_trajectoryAccessor;
			_q_trajectoryAccessor = (CoDbaTrajectoryAccessor*)0;
		}
		if (_q_dbConnection != (CoDbaConnection*)0)
		{
			delete _q_dbConnection;
			_q_dbConnection = (CoDbaConnection*)0;
		}

		//********************************************************************
		// Status is Ok
		//********************************************************************

		CoStc_g_report.AddStatus(TRUE);
		CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 202),
								_platform.GetCode());
		if (_technicalStatus.HasAttribute("CYCLE_NUMBER") == TRUE)
		{
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 203),
									_technicalStatus.GetAttribute("CYCLE_NUMBER").GetIntValue());
		}
		else
		{
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 203), "EOL");
		}
		if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
			(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
			(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
			(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
		{
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 204),
									_technicalStatus.GetAttribute("DESPR_NTS").GetIntValue() +
									_technicalStatus.GetAttribute("DESPR_NTF").GetIntValue());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 205),
									_descentProfile.GetRecordCount());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 206),
									_technicalStatus.GetAttribute("ASCPR_NTS").GetIntValue() +
									_technicalStatus.GetAttribute("ASCPR_NTF").GetIntValue());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 207),
									_ascentProfile.GetRecordCount());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 208),
									_technicalStatus.GetAttribute("ITEMP_Internal_count").GetIntValue());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 209),
									_immersionDrift.GetRecordCount());
		}
		else if (_version == 4.0)
		{
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 210),
									_technicalStatus.GetAttribute("TRANS_DPROF_Emitted").GetIntValue());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 211),
									_technicalStatus.GetAttribute("TRANS_DPROF_Distinc").GetIntValue());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 212),
									_technicalStatus.GetAttribute("TRANS_APROF_Emitted").GetIntValue());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 213),
									_technicalStatus.GetAttribute("TRANS_APROF_Distinc").GetIntValue());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 214),
									_technicalStatus.GetAttribute("TRANS_IDRIFT_Emitted").GetIntValue());
			CoStc_g_report.AddValue(CoStc_g_catManager.GetInfo(1, 215),
									_technicalStatus.GetAttribute("TRANS_IDRIFT_Distinc").GetIntValue());
		}
		CoStc_g_report.End();
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 576", 210);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : ExtractArgosMessages
// ROL : Extract the argos messages from the input file
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
ExtractArgosMessages(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDfaArgosFile l_argosFile;				// The input file
	RWCString l_beaconCode = "";			// The beacon code
	int l_messageCount = 0;					// number of Argos Messages
	int l_messageCounter = 0;				// loop counter

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Read the cycle file and memorize the Argos messages
		//********************************************************************

		l_argosFile.SetFileName(_inputFileName);
		l_argosFile.Read(_argosMessages);

		//********************************************************************
		// Check if there is at least one Argos message
		//********************************************************************

		if (_argosMessages.isEmpty() == TRUE)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 623", 211);
			CO_THROW(CoStc_k_error);
		}

		//********************************************************************
		// Check if all the Argos messages are relative to the same profiler
		//********************************************************************

		l_beaconCode = _argosMessages.first()->GetBeaconCode();
		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = 0; l_messageCounter < l_messageCount; l_messageCounter++)
		{
			if (_argosMessages.at(l_messageCounter)->GetBeaconCode() != l_beaconCode)
			{
				CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 637", 212);
				CO_THROW(CoStc_k_error);
			}
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 649", 213);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : RetrieveProfilerInfo
// ROL : Retrieve the info for the current profiler
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
RetrieveProfilerInfo(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWCString l_beaconCode = "";			// the platform beacon code

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve the beacon code and select the corresponding platform
		//********************************************************************

		l_beaconCode = _argosMessages.at(0)->GetBeaconCode();
		_q_platformAccessor->SelectByBeaconCode(l_beaconCode, _platform);

		//********************************************************************
		// Check the mandatory attributes
		//********************************************************************

		_platform.GetAttribute("PR_TYPE");
		_version = _platform.GetAttribute("PR_VERSION").GetDoubleValue();
		_platform.GetAttribute("PR_CRC_TYPE");
		_platform.GetAttribute("PR_LAUNCH_DATETIME");
		_platform.GetAttribute("PR_LAUNCH_LATITUDE");
		_platform.GetAttribute("PR_LAUNCH_LONGITUDE");
		_platform.GetAttribute("PR_REFERENCE_DATETIME");
		_platform.GetAttribute("PR_CYCLE_PERIOD_1");
		_platform.GetAttribute("PR_CYCLE_PERIOD_2");
		_platform.GetAttribute("PR_IMMERSION_DRIFT_PERIOD");
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 709", 214);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : ComputeCycleNumber
// ROL : Compute the cycle number usingthe profiler informations
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
ComputeCycleNumber(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWTime l_launchDateTime =				// The launch date-time
		(unsigned long)0;
	RWTime l_referenceDateTime =			// The reference date-time
		(unsigned long)0;
	int l_cyclePeriod1 = 0;					// The first cycle period
	int l_cyclePeriod2 = 0;					// The second cycle period
	CoDatLocatedRecord* ql_record =			// The first valid surface record
		(CoDatLocatedRecord*)0;
    int l_messageCount = 0;					// number of Argos messages
    int l_messageCounter = 0;				// loop counter
    RWTime l_cycleDateTime =				// Date-time of the first valid surface record
		(unsigned long)0;
	int l_cycleNumber = 0;					// the cycle number
    RWTime l_currentDateTime =				// Date-time used for computation
		(unsigned long)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retreive the required platform attributes
		//********************************************************************

		l_launchDateTime = _platform.GetAttribute("PR_LAUNCH_DATETIME").GetDateTimeValue();
		l_referenceDateTime = _platform.GetAttribute("PR_REFERENCE_DATETIME").GetDateTimeValue();
		l_cyclePeriod1 = _platform.GetAttribute("PR_CYCLE_PERIOD_1").GetIntValue();
		l_cyclePeriod2 = _platform.GetAttribute("PR_CYCLE_PERIOD_2").GetIntValue();

		//********************************************************************
		// Retrieve the first valid surface record, and it's date-time
		//********************************************************************

		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = 0;
			 (l_messageCounter < l_messageCount) && (ql_record == (CoDatLocatedRecord*)0);
			 l_messageCounter++)
		{
			ql_record = _argosMessages.at(l_messageCounter)->GetSurfaceRecord();
		}
		if (ql_record == (CoDatLocatedRecord*)0)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 776", 215);
			CO_THROW(CoStc_k_error);
		}
		l_cycleDateTime = ql_record->GetDateTime();

		//********************************************************************
		// Check if the cycle date-time is subsequent to the launch date-time
		//********************************************************************

		if (l_cycleDateTime < l_launchDateTime)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 787", 216);
			CO_THROW(CoStc_k_error);
		}

		//********************************************************************
		// Compute the cycle number
		//********************************************************************

		l_cycleNumber = 0;
		if (l_cycleDateTime > l_referenceDateTime + 86400)
		{
			do
			{
				l_cycleNumber++;
				l_currentDateTime = l_referenceDateTime +
									int((l_cycleNumber + 1) / 2) * l_cyclePeriod1 * 86400 +
									int((l_cycleNumber) / 2) * l_cyclePeriod2 * 86400;
			}
			while (l_currentDateTime + 86400 < l_cycleDateTime);
		}

		//********************************************************************
		// And memorize the cycle number
		//********************************************************************

		_technicalStatus.AddAttribute(CoDatAttribute("CYCLE_NUMBER", l_cycleNumber));
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 821", 217);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : MarkDoubleArgosMessages
// ROL : Mark the double in the Argos messages
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
MarkDoubleArgosMessages(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_messageCount = 0;					// number of Argos messages
	int l_messageCounter = 0;				// loop counter
	CoDatArgosMessage* ql_argosMessage =	// the current Argos message
		(CoDatArgosMessage*)0;
	CoDatTechnicalStatus l_dummyStatus;		// a dummy technical status
	CoDatLocatedRecord* ql_surfaceRecord =	// the current surface record
		(CoDatLocatedRecord*)0;
	RWTPtrSlist<CoDatProvorFrame> l_frames;	// the list of Provor frames for the current message
	CoDatProvorFrame* ql_frame =				// the curent Provor frame
		(CoDatProvorFrame*)0;
	RWCString l_frameId = "";					// id of the current frame
	RWTValSlist<RWCString> l_frameIds;			// the list of frame id

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// For each argos message (last to first)
		//********************************************************************

		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = l_messageCount - 1; l_messageCounter >= 0; l_messageCounter--)
		{
			//****************************************************************
			// Get the list of Provor frames
			//****************************************************************

			ql_argosMessage = _argosMessages.at(l_messageCounter);
			ql_argosMessage->GetProvorFrames(_platform,
											 l_dummyStatus,
											 ql_surfaceRecord,
											 l_frames);

			//****************************************************************
			// For each Provor frame
			//****************************************************************

			while (l_frames.isEmpty() == FALSE)
			{
				//************************************************************
				// Retrieve the current frame and compute it's id
				//************************************************************

				ql_frame = l_frames.removeFirst();
				l_frameId = RWCString(ql_frame->GetEmittingSatellite()) +
							CoStcTools_ToString(ql_frame->GetEmittingDateTime(), "-%d/%m/%Y %H:%M:%S");
				delete ql_frame;

				//************************************************************
				// If the id exists, mark the message
				//************************************************************

				if (l_frameIds.contains(l_frameId) == TRUE)
				{
					ql_argosMessage->SetDoubleFlag(TRUE);
					l_frames.clearAndDestroy();
				}
				else
				{
					l_frameIds.append(l_frameId);
				}
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 918", 218);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : ExtractProvorFrames
// ROL : Extract the Provor's datas from the argos messages and instanciate
//	 the corresponding frame objects
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
ExtractProvorFrames(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_messageCount = 0;					// number of Argos messages
	int l_messageCounter = 0;				// loop counter
	CoDatArgosMessage* ql_argosMessage =	// the current Argos message
		(CoDatArgosMessage*)0;
	CoDatLocatedRecord* ql_surfaceRecord =	// the current surface record
		(CoDatLocatedRecord*)0;
	RWTPtrSlist<CoDatProvorFrame> l_frames;	// the list of Provor frames for the current message
	CoDatProvorFrame* ql_frame =			// the curent Provor frame
		(CoDatProvorFrame*)0;
	int l_frameType = 0;					// the type of the current frame

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Sort the Argos messages
		//********************************************************************

		CoStcTools_Sort(_argosMessages, CoDatArgosMessage::CompareDateTime);

		//********************************************************************
		// For each argos message
		//********************************************************************

		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = 0; l_messageCounter < l_messageCount; l_messageCounter++)
		{
			//****************************************************************
			// Get the list of Provor frames
			//****************************************************************

			ql_argosMessage = _argosMessages.at(l_messageCounter);
			ql_argosMessage->GetProvorFrames(_platform,
											 _technicalStatus,
											 ql_surfaceRecord,
											 l_frames);

			//****************************************************************
			// For each Provor frame
			//****************************************************************

			while (l_frames.isEmpty() == FALSE)
			{
				//************************************************************
				// Retrieve the current frame and it's type
				//************************************************************

				ql_frame = l_frames.removeFirst();
				l_frameType = ql_frame->GetType();

				//************************************************************
				// If it is a Technical Frame
				//************************************************************

				if (l_frameType == 0)
				{
					//********************************************************
					// Memorize it
					//********************************************************

					_technicalFrames.append(ql_frame);

					//********************************************************
					// If the surface record exists, set it's temperature
					//********************************************************

					if ((ql_frame->IsCrcOk() == TRUE) &&
						(ql_surfaceRecord != (CoDatLocatedRecord*)0))
					{
						if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
							(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
							(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
							(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
						{
							ql_surfaceRecord->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature,
																	  ql_frame->GetSurfaceTemperatureValue()));
						}
					}
				}

				//************************************************************
				// If it is a Descent Frame
				//************************************************************

				else if ((l_frameType == 1) || (l_frameType == 4))
				{
					_descentFrames.append(ql_frame);
				}

				//************************************************************
				// If it is a Drift Frame
				//************************************************************

				else if ((l_frameType == 2) || (l_frameType == 5))
				{
					_driftFrames.append(ql_frame);
				}

				//************************************************************
				// If it is an Ascent Frame
				//************************************************************

				else if ((l_frameType == 3) || (l_frameType == 6))
				{
					_ascentFrames.append(ql_frame);
				}
			}

			//****************************************************************
			// Memorize the surface record
			//****************************************************************

			if ((ql_surfaceRecord != (CoDatLocatedRecord*)0) &&
				(ql_argosMessage->GetDoubleFlag() == FALSE))
			{
				_surfaceDrift.AddRecord(ql_surfaceRecord);
			}
		}

		//********************************************************************
		// Sort the surface drift
		//********************************************************************

		_surfaceDrift.SortRecordsByDateTime();
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1077", 219);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : UpdateTransmissionStatistics
// ROL : Update the transmission statistics
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
UpdateTransmissionStatistics(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_receivedFrames = 0;				// Number of received frames (without doubles)
	int l_completeFrames = 0;				// Number of complete frames (without doubles)
	int l_incompleteFrames = 0;				// Number of incomplete frames
	int l_badTypeFrames = 0;				// Number of bad type frames
	int l_crcOkFrames = 0;					// Number of crc ok frames (without doubles)
	int l_receivedTechnicalFrames = 0;		// Number of received technical frames (without doubles)
	int l_crcOkTechnicalFrames = 0;			// Number of crc ok technical frames (without doubles)
	int l_receivedDescentFrames = 0;		// Number of received descent frames (without doubles)
	int l_completeDescentFrames = 0;		// Number of complete descent frames (without doubles)
	int l_incompleteDescentFrames = 0;		// Number of complete descent frames (without doubles)
	int l_crcOkDescentFrames = 0;			// Number of crc ok descent frames (without doubles)
	int l_receivedDriftFrames = 0;			// Number of received drift frames (without doubles)
	int l_completeDriftFrames = 0;			// Number of complete drift frames (without doubles)
	int l_incompleteDriftFrames = 0;		// Number of complete drift frames (without doubles)
	int l_crcOkDriftFrames = 0;				// Number of crc ok drift frames (without doubles)
	int l_receivedAscentFrames = 0;			// Number of received ascent frames (without doubles)
	int l_completeAscentFrames = 0;			// Number of complete ascent frames (without doubles)
	int l_incompleteAscentFrames = 0;		// Number of complete ascent frames (without doubles)
	int l_crcOkAscentFrames = 0;			// Number of crc ok ascent frames (without doubles)
	int l_frameCount = 0;					// number of frames
	int l_frameCounter = 0;					// loop counter
	RWTime l_emittingDateTime =				// the current emitting date/time
		(unsigned long)0;
	char l_emittingSatellite ='\0';			// the current emitting satellite
	CoDatProvorFrame* ql_frame =			// the current Provor frame
		(CoDatProvorFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Initialize the counts
		//********************************************************************

		l_incompleteDescentFrames = _technicalStatus.GetAttribute("TRANS_DPROF_Incompl").GetIntValue();
		l_incompleteDriftFrames = _technicalStatus.GetAttribute("TRANS_IDRIFT_Incompl").GetIntValue();
		l_incompleteAscentFrames = _technicalStatus.GetAttribute("TRANS_APROF_Incompl").GetIntValue();
		l_incompleteFrames = _technicalStatus.GetAttribute("TRANS_OTHER_Incompl").GetIntValue() +
							 l_incompleteDescentFrames +
							 l_incompleteDriftFrames +
							 l_incompleteAscentFrames;
		l_badTypeFrames = _technicalStatus.GetAttribute("TRANS_Bad_type").GetIntValue();
		l_receivedDescentFrames = l_incompleteDescentFrames;
		l_receivedDriftFrames = l_incompleteDriftFrames;
		l_receivedAscentFrames = l_incompleteAscentFrames;
		l_receivedFrames = l_incompleteFrames + l_badTypeFrames;

		//********************************************************************
		// Remove doubles from technical frames
		//********************************************************************

		l_frameCount = _technicalFrames.entries();
		if (l_frameCount != 0)
		{
			CoStcTools_Sort(_technicalFrames, CoDatDataFrame::CompareEmittingDateTimeAndSatellite);
			l_emittingDateTime = RWTime((unsigned long)0);
			l_emittingSatellite = '\0';
			for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
			{
				ql_frame = _technicalFrames.at(l_frameCounter);
				if ((ql_frame->GetEmittingDateTime() != l_emittingDateTime) ||
					(ql_frame->GetEmittingSatellite() != l_emittingSatellite))
				{
					l_receivedFrames++;
					l_completeFrames++;
					l_receivedTechnicalFrames++;
					if (ql_frame->IsCrcOk() == TRUE)
					{
						l_crcOkFrames++;
						l_crcOkTechnicalFrames++;
					}
					l_emittingDateTime = ql_frame->GetEmittingDateTime();
					l_emittingSatellite = ql_frame->GetEmittingSatellite();
				}
			}
		}

		//********************************************************************
		// Remove doubles from descent frames
		//********************************************************************

		l_frameCount = _descentFrames.entries();
		if (l_frameCount != 0)
		{
			CoStcTools_Sort(_descentFrames, CoDatProvorFrame::CompareEmittingDateTimeAndSatellite);
			l_emittingDateTime = RWTime((unsigned long)0);
			l_emittingSatellite = '\0';
			for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
			{
				ql_frame = _descentFrames.at(l_frameCounter);
				if ((ql_frame->GetEmittingDateTime() != l_emittingDateTime) ||
					(ql_frame->GetEmittingSatellite() != l_emittingSatellite))
				{
					l_receivedFrames++;
					l_completeFrames++;
					l_receivedDescentFrames++;
					l_completeDescentFrames++;
					if (ql_frame->IsCrcOk() == TRUE)
					{
						l_crcOkFrames++;
						l_crcOkDescentFrames++;
					}
					l_emittingDateTime = ql_frame->GetEmittingDateTime();
					l_emittingSatellite = ql_frame->GetEmittingSatellite();
				}
			}
		}

		//********************************************************************
		// Remove doubles from immersion drift frames
		//********************************************************************

		l_frameCount = _driftFrames.entries();
		if (l_frameCount != 0)
		{
			CoStcTools_Sort(_driftFrames, CoDatProvorFrame::CompareEmittingDateTimeAndSatellite);
			l_emittingDateTime = RWTime((unsigned long)0);
			l_emittingSatellite = '\0';
			for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
			{
				ql_frame = _driftFrames.at(l_frameCounter);
				if ((ql_frame->GetEmittingDateTime() != l_emittingDateTime) ||
					(ql_frame->GetEmittingSatellite() != l_emittingSatellite))
				{
					l_receivedFrames++;
					l_completeFrames++;
					l_receivedDriftFrames++;
					l_completeDriftFrames++;
					if (ql_frame->IsCrcOk() == TRUE)
					{
						l_crcOkFrames++;
						l_crcOkDriftFrames++;
					}
					l_emittingDateTime = ql_frame->GetEmittingDateTime();
					l_emittingSatellite = ql_frame->GetEmittingSatellite();
				}
			}
		}

		//********************************************************************
		// Remove doubles from ascent frames
		//********************************************************************

		l_frameCount = _ascentFrames.entries();
		if (l_frameCount != 0)
		{
			CoStcTools_Sort(_ascentFrames, CoDatProvorFrame::CompareEmittingDateTimeAndSatellite);
			l_emittingDateTime = RWTime((unsigned long)0);
			l_emittingSatellite = '\0';
			for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
			{
				ql_frame = _ascentFrames.at(l_frameCounter);
				if ((ql_frame->GetEmittingDateTime() != l_emittingDateTime) ||
					(ql_frame->GetEmittingSatellite() != l_emittingSatellite))
				{
					l_receivedFrames++;
					l_completeFrames++;
					l_receivedAscentFrames++;
					l_completeAscentFrames++;
					if (ql_frame->IsCrcOk() == TRUE)
					{
						l_crcOkFrames++;
						l_crcOkAscentFrames++;
					}
					l_emittingDateTime = ql_frame->GetEmittingDateTime();
					l_emittingSatellite = ql_frame->GetEmittingSatellite();
				}
			}
		}

		//********************************************************************
		// Update the status
		//********************************************************************

		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Received", l_receivedFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Complete", l_completeFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_CRC_OK", l_crcOkFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Tech_frames", l_receivedTechnicalFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Tech_OK", l_crcOkTechnicalFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_DPROF_Rcvd", l_receivedDescentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_DPROF_Complet", l_completeDescentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_DPROF_OK", l_crcOkDescentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Rcvd", l_receivedDriftFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Complet", l_completeDriftFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_IDRIFT_OK", l_crcOkDriftFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Rcvd", l_receivedAscentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Complet", l_completeAscentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_OK", l_crcOkAscentFrames));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1299", 220);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : UpdateArgosStatistics
// ROL : Update the Argos statistics
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
UpdateArgosStatistics(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_messageCount = 0;					// Number of Argos messages
	int l_messageCounter = 0;				// Loop counter
	CoDatArgosMessage* ql_argosMessage =	// The current Argos message
		(CoDatArgosMessage*)0;
	CoDatLocatedRecord* ql_record =			// The surface record
		(CoDatLocatedRecord*)0;
	char l_class = '\0';					// current Argos class
	int l_argosClass1 = 0;					// number of class 1 messages
	int l_argosClass2 = 0;					// number of class 2 messages
	int l_argosClass3 = 0;					// number of class 3 messages
	int l_argosRejected = 0;				// number of rejected messages
	int l_argosTotal = 0;					// number of messages

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// For each Argos messages
		//********************************************************************

		l_messageCount = _argosMessages.entries();
		for (l_messageCounter = 0; l_messageCounter < l_messageCount; l_messageCounter++)
		{
			//****************************************************************
			// If the current Argos message is not the double for another one
			//****************************************************************

			ql_argosMessage = _argosMessages.at(l_messageCounter);
			if (ql_argosMessage->GetDoubleFlag() == FALSE)
			{
				//************************************************************
				// if the argos message has a surface record
				//************************************************************

				ql_record = ql_argosMessage->GetSurfaceRecord();
				if (ql_record != (CoDatLocatedRecord*)0)
				{
					//********************************************************
					// Depending of the current Argos class
					//********************************************************

					l_class = ql_record->GetPosition().GetClass();
					switch (l_class)
					{
					    case '1' :
							l_argosClass1++;
					    break;

					    case '2' :
							l_argosClass2++;
					    break;

					    case '3' :
							l_argosClass3++;
					    break;

					    default :
							l_argosRejected++;
					    break;
					}
				}

				//************************************************************
				// If the argos message hos no surface record
				//************************************************************

				else
				{
					l_argosRejected++;
				}
			}
		}

		//********************************************************************
		// Update the status
		//********************************************************************

		l_argosTotal = l_argosClass1 + l_argosClass2 + l_argosClass3 + l_argosRejected;
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Class_1", l_argosClass1));
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Class_2", l_argosClass2));
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Class_3", l_argosClass3));
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Rejected", l_argosRejected));
		_technicalStatus.AddAttribute(CoDatAttribute("POS_Total", l_argosTotal));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1416", 221);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : FilterTechnicalFrames
// ROL : Filter the list of technical frames in order to find a valid one
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
FilterTechnicalFrames(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_frameCount = 0;					// number of frames
	int l_frameCounter = 0;					// loop counter
	CoDatProvorFrame* ql_frame =			// the curent Provor frame
		(CoDatProvorFrame*)0;
	RWTime l_floatTime = (unsigned long)0;	// the float time
	RWTime l_argosTime = (unsigned long)0;	// the argos time

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Find a valid technical Frame : 1st chance
		//********************************************************************

		l_frameCount = _technicalFrames.entries();
		if (l_frameCount == 0)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1459", 222);
			CO_THROW(CoStc_k_error);
		}
		for (l_frameCounter = 0, ql_frame = (CoDatProvorFrame*)0;
			 (l_frameCounter < l_frameCount) && (ql_frame == (CoDatProvorFrame*)0);
			 l_frameCounter++)
		{
			ql_frame = _technicalFrames.at(l_frameCounter);
			if (ql_frame->IsCrcOk() == FALSE)
			{
				ql_frame = (CoDatProvorFrame*)0;
			}
		}

		//********************************************************************
		// Find a valid technical Frame : 2nd chance
		//********************************************************************

		if ((ql_frame == (CoDatProvorFrame*)0) &&
			(_ignoreCrcErrors == TRUE))
		{
			ql_frame = _technicalFrames.first();
		}

		//********************************************************************
		// Find a valid technical Frame : No more chance
		//********************************************************************

		if (ql_frame == (CoDatProvorFrame*)0)
		{
			CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1489", 223);
			CO_THROW(CoStc_k_error);
		}

		//********************************************************************
		// Compute the time correction. The time correction is stored in the
		// technical status with the seconds but is rounded to the minute
		// for future computations
		//********************************************************************

		l_floatTime = ql_frame->GetFloatTime();
		l_argosTime = ql_frame->GetEmittingDateTime();
		l_argosTime = RWTime(l_argosTime.hour(),
							 l_argosTime.minute(),
							 l_argosTime.second());
		_timeCorrection = l_floatTime.seconds() - l_argosTime.seconds();
		if (_timeCorrection > 86400 / 2)
		{
			_timeCorrection -= 86400;
		}
		_technicalStatus.AddAttribute(CoDatAttribute("TIMEC_Internal_time", l_floatTime));
		_technicalStatus.AddAttribute(CoDatAttribute("TIMEC_Argos_time", l_argosTime));
		_technicalStatus.AddAttribute(CoDatAttribute("TIMEC_Time_correct", _timeCorrection));
		_timeCorrection = int(rint(double(_timeCorrection) / 60.0)) * 60;

		//********************************************************************
		// Initialize the technical status
		//********************************************************************

		ql_frame->GetTechnicalStatus(_timeCorrection, _technicalStatus);
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1527", 224);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : FilterDescentFrames
// ROL : Filter the list of descent frames in order to remove doubles
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
FilterDescentFrames(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_recombinedFrames = 0;				// Number of recombined frames
	int l_recombinedDescentFrames = 0;		// Number of recombined descent frames
	int l_distinctDescentFrames = 0;		// Number of distinct descent frames

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Initialize the counts
		//********************************************************************

		if (_technicalStatus.HasAttribute("TRANS_Recombined") == TRUE)
		{
			l_recombinedFrames = _technicalStatus.GetAttribute("TRANS_Recombined").GetIntValue();
		}

		//********************************************************************
		// Call the generic filter method
		//********************************************************************

		FilterDataFrames(_descentFrames, _ignoreCrcErrors, l_recombinedDescentFrames);

		//********************************************************************
		// Update the status
		//********************************************************************

		l_recombinedFrames += l_recombinedDescentFrames;
		l_distinctDescentFrames = _descentFrames.entries();
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Recombined", l_recombinedFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_DPROF_Recom", l_recombinedDescentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_DPROF_Distinc", l_distinctDescentFrames));
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1592", 225);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : FilterDriftFrames
// ROL : Filter the list of drift frames in order to remove doubles
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
FilterDriftFrames(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_recombinedFrames = 0;				// Number of recombined frames
	int l_recombinedDriftFrames = 0;		// Number of recombined drift frames
	int l_distinctDriftFrames = 0;		// Number of distinct drift frames

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Initialize the counts
		//********************************************************************

		if (_technicalStatus.HasAttribute("TRANS_Recombined") == TRUE)
		{
			l_recombinedFrames = _technicalStatus.GetAttribute("TRANS_Recombined").GetIntValue();
		}

		//********************************************************************
		// Call the generic filter method
		//********************************************************************

		FilterDataFrames(_driftFrames, _ignoreCrcErrors, l_recombinedDriftFrames);

		//********************************************************************
		// Update the status
		//********************************************************************

		l_recombinedFrames += l_recombinedDriftFrames;
		l_distinctDriftFrames = _driftFrames.entries();
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Recombined", l_recombinedFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Recom", l_recombinedDriftFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Distinc", l_distinctDriftFrames));
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1657", 226);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : FilterAscentFrames
// ROL : Filter the list of ascent frames in order to remove doubles
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
FilterAscentFrames(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_recombinedFrames = 0;				// Number of recombined frames
	int l_recombinedAscentFrames = 0;		// Number of recombined ascent frames
	int l_distinctAscentFrames = 0;		// Number of distinct ascent frames

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Initialize the counts
		//********************************************************************

		if (_technicalStatus.HasAttribute("TRANS_Recombined") == TRUE)
		{
			l_recombinedFrames = _technicalStatus.GetAttribute("TRANS_Recombined").GetIntValue();
		}

		//********************************************************************
		// Call the generic filter method
		//********************************************************************

		FilterDataFrames(_ascentFrames, _ignoreCrcErrors, l_recombinedAscentFrames);

		//********************************************************************
		// Update the status
		//********************************************************************

		l_recombinedFrames += l_recombinedAscentFrames;
		l_distinctAscentFrames = _ascentFrames.entries();
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_Recombined", l_recombinedFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Recom", l_recombinedAscentFrames));
		_technicalStatus.AddAttribute(CoDatAttribute("TRANS_APROF_Distinc", l_distinctAscentFrames));
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1722", 227);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : FilterDataFrames
// ROL : Filter a list of frames in order to remove doubles
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
FilterDataFrames(
	RWTPtrSlist<CoDatProvorFrame>& p_list,	// I : the list to filter
	RWBoolean p_ignoreCrcErrors,			// I : ignore the crc errors ?
	int& p_recombinedFrames					// O : number of recombined frames
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWTPtrSlist<CoDatProvorFrame> l_temp;	// temporary list
	RWTPtrSlist<CoDatProvorFrame> l_final;	// the filtered list
	int l_counter = 0;						// loop counter

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Sort the frame list
		//********************************************************************

		CoStcTools_Sort(p_list, CoDatProvorFrame::CompareId);

		//********************************************************************
		// while the frame list is not empty
		//********************************************************************

		p_recombinedFrames = 0;
		while (p_list.isEmpty() == FALSE)
		{
			//****************************************************************
			// Copy in the temp list all the frames which have the same id
			//****************************************************************

			l_temp.append(p_list.removeFirst());
			while ((p_list.isEmpty() == FALSE) &&
				   (p_list.first()->GetId() == l_temp.first()->GetId()))
			{
				l_temp.append(p_list.removeFirst());
			}

			//****************************************************************
			// Find the first frame which has a good Crc (if exists)
			//****************************************************************

			l_counter = 0;
			while ((l_counter < l_temp.entries()) &&
				   (l_temp.at(l_counter)->IsCrcOk() == FALSE))
			{
				l_counter++;
			}

			//****************************************************************
			// if a frame with a good Crc has been found
			//****************************************************************

			if (l_counter < l_temp.entries())
			{
				//************************************************************
				// Append this frame to the filtered list and destroy
				// other frames
				//************************************************************

				l_final.append(l_temp.removeAt(l_counter));
				l_temp.clearAndDestroy();
			}

			//****************************************************************
			// If no frame with a good Crc has been found
			//****************************************************************

			else
			{
				//************************************************************
				// If the list has at least three frames, try to reconstruct
				// a new one
				//************************************************************

				if (l_temp.entries() >= 3)
				{
					l_temp.first()->Recombined(l_temp);
					if (_platform.GetAttribute("PR_CRC_TYPE").GetIntValue() == 8)
					{
						l_temp.first()->Check8BitsCrc();
					}
					else
					{
						l_temp.first()->Check16BitsCrc();
					}
					if (l_temp.first()->IsCrcOk() == TRUE)
					{
						p_recombinedFrames++;
						l_final.append(l_temp.removeFirst());
					}
					else
					{
						if (p_ignoreCrcErrors == TRUE)
						{
							l_final.append(l_temp.removeFirst());
						}
					}
				}
				else
				{
					if (p_ignoreCrcErrors == TRUE)
					{
						l_final.append(l_temp.removeFirst());
					}
				}
				l_temp.clearAndDestroy();
			} 
		}

		//********************************************************************
		// Update the input list
		//********************************************************************

		while (l_final.isEmpty() == FALSE)
		{
			p_list.append(l_final.removeFirst());
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1870", 228);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : CheckEndOfLife
// ROL : Check for the end of life of the float
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
CheckEndOfLife(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// If the three profiles are empty
		//********************************************************************

		if ((_descentFrames.entries() == 0) &&
			(_driftFrames.entries() == 0) &&
			(_ascentFrames.entries() == 0))
		{
			//****************************************************************
			// This is the float end of life, there's no cycle number
			//****************************************************************

			_technicalStatus.RemoveAttribute("CYCLE_NUMBER");
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 1921", 229);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : FillDescentProfile
// ROL : Fill the descent profile
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
FillDescentProfile(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_cycleNumber = 0;					// the cycle number
	CoStcFileConf l_conf;					// access to the configuration file
	int l_maxLocateClass = 0;					// the max acceptable location QC
	int l_maxLocateSpan = 0;				// the max acceptable locate span
	int l_EVdelay = 0;						// the EV delay
	RWTime l_profileTime =					// time of the profile
		(unsigned long)0;
	char l_profileTimeQuality = '0';		// Quality on profile date/time
	CoDatProfile l_previousTrajectory;		// the previous surface drift
	RWTime l_lastSurfaceTime =				// The last surface time
		(unsigned long)0;
	CoDatPosition l_position;				// position of the profile
	int l_frameCount = 0;					// The number of frames
	int l_frameCounter = 0;					// loop counter
	CoDatProvorFrame* ql_frame =			// current frame
		(CoDatProvorFrame*)0;
	RWTPtrSlist<CoDatRecord> l_records;		// temporary list of records
	int l_recordCount = 0;					// Number of records
	int l_recordCounter = 0;				// Loop counter
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	CoDatMeasure l_measure;					// the current measure
	double l_pressureValue = 0.0;			// the previous pressure value
	RWTValSlist<double> l_pressures;		// the table of pressure values
	int l_pressureIndex = 0;				// index over the table of pressure values

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve the cycle number
		//********************************************************************

		l_cycleNumber = _technicalStatus.GetAttribute("CYCLE_NUMBER").GetIntValue();

		//********************************************************************
		// Retrieve the locate qc and the locate span from the conf file
		//********************************************************************

		l_maxLocateClass = atoi(l_conf.GetBlockValue("PROFILER_CYCLE_DECODER",
													 "MAX_LOCATE_QC"));
		l_maxLocateSpan = atoi(l_conf.GetBlockValue("PROFILER_CYCLE_DECODER",
													"MAX_LOCATE_SPAN"));

		//********************************************************************
		// Compute the EV delay
		//********************************************************************

		if ((_version == 2.2) || (_version == 3.21))
		{
			l_EVdelay = 130;
		}
		else
		{
			l_EVdelay = 0;
		}

		//********************************************************************
		// Compute the profile date/time : 
		// Cycle 0 : the launch date + the descent start time
		// Cycle n : the last surface date + the descent start time
		//********************************************************************

		l_profileTime = _technicalStatus.GetAttribute("DESFC_Start_time").GetDateTimeValue() +
						_technicalStatus.GetAttribute("DESFC_Surface_valve").GetIntValue() * l_EVdelay;
		l_profileTimeQuality = '1';
		if (l_cycleNumber == 0)
		{
			l_profileTime = RWTime(RWDate(_platform.GetAttribute("PR_LAUNCH_DATETIME").GetDateTimeValue()),
								   l_profileTime.hour(),
								   l_profileTime.minute(),
								   l_profileTime.second());
			if (l_profileTime < _platform.GetAttribute("PR_LAUNCH_DATETIME").GetDateTimeValue())
			{
				l_profileTime = l_profileTime + 86400;
			}
		}
		else
		{
			_q_trajectoryAccessor->SelectByPlatformCodeAndCvNumber(_platform.GetCode(),
																   l_cycleNumber - 1,
																   l_previousTrajectory);
			l_previousTrajectory.SortRecordsByDateTimeInv();
			if (l_previousTrajectory.GetRecordCount() > 0)
			{
				l_lastSurfaceTime = l_previousTrajectory.GetRecord(0)->GetDateTime();
				l_profileTimeQuality = '1';
			}
			else
			{
				l_lastSurfaceTime = _platform.GetAttribute("PR_REFERENCE_DATETIME").GetDateTimeValue() +
					86400 * _platform.GetAttribute("PR_CYCLE_PERIOD_1").GetIntValue() * int(l_cycleNumber / 2) +
					86400 * _platform.GetAttribute("PR_CYCLE_PERIOD_2").GetIntValue() * int((l_cycleNumber - 1) / 2) -
					_timeCorrection;
				l_profileTimeQuality = '3';
			}
			l_profileTime = RWTime(RWDate(l_lastSurfaceTime),
								   l_profileTime.hour(),
								   l_profileTime.minute(),
								   l_profileTime.second());
			if (l_profileTime < l_lastSurfaceTime)
			{
				l_profileTime = l_profileTime + 86400;
			}
		}
		_descentProfile.SetDateTime(l_profileTime);
		_descentProfile.SetDateTimeQuality(l_profileTimeQuality);

		//********************************************************************
		// Compute the profile position
		// Cycle 0 : the launch position
		// Cycle n : the last valid surface drift position
		//********************************************************************

		if (l_cycleNumber == 0)
		{
			l_position.SetLatitude(_platform.GetAttribute("PR_LAUNCH_LATITUDE").GetDoubleValue());
			l_position.SetLongitude(_platform.GetAttribute("PR_LAUNCH_LONGITUDE").GetDoubleValue());
			l_position.SetQuality('1');
		}
		else
		{
			ComputeProfilePosition(l_previousTrajectory,
								   l_profileTime,
								   l_maxLocateClass,
								   l_maxLocateSpan,
								   l_position);
		}
		_descentProfile.SetPosition(l_position);

		//********************************************************************
		// Compute the table of pressures values
		//********************************************************************

		if (_version == 4.0)
		{
			ComputePressureTable(_version,
								 _technicalStatus.GetAttribute("PRCFG_Profile_option").GetIntValue(),
								 'D', l_pressures);
		}

		//********************************************************************
		// For each descent frame
		//********************************************************************

		l_frameCount = _descentFrames.entries();
		for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
		{
			//****************************************************************
			// Get the current frame
			//****************************************************************

			ql_frame = _descentFrames.at(l_frameCounter);

			//****************************************************************
			// Get the records
			//****************************************************************

			ql_frame->GetRecords(l_profileTime, l_records);

			//****************************************************************
			// If needed, compute the pressure index
			//****************************************************************

			if (_version == 4.0)
			{
				l_pressureIndex = (ql_frame->GetFrameNumber() - 1) * 10;
				if (ql_frame->GetJoker() == 2)
				{
					l_pressureIndex += 5;
				}
			}

			//****************************************************************
			// For each record of the frame
			//****************************************************************

			while (l_records.entries() != 0)
			{
				//************************************************************
				// Retrieve the record
				//************************************************************

				ql_record = l_records.removeFirst();

				//************************************************************
				// Eventually find it's pressure
				//************************************************************

				if (_version == 4.0)
				{
					ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure,
													   l_pressures.at(l_pressureIndex)));
					l_pressureIndex++;
				}

				//************************************************************
				// COmpute the other parameters (PSAL, DENS, ...)
				//************************************************************

				if ((_version == 3.21) || (_version == 3.5) || (_version == 3.61) ||
					(_version == 3.8) || (_version == 3.81))
				{
					ql_record->ComputeAndAddPsalMeasure();
					ql_record->ComputeAndAddDensMeasure();
				}
				else if ((_version == 3.51) || (_version == 3.52) || (_version == 4.0) ||
						 (_version == 4.1) || (_version == 4.11))
				{
					ql_record->ComputeAndAddDensMeasure();
				}

				//************************************************************
				// And append the record to the profile
				//************************************************************

				_descentProfile.AddRecord(ql_record);
			}
		}

		//********************************************************************
		// Readjust the pressures for the 2048 dbar modulo (only for non
		// interlaced versions
		//********************************************************************

		if ((_version == 2.2) || (_version == 3.21))
		{
			l_recordCount = _descentProfile.GetRecordCount();
			if (l_recordCount > 0)
			{
				l_pressureValue = _descentProfile.GetRecord(0)->GetMeasure(CoDat_k_ppPressure).GetValue();
			}
			for (l_recordCounter = 1; l_recordCounter < l_recordCount; l_recordCounter++)
			{
				ql_record = _descentProfile.GetRecord(l_recordCounter);
				l_measure = ql_record->GetMeasure(CoDat_k_ppPressure);
				if ((l_measure.GetValue() < l_pressureValue - 1024) &&
					(ql_record->GetDateTime() == CoDat_k_undefinedDateTime))
				{
					l_measure.SetValue(l_measure.GetValue() + 2048);
					ql_record->AddMeasure(l_measure);
				}
				l_pressureValue = l_measure.GetValue();
			}
		}

		//********************************************************************
		// Sort the profile
		//********************************************************************

		_descentProfile.SortRecords(CoDat_k_ppPressure);

		//********************************************************************
		// Readjust the records date-time for the 512 minutes modulo
		//********************************************************************

		l_recordCount = _descentProfile.GetRecordCount();
		if (l_recordCount > 0)
		{
			l_profileTime = _descentProfile.GetRecord(0)->GetDateTime();
		}
		for (l_recordCounter = 1; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			ql_record = _descentProfile.GetRecord(l_recordCounter);
			if (ql_record->GetDateTime() != CoDat_k_undefinedDateTime)
			{
				if (ql_record->GetDateTime() < l_profileTime)
				{
					ql_record->SetDateTime(ql_record->GetDateTime() + 512 * 60);
				}
				l_profileTime = ql_record->GetDateTime();
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 2228", 230);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : FillImmersionDrift
// ROL : Fill the immersion drift
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
FillImmersionDrift(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_immersionDriftPeriod = 0;			// the immersion drift period (in minutes)
	RWTime l_profileTime =					// time of the profile
		(unsigned long)0;
	int l_frameCount = 0;					// The number of frames
	int l_frameCounter = 0;					// loop counter
	CoDatProvorFrame* ql_frame =			// current frame
		(CoDatProvorFrame*)0;
	int l_minDay = 0;						// The minimum date
	int l_maxDay = 0;						// The maximum date
	int l_minModuloDay = 0;					// The minimum date when there is a modulo
	int l_frameDay = 0;						// The day of the current frame
	RWTPtrSlist<CoDatRecord> l_records;		// temporary list of records
	int l_recordCount = 0;					// The number of records
	int l_recordCounter = 0;				// loop counter
	CoDatRecord* ql_record =				// current record
		(CoDatRecord*)0;
	RWTime l_recordDateTime =				// date/time of the current record
		(unsigned long)0;
	RWTime l_firstRecordDateTime =			// the first record date-time
		(unsigned long)0;
	int l_emittedRecordCount = 0;			// number of emitted records
	RWTime l_lastRecordDateTime =			// the last record date-time
		(unsigned long)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the immersion drift period
		//********************************************************************

		l_immersionDriftPeriod = _platform.GetAttribute("PR_IMMERSION_DRIFT_PERIOD").GetIntValue();

		//********************************************************************
		// Compute the profile date-time : this is the descent end-time
		//********************************************************************

		l_profileTime = _technicalStatus.GetAttribute("DESFC_End_time").GetDateTimeValue();
		l_profileTime = RWTime(RWDate(_descentProfile.GetDateTime()),
							   l_profileTime.hour(),
							   l_profileTime.minute(),
							   l_profileTime.second());
		if (l_profileTime < _descentProfile.GetDateTime())
		{
			l_profileTime = l_profileTime + 86400;
		}
		_immersionDrift.SetDateTime(l_profileTime);
		_immersionDrift.SetDateTimeQuality(_descentProfile.GetDateTimeQuality());

		//********************************************************************
		// Adjust the frame date-time : the first frame must start at day 0
		// and there should be no modulo
		//********************************************************************

		l_frameCount = _driftFrames.entries();
		l_minDay = 64;
		l_minModuloDay = 64;
		l_maxDay = -1;
		for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
		{
			l_frameDay = _driftFrames.at(l_frameCounter)->GetRelativeDating() / 32;
			if (l_frameDay > l_maxDay)
			{
				l_maxDay = l_frameDay;
			}
			if (l_frameDay < l_minDay)
			{
				l_minDay = l_frameDay;
			}
			if ((l_frameDay > 32) && (l_frameDay < l_minModuloDay))
			{
				l_minModuloDay = l_frameDay;
			}
		}
		if (l_maxDay - l_minDay > 32)
		{
			l_minDay = l_minModuloDay;
		}
		for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
		{
			ql_frame = _driftFrames.at(l_frameCounter);
			l_frameDay = ql_frame->GetRelativeDating() / 32;
			if (l_frameDay >= l_minDay)
			{
				ql_frame->SetRelativeDating(ql_frame->GetRelativeDating() - 32 * l_minDay);
			}
			else
			{
				ql_frame->SetRelativeDating(ql_frame->GetRelativeDating() + 32 * (64 - l_minDay));
			}
		}

		//********************************************************************
		// Adjust the date-time for the first record. The first record must
		// be subsequent to the profile time
		//********************************************************************

		CoStcTools_Sort(_driftFrames, CoDatProvorFrame::CompareRelativeDating);
		if (_driftFrames.entries() > 0)
		{
			ql_frame = _driftFrames.first();
			l_profileTime = RWTime(RWDate(l_profileTime),
								   ql_frame->GetRelativeDating() % 32, 0, 0);
			if (l_profileTime - _timeCorrection < _immersionDrift.GetDateTime())
			{
				l_profileTime = l_profileTime + 86400;
			}
		}

		//********************************************************************
		// For each immersion drift frame
		//********************************************************************

		l_frameCount = _driftFrames.entries();
		for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
		{
			//****************************************************************
			// Get the current frame
			//****************************************************************

			ql_frame = _driftFrames.at(l_frameCounter);

			//****************************************************************
			// Get the records
			//****************************************************************

			ql_frame->GetRecords(l_profileTime, l_records);

			//****************************************************************
			// Append each record to the profile
			//****************************************************************

			while (l_records.entries() != 0)
			{
				ql_record = l_records.removeFirst();
				if ((_version == 3.21) || (_version == 3.5) || (_version == 3.61) ||
					(_version == 3.8) || (_version == 3.81))
				{
					ql_record->ComputeAndAddPsalMeasure();
					ql_record->ComputeAndAddDensMeasure();
				}
				else if ((_version == 3.51) || (_version == 3.52) || (_version == 4.0) ||
						 (_version == 4.1) || (_version == 4.11))
				{
					ql_record->ComputeAndAddDensMeasure();
				}
				_immersionDrift.AddRecord(ql_record);
			}
		}

		//********************************************************************
		// If the record are not interlaced
		//********************************************************************

		if ((_version == 2.2) || (_version == 3.21) || (_version == 4.0))
		{
			//****************************************************************
			// For each record
			//****************************************************************

			l_recordCount = _immersionDrift.GetRecordCount();
			for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
			{
				//************************************************************
				// Retrieve the current record
				//************************************************************

				ql_record = _immersionDrift.GetRecord(l_recordCounter);

				//************************************************************
				// If the record date-time is defined, correct it and memorize it
				//************************************************************

				if (ql_record->GetDateTime().isValid() == TRUE)
				{
					l_recordDateTime = ql_record->GetDateTime() - _timeCorrection;
					ql_record->SetDateTime(l_recordDateTime);
				}

				//************************************************************
				// If the record date-time is not defined; compute it
				//************************************************************

				else
				{
					l_recordDateTime = l_recordDateTime + 60 * l_immersionDriftPeriod;
					ql_record->SetDateTime(l_recordDateTime);
				}
			}
		}

		//********************************************************************
		// If the records are interlaced
		//********************************************************************

		else if ((_version == 2.6) || (_version == 2.7) || (_version == 3.3) ||
				 (_version == 3.5) || (_version == 3.51) || (_version == 3.52) ||
				 (_version == 3.61) || (_version == 3.8) || (_version == 3.81) ||
				 (_version == 4.1) || (_version == 4.11))
		{
			//****************************************************************
			// Compute the first record date-time and the last record date-time
			//****************************************************************

			l_recordCount = _immersionDrift.GetRecordCount();
			if (l_recordCount != 0)
			{
				l_firstRecordDateTime = _immersionDrift.GetRecord(0)->GetDateTime() - _timeCorrection;
				l_emittedRecordCount = _technicalStatus.GetAttribute("ITEMP_Internal_count").GetIntValue();
				l_lastRecordDateTime = l_firstRecordDateTime +
									   (l_emittedRecordCount - 1) * 60 * l_immersionDriftPeriod;
			}

			//****************************************************************
			// For each record
			//****************************************************************

			for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
			{
				//************************************************************
				// Retrieve the current record
				//************************************************************

				ql_record = _immersionDrift.GetRecord(l_recordCounter);

				//************************************************************
				// If the record date-time is defined, correct it and memorize it
				//************************************************************

				if (ql_record->GetDateTime().isValid() == TRUE)
				{
					l_recordDateTime = ql_record->GetDateTime() - _timeCorrection;
					ql_record->SetDateTime(l_recordDateTime);
				}

				//************************************************************
				// If the record date-time is not defined; compute it
				//************************************************************

				else
				{
					l_recordDateTime = l_recordDateTime + 2 * 60 * l_immersionDriftPeriod;
					if (l_recordDateTime > l_lastRecordDateTime)
					{
						l_recordDateTime = l_firstRecordDateTime + 60 * l_immersionDriftPeriod;
					}
					ql_record->SetDateTime(l_recordDateTime);
				}
			}
		}

		//********************************************************************
		// Sort the profile
		//********************************************************************

		_immersionDrift.SortRecordsByDateTime();
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 2518", 231);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : FillAscentProfile
// ROL : Fill the ascent profile
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
FillAscentProfile(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_cycleNumber = 0;					// the cycle number
	CoStcFileConf l_conf;					// access to the configuration file
	char l_maxLocateClass = 0;				// the max acceptable location class
	int l_maxLocateSpan = 0;				// the max acceptable locate span
	RWTime l_profileTime =					// time of the profile
		(unsigned long)0;
	CoDatPosition l_position;				// position of the profile
	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	int l_frameCount = 0;					// The number of frames
	int l_frameCounter = 0;					// loop counter
	CoDatProvorFrame* ql_frame =			// current frame
		(CoDatProvorFrame*)0;
	RWTPtrSlist<CoDatRecord> l_records;		// temporary list of records
	RWTValSlist<double> l_pressures;		// the table of pressure values
	int l_pressureIndex = 0;				// index over the table of pressure values

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve the cycle number
		//********************************************************************

		l_cycleNumber = _technicalStatus.GetAttribute("CYCLE_NUMBER").GetIntValue();

		//********************************************************************
		// Retrieve the locate qc and the locate span from the conf file
		//********************************************************************

		l_maxLocateClass = char(atoi(l_conf.GetBlockValue("PROFILER_CYCLE_DECODER",
														  "MAX_LOCATE_QC"))) + '0';
		l_maxLocateSpan = atoi(l_conf.GetBlockValue("PROFILER_CYCLE_DECODER",
													"MAX_LOCATE_SPAN"));

		//********************************************************************
		// Compute the profile time
		//********************************************************************

		l_profileTime = _technicalStatus.GetAttribute("ASCFC_End_time").GetDateTimeValue();
		l_profileTime = RWTime(RWDate(_surfaceDrift.GetRecord(0)->GetDateTime()),
							   l_profileTime.hour(),
							   l_profileTime.minute(),
							   l_profileTime.second());
		if (l_profileTime > _surfaceDrift.GetRecord(0)->GetDateTime())
		{
			l_profileTime = l_profileTime - 86400;
		}
		_ascentProfile.SetDateTime(l_profileTime);
		_ascentProfile.SetDateTimeQuality('1');

		//********************************************************************
		// Compute the profile position
		//********************************************************************

		ComputeProfilePosition(_surfaceDrift,
							   l_profileTime,
							   l_maxLocateClass,
							   l_maxLocateSpan,
							   l_position);
		_ascentProfile.SetPosition(l_position);

		//********************************************************************
		// Compute the start date-time of the ascending profile :
		// - If ASCFC_Start_time is not defined (version prior to 3.61), the
		//   start date-time is calculated with the database informations 
		// - If ASCFC_Start_time is defined, use it
		//********************************************************************

		if (_technicalStatus.HasAttribute("ASCFC_Start_time") == FALSE)
		{
			l_profileTime = _platform.GetAttribute("PR_REFERENCE_DATETIME").GetDateTimeValue() +
				86400 * _platform.GetAttribute("PR_CYCLE_PERIOD_1").GetIntValue() * int((l_cycleNumber + 1) / 2) +
				86400 * _platform.GetAttribute("PR_CYCLE_PERIOD_2").GetIntValue() * int(l_cycleNumber / 2) -
				_timeCorrection;
		}
		else
		{
			l_profileTime = _technicalStatus.GetAttribute("ASCFC_Start_time").GetDateTimeValue();
			l_profileTime = RWTime(RWDate(_ascentProfile.GetDateTime()),
								   l_profileTime.hour(),
								   l_profileTime.minute(),
								   l_profileTime.second());
			if (l_profileTime > _ascentProfile.GetDateTime())
			{
				l_profileTime = l_profileTime - 86400;
			}
		}

		//********************************************************************
		// Compute the table of pressures values
		//********************************************************************

		if (_version == 4.0)
		{
			ComputePressureTable(_version,
								 _technicalStatus.GetAttribute("PRCFG_Profile_option").GetIntValue(),
								 'A', l_pressures);
		}

		//********************************************************************
		// For each ascent frame
		//********************************************************************

		l_frameCount = _ascentFrames.entries();
		for (l_frameCounter = 0; l_frameCounter < l_frameCount; l_frameCounter++)
		{
			//****************************************************************
			// Get the current frame
			//****************************************************************

			ql_frame = _ascentFrames.at(l_frameCounter);

			//****************************************************************
			// Get the records
			//****************************************************************

			ql_frame->GetRecords(l_profileTime, l_records);

			//****************************************************************
			// If needed, compute the pressure index
			//****************************************************************

			if (_version == 4.0)
			{
				l_pressureIndex = (ql_frame->GetFrameNumber() - 16) * 10;
				if (ql_frame->GetJoker() == 2)
				{
					l_pressureIndex += 5;
				}
			}

			//****************************************************************
			// For each record of the frame
			//****************************************************************

			while (l_records.entries() != 0)
			{
				//************************************************************
				// Retrieve the record
				//************************************************************

				ql_record = l_records.removeFirst();

				//************************************************************
				// Eventually find it's pressure
				//************************************************************

				if (_version == 4.0)
				{
					ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure,
													   l_pressures.at(l_pressureIndex)));
					l_pressureIndex++;
				}

				//************************************************************
				// COmpute the other parameters (PSAL, DENS, ...)
				//************************************************************

				if ((_version == 3.21) || (_version == 3.5) || (_version == 3.61) ||
					(_version == 3.8) || (_version == 3.81))
				{
					ql_record->ComputeAndAddPsalMeasure();
					ql_record->ComputeAndAddDensMeasure();
				}
				else if ((_version == 3.51) || (_version == 3.52) || (_version == 4.0) ||
						 (_version == 4.1) || (_version == 4.11))
				{
					ql_record->ComputeAndAddDensMeasure();
				}

				//************************************************************
				// And append the record to the profile
				//************************************************************

				_ascentProfile.AddRecord(ql_record);
			}
		}

		//********************************************************************
		// Sort the profile
		//********************************************************************

		_ascentProfile.SortRecordsInv(CoDat_k_ppPressure);
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 2738", 232);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : ComputeParametersMeasurementsInformations
// ROL : Compute for each profile the parameters measurements informations
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
ComputeParametersMeasurementsInformations(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_prsep = 0.0;					// PRSEP
	int l_surfaceRelative = 0;				// number of surface relative measures
	int l_surfaceAbsolute = 0;				// number of surface absolute measures
	int l_depthRelative = 0;				// number of depth relative measures
	int l_depthAbsolute = 0;				// number of depth absolute measures

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Compute and update the temperature measurements informations
		//********************************************************************

		l_prsep = _technicalStatus.GetAttribute("DESPR_PRSEP").GetDoubleValue();
		ComputeMeasureStatistics(_descentProfile, CoDat_k_ppTemperature, l_prsep,
								 l_surfaceRelative, l_surfaceAbsolute,
								 l_depthRelative, l_depthAbsolute);
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Surf_relative", l_surfaceRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Surf_absolute", l_surfaceAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Surf_total",
													 l_surfaceRelative + l_surfaceAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Depth_relative", l_depthRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Depth_absolute", l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Depth_total",
													 l_depthRelative + l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Profile_total",
													 l_surfaceRelative + l_surfaceAbsolute +
													 l_depthRelative + l_depthAbsolute));

		//********************************************************************
		// Compute and update the conductivity measurements informations
		//********************************************************************

		if ((_version == 3.21) || (_version == 3.3) || (_version == 3.5) ||
			(_version == 3.61) || (_version == 3.8) || (_version == 3.81))
		{
			ComputeMeasureStatistics(_descentProfile, CoDat_k_ppConductivity, l_prsep,
									 l_surfaceRelative, l_surfaceAbsolute,
									 l_depthRelative, l_depthAbsolute);
		}
		else if ((_version == 3.51) || (_version == 3.52) || (_version == 4.1) ||
				 (_version == 4.11))
		{
			ComputeMeasureStatistics(_descentProfile, CoDat_k_ppSalinity, l_prsep,
									 l_surfaceRelative, l_surfaceAbsolute,
									 l_depthRelative, l_depthAbsolute);
		}
		_technicalStatus.AddAttribute(CoDatAttribute("DCOND_Surf_relative", l_surfaceRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("DCOND_Surf_absolute", l_surfaceAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DCOND_Surf_total",
													 l_surfaceRelative + l_surfaceAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DCOND_Depth_relative", l_depthRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("DCOND_Depth_absolute", l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DCOND_Depth_total",
													 l_depthRelative + l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DCOND_Profile_total",
													 l_surfaceRelative + l_surfaceAbsolute +
													 l_depthRelative + l_depthAbsolute));

		//********************************************************************
		// Compute and update the temperature measurements informations
		//********************************************************************

		ComputeMeasureStatistics(_immersionDrift, CoDat_k_ppTemperature, 0.0,
								 l_depthRelative, l_depthAbsolute,
								 l_depthRelative, l_depthAbsolute);
		_technicalStatus.AddAttribute(CoDatAttribute("ITEMP_Drift_relative", l_depthRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("ITEMP_Drift_absolute", l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ITEMP_Drift_total",
													 l_depthRelative + l_depthAbsolute));

		//********************************************************************
		// Compute and update the conductivity measurements informations
		//********************************************************************

		if ((_version == 3.21) || (_version == 3.3) || (_version == 3.5) ||
			(_version == 3.61) || (_version == 3.8) || (_version == 3.81))
		{
			ComputeMeasureStatistics(_immersionDrift, CoDat_k_ppConductivity, 0.0,
									 l_depthRelative, l_depthAbsolute,
									 l_depthRelative, l_depthAbsolute);
		}
		else if ((_version == 3.51) || (_version == 3.52) || (_version == 4.1) ||
				 (_version == 4.11))
		{
			ComputeMeasureStatistics(_immersionDrift, CoDat_k_ppSalinity, 0.0,
									 l_depthRelative, l_depthAbsolute,
									 l_depthRelative, l_depthAbsolute);
		}
		_technicalStatus.AddAttribute(CoDatAttribute("ICOND_Drift_relative", l_depthRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("ICOND_Drift_absolute", l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ICOND_Drift_total",
													 l_depthRelative + l_depthAbsolute));

		//********************************************************************
		// Compute and update the temperature measurements informations
		//********************************************************************

		l_prsep = _technicalStatus.GetAttribute("ASCPR_PRSEP").GetDoubleValue();
		ComputeMeasureStatistics(_ascentProfile, CoDat_k_ppTemperature, l_prsep,
								 l_surfaceRelative, l_surfaceAbsolute,
								 l_depthRelative, l_depthAbsolute);
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Surf_relative", l_surfaceRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Surf_absolute", l_surfaceAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Surf_total",
													 l_surfaceRelative + l_surfaceAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Depth_relative", l_depthRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Depth_absolute", l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Depth_total",
													 l_depthRelative + l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Profile_total",
													 l_surfaceRelative + l_surfaceAbsolute +
													 l_depthRelative + l_depthAbsolute));

		//********************************************************************
		// Compute and update the conductivity measurements informations
		//********************************************************************

		if ((_version == 3.21) || (_version == 3.3) || (_version == 3.5) ||
			(_version == 3.61) || (_version == 3.8) || (_version == 3.81))
		{
			ComputeMeasureStatistics(_ascentProfile, CoDat_k_ppConductivity, l_prsep,
									 l_surfaceRelative, l_surfaceAbsolute,
									 l_depthRelative, l_depthAbsolute);
		}
		else if ((_version == 3.51) || (_version == 3.52) || (_version == 4.1) ||
				 (_version == 4.11))
		{
			ComputeMeasureStatistics(_ascentProfile, CoDat_k_ppSalinity, l_prsep,
									 l_surfaceRelative, l_surfaceAbsolute,
									 l_depthRelative, l_depthAbsolute);
		}
		_technicalStatus.AddAttribute(CoDatAttribute("ACOND_Surf_relative", l_surfaceRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("ACOND_Surf_absolute", l_surfaceAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ACOND_Surf_total",
													 l_surfaceRelative + l_surfaceAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ACOND_Depth_relative", l_depthRelative));
		_technicalStatus.AddAttribute(CoDatAttribute("ACOND_Depth_absolute", l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ACOND_Depth_total",
													 l_depthRelative + l_depthAbsolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ACOND_Profile_total",
													 l_surfaceRelative + l_surfaceAbsolute +
													 l_depthRelative + l_depthAbsolute));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 2915", 233);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : ComputeParametersMeasurementsInformationsWithoutPrsep
// ROL : Compute for each profile the parameters measurements informations
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
ComputeParametersMeasurementsInformationsWithoutPrsep(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_dummy = 0;						// a dummy value
	int l_relative = 0;						// number of relative measures
	int l_absolute = 0;						// number of absolute measures

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Compute and update the temperature measurements informations
		//********************************************************************

		ComputeMeasureStatistics(_descentProfile, CoDat_k_ppTemperature, -1.0,
								 l_dummy, l_dummy, l_relative, l_absolute);
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Profile_rel", l_relative));
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Profile_abs", l_absolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DTEMP_Profile_total",
													 l_relative + l_absolute));

		//********************************************************************
		// Compute and update the salinity measurements informations
		//********************************************************************

		ComputeMeasureStatistics(_descentProfile, CoDat_k_ppSalinity, -1.0,
								 l_dummy, l_dummy, l_relative, l_absolute);
		_technicalStatus.AddAttribute(CoDatAttribute("DPSAL_Profile_rel", l_relative));
		_technicalStatus.AddAttribute(CoDatAttribute("DPSAL_Profile_abs", l_absolute));
		_technicalStatus.AddAttribute(CoDatAttribute("DPSAL_Profile_total",
													 l_relative + l_absolute));

		//********************************************************************
		// Compute and update the temperature measurements informations
		//********************************************************************

		ComputeMeasureStatistics(_immersionDrift, CoDat_k_ppTemperature, -1.0,
								 l_dummy, l_dummy, l_relative, l_absolute);
		_technicalStatus.AddAttribute(CoDatAttribute("ITEMP_Drift_relative", l_relative));
		_technicalStatus.AddAttribute(CoDatAttribute("ITEMP_Drift_absolute", l_absolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ITEMP_Drift_total",
													 l_relative + l_absolute));

		//********************************************************************
		// Compute and update the salinity measurements informations
		//********************************************************************

		ComputeMeasureStatistics(_immersionDrift, CoDat_k_ppSalinity, -1.0,
								 l_dummy, l_dummy, l_relative, l_absolute);
		_technicalStatus.AddAttribute(CoDatAttribute("IPSAL_Drift_relative", l_relative));
		_technicalStatus.AddAttribute(CoDatAttribute("IPSAL_Drift_absolute", l_absolute));
		_technicalStatus.AddAttribute(CoDatAttribute("IPSAL_Drift_total",
													 l_relative + l_absolute));

		//********************************************************************
		// Compute and update the temperature measurements informations
		//********************************************************************

		ComputeMeasureStatistics(_ascentProfile, CoDat_k_ppTemperature, -1.0,
								 l_dummy, l_dummy, l_relative, l_absolute);
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Profile_rel", l_relative));
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Profile_abs", l_absolute));
		_technicalStatus.AddAttribute(CoDatAttribute("ATEMP_Profile_total",
													 l_relative + l_absolute));

		//********************************************************************
		// Compute and update the salinity measurements informations
		//********************************************************************

		ComputeMeasureStatistics(_ascentProfile, CoDat_k_ppSalinity, -1.0,
								 l_dummy, l_dummy, l_relative, l_absolute);
		_technicalStatus.AddAttribute(CoDatAttribute("APSAL_Profile_rel", l_relative));
		_technicalStatus.AddAttribute(CoDatAttribute("APSAL_Profile_abs", l_absolute));
		_technicalStatus.AddAttribute(CoDatAttribute("APSAL_Profile_total",
													 l_relative + l_absolute));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 3021", 234);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : WriteDecodedCycleFile
// ROL : Initialize and write the decoded cycle file
//
//****************************************************************************
void
CoAppProvorCycleDecoder::
WriteDecodedCycleFile(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDfaDecodedCycleFile l_file;			// the decoded cycle file
	CoStcFileConf l_fileConf;				// Access to the configuration file
	RWCString l_decoderVersion = "";		// the decoder version

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Set the output file name
		//********************************************************************

		l_file.SetFileName(_outputFileName);

		//********************************************************************
		// Set the decoder version
		//********************************************************************

		l_decoderVersion = l_fileConf.GetBlockValue("PROFILER_CYCLE_DECODER", "DECODER_VERSION");
		l_file.SetDecoderVersion(l_decoderVersion);

		//********************************************************************
		// Write the content of the file
		//********************************************************************

		l_file.Write(_platform,
					 _descentProfile,
					 _immersionDrift,
					 _ascentProfile,
					 _surfaceDrift,
					 _technicalStatus);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 3085", 235);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : ComputeProfilePosition
// ROL : Compute the position of a profile in a trajectory with criteria
//
//****************************************************************************
/*static*/ void
CoAppProvorCycleDecoder::
ComputeProfilePosition(
	const CoDatProfile& p_trajectory,		// I : the trajectory
	const RWTime& p_profileDateTime,		// I : the profile date-time
	char p_maxLocateClass,					// I : the max acceptable location class
	int p_maxLocateSpan,					// I : the max acceptable locate span
	CoDatPosition& p_position				// O : the profile position
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Initialize the return psotion
		//********************************************************************

		p_position = CoDatPosition();
		p_position.SetQuality('9');

		//********************************************************************
		// Compute the profile position, ie find the first valid surface
		// position
		//********************************************************************

		l_recordCount = p_trajectory.GetRecordCount();
		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			ql_record = p_trajectory.GetRecord(l_recordCounter);
			if (ql_record->GetPosition().GetClass() >= p_maxLocateClass)
			{
				p_position = ql_record->GetPosition();
				if (ql_record->GetDateTime() > p_profileDateTime + p_maxLocateSpan * 60)
				{
					p_position.SetQuality('3');
				}
				else
				{
					p_position.SetQuality('1');
				}
				l_recordCounter = l_recordCount;
			}
		}

		//********************************************************************
		// If the position was not found, use the best position available
		//********************************************************************

		if ((p_position.GetLatitude() == CoDat_k_undefinedDouble) &&
			(l_recordCount != 0))
		{
			p_position.SetClass('0');
			for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
			{
				ql_record = p_trajectory.GetRecord(l_recordCounter);
				if (ql_record->GetPosition().GetClass() > p_position.GetClass())
				{
					p_position = ql_record->GetPosition();
				}
			}
			p_position.SetQuality('3');
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 3181", 236);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : ComputeMeasureStatistics
// ROL : Compute the measure statistics for a profile and a parameter
//
//****************************************************************************
/*static*/ void
CoAppProvorCycleDecoder::
ComputeMeasureStatistics(
	const CoDatProfile& p_profile,			// I : the concerned profile
	const RWCString& p_parameter,			// I : the concerned parameter
	double p_prsep,							// I : the prsep value to use
	int& p_surfaceRelative,					// O : number of surface relative measure
	int& p_surfaceAbsolute,					// O : number of surface absolute measure
	int& p_depthRelative,					// O : number of depth relative measure
	int& p_depthAbsolute					// O : number of depth absolute measure
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	CoDatRecord* ql_record =				// current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Reset the parameters
		//********************************************************************

		p_surfaceRelative = 0;
		p_surfaceAbsolute = 0;
		p_depthRelative = 0;
		p_depthAbsolute = 0;

		//********************************************************************
		// For each record
		//********************************************************************

		l_recordCount = p_profile.GetRecordCount();
		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			//****************************************************************
			// Retrieve the current record
			//****************************************************************

			ql_record = p_profile.GetRecord(l_recordCounter);

			//****************************************************************
			// If the record contains the concerned parameter
			//****************************************************************

			if (ql_record->HasMeasure(p_parameter) == TRUE)
			{
				//************************************************************
				// If the current record is in surface
				//************************************************************

				if (ql_record->GetMeasure(CoDat_k_ppPressure).GetValue() < p_prsep)
				{
					if (ql_record->GetMeasure(p_parameter).GetQuality() == '5')
					{
						p_surfaceRelative++;
					}
					else
					{
						p_surfaceAbsolute++;
					}
				}

				//************************************************************
				// If the current record is in depth
				//************************************************************

				else
				{
					if (ql_record->GetMeasure(p_parameter).GetQuality() == '5')
					{
						p_depthRelative++;
					}
					else
					{
						p_depthAbsolute++;
					}
				}
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 3291", 237);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : ComputePressureTable
// ROL : Compute the table containing the pressures used by a provor sbe
//
//****************************************************************************
/*static*/ void
CoAppProvorCycleDecoder::
ComputePressureTable(
	double p_version,						// I : the profiler version
	int p_option,							// I : the profile option (1..18)
	char p_direction,						// I : the profile direction (D or A)
	RWTValSlist<double>& p_pressures		// I : the array of pressures values
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_directionIndex = 0;				// index which depends of the direction
	double l_pressure = 0.0;				// current pressure value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Clear the pressure table
		//********************************************************************

		p_pressures.clear();

		//********************************************************************
		// For provor 4.0
		//********************************************************************

		if (p_version == 4.0)
		{
			//****************************************************************
			// Compute the direction index
			//****************************************************************

			if (p_direction == 'D')
			{
				l_directionIndex = 0;
			}
			else
			{
				l_directionIndex = 1;
			}

			//****************************************************************
			// surface
			//****************************************************************

			while (l_pressure < CoApp_k_profileOptions_4_0[p_option - 1][l_directionIndex][1])
			{
				l_pressure += CoApp_k_profileOptions_4_0[p_option - 1][l_directionIndex][0];
				p_pressures.append(l_pressure);
			}

			//****************************************************************
			// middle
			//****************************************************************

			while (l_pressure < CoApp_k_profileOptions_4_0[p_option - 1][l_directionIndex][3])
			{
				l_pressure += CoApp_k_profileOptions_4_0[p_option - 1][l_directionIndex][2];
				p_pressures.append(l_pressure);
			}

			//****************************************************************
			// depth
			//****************************************************************

			while (l_pressure < CoApp_k_profileOptions_4_0[p_option - 1][l_directionIndex][5])
			{
				l_pressure += CoApp_k_profileOptions_4_0[p_option - 1][l_directionIndex][4];
				p_pressures.append(l_pressure);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(1, "CoAppProvorCycleDecoder.cpp 1.38 3390", 238);
		CO_RETHROW;
	}
}










