//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoDatApexFrame 
// ROL : This class is the base class for all the classes witch implements
//       a Apex frame. A Apex frame is initialized with the informations
//       contained in the body of an Argos message. The structure of a
//       Apex frame is :
//       [0-7] : The crc value
//       [8-15] : The frame number
//       [16-255] : The frame datas (depending of the frame type)
//
// CRE : 09/05/2001
// AUT : ST
// VER : @(#)1.11
//       @(#)03/12/22
//
//****************************************************************************
// 
// HST : 09/05/2001 ST  Creation
//		 07/09/2001 ST  FAE 127 : Use the density
//		 11/02/2002 ST  Do not use CoStcTPtrSlist any more
//		 09/04/2002 ST  Manage versions of Apex profilers
//		 12/08/2002 ST  Now use CoDatRecord
//		 27/08/2002 ST  Emitting date/time and satellite are managed by DataFrame
//		 11/10/2002 ST  Coriolis V2 compatibility
//		 08/10/2003 ST  Manage Apex V11.0
//		 13/10/2003 ST  Some Apex V11 transmit more than 16 messages
//		 22/12/2003 ST  Most of the work now managed by CoDatApexSuperFrame
//
//****************************************************************************
//
// Copyright, IFREMER, 1999
//
//****************************************************************************
static const char CoDatApexFrame_IDENT[] = "@(#) CORIOLIS : CoDatApexFrame.cpp 1.11 03/12/22";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoDatApexFrame_IN_
#include <CoDatApexFrame.h>
#undef  _CoDatApexFrame_IN_
#include <CoDatAttribute.h>
#include <CoDatMeasure.h>
#include <CoDatPhysicalParameterCodes.h>
#include <CoDatRecord.h>
#include <CoDatTechnicalStatus.h>
#include <CoStcCatManager.h>
#include <CoStcError.h>
#include <CoStcTools.h>
#include <iomanip.h>
#include <stdlib.h>



//----------------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------------
const int CoDat_k_20bitsArgosIdFrameLength = 256;
const int CoDat_k_28bitsArgosIdFrameLength = 248;




//****************************************************************************
//
// SPG : CoDatApexFrame
// ROL : Constructor
//
//****************************************************************************
CoDatApexFrame::
CoDatApexFrame(
    )
    : CoDatDataFrame()
    , _bitVec()
	, _version(0.0)
    , _isCrcOk(FALSE)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : ~CoDatApexFrame
// ROL : Destructor
//
//****************************************************************************
CoDatApexFrame::
~CoDatApexFrame(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : Initialize
// ROL : Initialize the instance with a 64 bytes string containing the
//		 ASCII hexa values of the 256 bits frame
//
//****************************************************************************
void
CoDatApexFrame::
Initialize(
    const RWCString& p_datas			// I : the data string
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWCString l_dataAdjusted = "";		// The data string, eventually adjusted

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Copy the data string and eventually adjust it's length
		//********************************************************************

		l_dataAdjusted = p_datas;
		if (l_dataAdjusted.length() < CoDatApexFrame_k_maxLength / 4)
		{
			l_dataAdjusted += RWCString('0', CoDatApexFrame_k_maxLength / 4 - l_dataAdjusted.length());
		}

		//********************************************************************
		// Fill the bit vector of the datas
		//********************************************************************

		_bitVec.SetValues(l_dataAdjusted);

		//********************************************************************
		// After initialization, the instance is valid
		//********************************************************************

		_isBad = FALSE;
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatApexFrame.cpp 1.11 179", 2600);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Check8BitsCrc
// ROL : Check if the 8 bits CRC of the frame is good
//
//****************************************************************************
RWBoolean
CoDatApexFrame::
Check8BitsCrc(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_lastIndex = 0;					// Index of the last byte to use to compute Crc
	int l_counter = 0;						// loop counter
	unsigned char l_value = 0;				// 8-bits-wide value
	unsigned char l_crc = 0;				// the calculated CRC
	unsigned char l_sum = 0;				// for the CRC scrambler

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Compute the index of the last bit to use to compute the crc
		//********************************************************************

		l_lastIndex = GetValidFrameLength(_version);

		//********************************************************************
		// For each 8-bits-wide value of the frame
		//********************************************************************

		for (l_counter = 8; l_counter < l_lastIndex; l_counter += 8)
		{
			//****************************************************************
			// Retrieve the 8-bits-wide value
			//****************************************************************

		    l_value = _bitVec.GetUInt(l_counter, 8);

			//****************************************************************
			// Compute the CRC
			//****************************************************************

		    l_crc = l_crc ^ l_value;
			l_sum = 0;
			if (l_crc == 0)
			{
				l_crc = 0xff;
			}
			if ((l_crc % 2) != 0)
			{
				l_sum++;
			}
			if (((l_crc >> 2) % 2) != 0)
			{
				l_sum++;
			}
			if (((l_crc >> 3) % 2) != 0)
			{
				l_sum++;
			}
			if (((l_crc >> 4) % 2) != 0)
			{
				l_sum++;
			}
			l_sum %= 2;
			l_crc = (l_crc >> 1) + (l_sum << 7);
		}

		//********************************************************************
		// Check if the calculated CRC match the frame's CRC
		//********************************************************************

		l_value = _bitVec.GetUInt(0, 8);
		if (l_crc == l_value)
		{
			_isCrcOk = TRUE;
		}
		else
		{
			_isCrcOk = FALSE;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexFrame.cpp 1.11 283", 2601);
		CO_RETHROW;
	}
	return _isCrcOk;
}




//****************************************************************************
//
// SPG : GetFrameNumber
// ROL : Retrieve the number of the frame
//
//****************************************************************************
unsigned int
CoDatApexFrame::
GetFrameNumber(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	unsigned int l_number = 0;			// the frame number

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the frame's type
		//********************************************************************

		if ((_version == 1.0) || (_version == 1.1) || (_version == 4.0) ||
			(_version == 10.0) || (_version == 11.0))
		{
			l_number = _bitVec.GetUInt(8, 8);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexFrame.cpp 1.11 332", 2602);
		CO_RETHROW;
	}
	return l_number;
}




//****************************************************************************
//
// SPG : Recombined
// ROL : Reconstruct the frame by doing statistics on bits of the frames in
//		 the parameter's list
//
//****************************************************************************
void
CoDatApexFrame::
Recombined(
    const RWTPtrSlist<CoDatApexFrame>& p_list	// I : list of frames
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_firstFrame = 0;					// Index of the first frame to use
	int l_bitCounter = 0;					// loop counter
	int l_true = 0;							// bit status counter
	int l_false = 0;						// bit status counter
	int l_frameCounter = 0;					// loop counter

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Find the index of the first frame to use (If there is an even number
		// of frames, the first one is not used
		//********************************************************************

		l_firstFrame = 1 - (p_list.entries() % 2);

		//********************************************************************
		// For each bit in the frame
		//********************************************************************

		for (l_bitCounter = 0; l_bitCounter < GetValidFrameLength(_version); l_bitCounter++)
		{
			//****************************************************************
			// Reset the counters of bit status
			//****************************************************************

			l_true = 0;
			l_false = 0;

			//****************************************************************
			// For each frame, count the number of true bits and the number
			// of false bits
			//****************************************************************

			for (l_frameCounter = l_firstFrame; l_frameCounter < p_list.entries(); l_frameCounter++)
			{
				if (p_list.at(l_frameCounter)->_bitVec.TestBit(l_bitCounter) == TRUE)
				{
					l_true++;
				}
				else
				{
					l_false++;
				}
			}

			//****************************************************************
			// set the bit status using the statistics result
			//****************************************************************

			if (l_true > l_false)
			{
				_bitVec.SetBit(l_bitCounter);
			}
			else
			{
				_bitVec.ClearBit(l_bitCounter);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexFrame.cpp 1.11 428", 2603);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : GetAsciiHexData
// ROL : Retrieve the content of the bit field in ascii hex form
//
//****************************************************************************
RWCString
CoDatApexFrame::
GetAsciiHexData(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWCString l_asciiHexData = "";			// The return value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the buffer
		//********************************************************************

		l_asciiHexData = _bitVec.GetValues();

		//********************************************************************
		// Eventually trunc it
		//********************************************************************

		l_asciiHexData = CoStcTools_TruncIfNeeded(l_asciiHexData,
												  GetValidFrameLength(_version) / 4);
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexFrame.cpp 1.11 479", 2604);
		CO_RETHROW;
	}
	return l_asciiHexData;
}




//****************************************************************************
//
// SPG : GetValidFrameLength
// ROL : Get the valid frame length (number of bits), depending of the version.
//
//****************************************************************************
/*static*/ int
CoDatApexFrame::
GetValidFrameLength(
	double p_version						// I : the profiler version
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_validFrameLength = 0;				// Ther valid frame length

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	//************************************************************************
	// Depending of the version
	//************************************************************************

	if ((p_version == 1.0) || (p_version == 1.1) || (p_version == 4.0))
	{
		l_validFrameLength = CoDat_k_20bitsArgosIdFrameLength;
	}
	else if ((p_version == 10.0) || (p_version == 11.0))
	{
		l_validFrameLength = CoDat_k_28bitsArgosIdFrameLength;
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    return l_validFrameLength;
}




//****************************************************************************
//
// SPG : CheckFrameDatasLength
// ROL : Check the length of the frame datas (ascii hex values)
//
//****************************************************************************
/*static*/ RWBoolean
CoDatApexFrame::
CheckFrameDatasLength(
    const RWCString& p_datas,				// I : the data string
	double p_version						// I : the profiler version
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWBoolean l_isValid = FALSE;			// The frame length is valid ?

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	//************************************************************************
	// Compare to the valid frame length
	//************************************************************************

	if (p_datas.length() == GetValidFrameLength(p_version) / 4)
	{
		l_isValid = TRUE;
	}
	else
	{
		l_isValid = FALSE;
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    return l_isValid;
}






//****************************************************************************
//
// SPG : GetFrameNumberFromAsciiHexDatas
// ROL : Return the 8-bits frame number as an int from the ASCII hex values of
//		 a frame. 
//
//****************************************************************************
/*static*/ int
CoDatApexFrame::
GetFrameNumberFromAsciiHexDatas(
    const RWCString& p_datas,				// I : the data string
	double p_version						// I : the profiler version
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	char ql_buffer[3];					// temp buffer
    int l_number = 0;					// the frame number

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    //************************************************************************
    // Depending of the version
    //************************************************************************

	if ((p_version == 1.0) || (p_version == 1.1) || (p_version == 4.0) ||
		(p_version == 10.0) || (p_version == 11.0))
	{
		ql_buffer[0] = p_datas[2];
		ql_buffer[1] = p_datas[3];
	}
	ql_buffer[2] = '\0';
	l_number = strtol(ql_buffer, (char**)0, 16);

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    return l_number;
}




//****************************************************************************
//
// SPG : CompareFrameNumber
// ROL : Compare two Apex frames number
//		 -1 if 1st < 2nd
//		 0 if 1st = 2nd
//		 1 if 1st > 2nd
//
//****************************************************************************
/*static*/ int
CoDatApexFrame::
CompareFrameNumber(
    void* qp_1stFrame,						// I : the first frame
    void* qp_2ndFrame,						// I : the second frame
    void*									// N : not used
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_1stFrameNumber = '\0';			// the first frame's number
	int l_2ndFrameNumber = '\0';			// the second frame's number
	int l_compare = 0;						// the comparison result

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve each frame's number
		//********************************************************************

		l_1stFrameNumber = ((CoDatApexFrame*)qp_1stFrame)->GetFrameNumber();
		l_2ndFrameNumber = ((CoDatApexFrame*)qp_2ndFrame)->GetFrameNumber();

		//********************************************************************
		// And compare them
		//********************************************************************

		if (l_1stFrameNumber < l_2ndFrameNumber)
		{
			l_compare = -1;
		}
		else if (l_1stFrameNumber == l_2ndFrameNumber)
		{
			l_compare = 0;
		}
		else
		{
			l_compare = 1;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatApexFrame.cpp 1.11 691", 2605);
		CO_RETHROW;
	}
	return l_compare;
}






