//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoDatArgosMessage 
// ROL : This class implements an Argos message. An Argos message contains
//		 an header and one or several data frames.
//		 The Argos message know how to read itself from a stream.
//
// CRE : 27/08/1999
// AUT : ST
// VER : @(#)1.24
//       @(#)04/03/02
//
//****************************************************************************
// 
// HST : 27/08/1999 ST  Creation
//		 23/09/1999 ST  File access tests
//		 02/11/1999 ST  Add TraceFrames method
//		 07/09/2001 ST  FAE 115 : Profiler info are no more profiler specific
//		 26/09/2001 ST  FAE 122 : Psal instead of Cndc in Metocean profilers
//		 07/12/2001 ST  Add Cmod buoy
//		 11/02/2002 ST  Add Move buoy
//		 09/04/2002 ST  Manage versions of Apex profilers
//		 13/08/2002 ST  FAE 205 : Add the _doubleFlag attribute
//		 27/08/2002 ST  Add Metocean float
//		 11/10/2002 ST  Coriolis V2 compatibility
//		 24/01/2003 ST  FDI 114 : Provor 3.81
//		 14/05/2003 ST  Compatibility with DOS files
//		 22/09/2003 ST  Move datas processed like Provor, Apex, ...
//		 08/10/2003 ST  Manage Apex 11.0 (31 byte message)
//		 14/10/2003 ST  Manage Provor 4.1 (31 byte message)
//		 22/10/2003 ST  Generalize Apex. Remove TraceApexFrames
//		 14/01/2004 ST  Manage Metocean 1.6
//		 02/03/2004 ST  Manage Provor 4.11
//
//****************************************************************************
//
// Copyright, IFREMER, 1999
//
//****************************************************************************
static const char CoDatArgosMessage_IDENT[] = "@(#) CORIOLIS : CoDatArgosMessage.cpp 1.24 04/03/02";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoDatArgosMessage_IN_
#include <CoDatArgosMessage.h>
#undef  _CoDatArgosMessage_IN_
#include <CoDatApexFrame.h>
#include <CoDatAttribute.h>
#include <CoDatCmodFrame.h>
#include <CoDatLocatedRecord.h>
#include <CoDatMetoceanFrame.h>
#include <CoDatMoveFrame.h>
#include <CoDatPlatform.h>
#include <CoDatPosition.h>
#include <CoDatProvorFrame.h>
#include <CoDatSurfaceDriftFrame.h>
#include <CoDatTechnicalStatus.h>
#include <CoStcCatManager.h>
#include <CoStcError.h>
#include <rw/rwdate.h>
#include <rw/tpslist.h>
#include <rw/tvsrtvec.h>
#include <stdlib.h>
#include <stream.h>



//----------------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------------
const int CoDat_k_frameHeaderLength = 22;




//****************************************************************************
//
// SPG : CoDatArgosMessage
// ROL : Constructor
//
//****************************************************************************
CoDatArgosMessage::
CoDatArgosMessage(
	)
	: _isBad(TRUE)
	, _programNumber("")
	, _beaconCode("")
	, _linesCount(0)
	, _sensors(0)
	, _satelliteName('\0')
	, _q_surfaceDriftFrame((CoDatSurfaceDriftFrame*)0)
	, _q_surfaceRecord((CoDatLocatedRecord*)0)
	, _message("")
	, _doubleFlag(FALSE)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : ~CoDatArgosMessage
// ROL : Destructor
//
//****************************************************************************
CoDatArgosMessage::
~CoDatArgosMessage(
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : operator>> (friend)
// ROL : Read itself from a stream
//
//****************************************************************************
/*friend*/ istream&
operator>>(
	istream& p_stream,				// I : the stream to read from
	CoDatArgosMessage& p_message		// I : the object itself
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Read the stream
		//********************************************************************

		p_message.Read(p_stream);
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 178", 100);
		CO_RETHROW;
	}
	return p_stream;
}




//****************************************************************************
//
// SPG : GetProvorFrames
// ROL : Extract the Provor datas form the message, create the frames, and
//	 return them
//
//****************************************************************************
void
CoDatArgosMessage::
GetProvorFrames(
	const CoDatPlatform& p_platform,		// I : the related platform
	CoDatTechnicalStatus& p_status,			// I : The technical status
	CoDatLocatedRecord*& qp_surfaceRecord,	// O : The surface drift record (if exists)
	RWTPtrSlist<CoDatProvorFrame>& p_frames	// O : the provor frames
	) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_version = 0.0;					// the profiler version
	int l_incompleteDescentFrameCount = 0;	// The count of incomplete frames
	int l_incompleteDriftFrameCount = 0;	// The count of incomplete frames
	int l_incompleteAscentFrameCount = 0;	// The count of incomplete frames
	int l_incompleteOtherFrameCount = 0;	// The count of incomplete frames
	int l_badTypeFrameCount = 0;			// The count of bad type frames
	int l_messageIndex = 0;					// Indexes the message datas
	RWCString l_frameDatas = "";			// Current frame datas
	RWTime l_frameDateTime =				// the frame emitting date/time
		(unsigned long)0;
	int l_type = 0;							// the frame type
	CoDatProvorFrame* ql_frame =			// The current frame object
		(CoDatProvorFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the profiler version
		//********************************************************************

		l_version = p_platform.GetAttribute("PR_VERSION").GetDoubleValue();

		//********************************************************************
		// return the surface drift record
		//********************************************************************

		qp_surfaceRecord = _q_surfaceRecord;

		//********************************************************************
		// Reset the parameters
		//********************************************************************

		p_frames.clear();
		if (p_status.HasAttribute("TRANS_DPROF_Incompl") == TRUE)
		{
			l_incompleteDescentFrameCount = p_status.GetAttribute("TRANS_DPROF_Incompl").GetIntValue();
		}
		if (p_status.HasAttribute("TRANS_IDRIFT_Incompl") == TRUE)
		{
			l_incompleteDriftFrameCount = p_status.GetAttribute("TRANS_IDRIFT_Incompl").GetIntValue();
		}
		if (p_status.HasAttribute("TRANS_APROF_Incompl") == TRUE)
		{
			l_incompleteAscentFrameCount = p_status.GetAttribute("TRANS_APROF_Incompl").GetIntValue();
		}
		if (p_status.HasAttribute("TRANS_OTHER_Incompl") == TRUE)
		{
			l_incompleteOtherFrameCount = p_status.GetAttribute("TRANS_OTHER_Incompl").GetIntValue();
		}
		if (p_status.HasAttribute("TRANS_Bad_type") == TRUE)
		{
			l_badTypeFrameCount = p_status.GetAttribute("TRANS_Bad_type").GetIntValue();
		}

		//********************************************************************
		// While we can extract frame datas in the message body
		//********************************************************************

		while (ExtractNextFrameDatas(l_messageIndex, l_frameDatas, l_frameDateTime) != 0)
		{
			//****************************************************************
			// For new Argos messages (> 3.8) adjust the frame length
			//****************************************************************

			if ((l_version == 3.8) || (l_version == 4.1))
			{
				l_frameDatas += "00";
			}
			else if ((l_version == 3.81) || (l_version == 4.11))
			{
				l_frameDatas = l_frameDatas(2, l_frameDatas.length() - 2);
				l_frameDatas += "00";
			}

			//****************************************************************
			// Check if the length of the frame datas is Ok, if not
			//****************************************************************

			if (CoDatProvorFrame::CheckFrameDatasLength(l_frameDatas, l_version) == FALSE)
			{
				l_type = CoDatProvorFrame::GetTypeFromAsciiHexDatas(l_frameDatas, l_version);
				if ((l_type == 1) || (l_type == 4))
				{
					l_incompleteDescentFrameCount++;
				}
				else if ((l_type == 2) || (l_type == 5))
				{
					l_incompleteDriftFrameCount++;
				}
				else if ((l_type == 3) || (l_type == 6))
				{
					l_incompleteAscentFrameCount++;
				}
				else
				{
					l_incompleteOtherFrameCount++;
				}
			}

			//****************************************************************
			// The length of the frame datas is Ok
			//****************************************************************

			else
			{
				//************************************************************
				// Create the frame object
				//************************************************************

				ql_frame = CreateProvorFrame(p_platform, l_frameDatas);

				//************************************************************
				// If the frame object exists, append it to the output list
				//************************************************************

				if (ql_frame != (CoDatProvorFrame*)0)
				{
					ql_frame->SetEmittingDateTime(l_frameDateTime);
					ql_frame->SetEmittingSatellite(_satelliteName);
					p_frames.append(ql_frame);
				}

				//************************************************************
				// Else, it means that the frame type was not good
				//************************************************************

				else
				{
					l_badTypeFrameCount++;
				}
			}
		}

		//********************************************************************
		// Update the status
		//********************************************************************

		p_status.AddAttribute(CoDatAttribute("TRANS_DPROF_Incompl", l_incompleteDescentFrameCount));
		p_status.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Incompl", l_incompleteDriftFrameCount));
		p_status.AddAttribute(CoDatAttribute("TRANS_APROF_Incompl", l_incompleteAscentFrameCount));
		p_status.AddAttribute(CoDatAttribute("TRANS_OTHER_Incompl", l_incompleteOtherFrameCount));
		p_status.AddAttribute(CoDatAttribute("TRANS_Bad_type", l_badTypeFrameCount));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 361", 101);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : TraceProvorFrames
// ROL : Extract the Provor datas form the message, create the frames, and
//		 trace their content
//
//****************************************************************************
void
CoDatArgosMessage::
TraceProvorFrames(
    const CoDatPlatform& p_platform,		// I : the related platform
    ostream& p_traceStream					// I : the trace stream
	) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_version = 0.0;					// the profiler version
	int l_messageIndex = 0;					// Indexes the message datas
	RWCString l_frameDatas = "";			// Current frame datas
	RWTime l_dummyDateTime =				// dummy date/time for date extraction
		RWTime((unsigned long)0);
	CoDatProvorFrame* ql_frame =			// The current frame object
		(CoDatProvorFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the profiler version
		//********************************************************************

		l_version = p_platform.GetAttribute("PR_VERSION").GetDoubleValue();

		//********************************************************************
		// While we can extract frame datas in the message body
		//********************************************************************

		while (ExtractNextFrameDatas(l_messageIndex, l_frameDatas, l_dummyDateTime) != 0)
		{
			//****************************************************************
			// For new Argos messages (> 3.8) adjust the frame length
			//****************************************************************

			if ((l_version == 3.8) || (l_version == 4.1))
			{
				l_frameDatas += "00";
			}
			else if ((l_version == 3.81) || (l_version == 4.11))
			{
				l_frameDatas = l_frameDatas(2, l_frameDatas.length() - 2);
				l_frameDatas += "00";
			}

			//****************************************************************
			// The length of the frame datas is Ok
			//****************************************************************

			if (CoDatProvorFrame::CheckFrameDatasLength(l_frameDatas, l_version) != 0)
			{
				//************************************************************
				// Trace the Hexa content of the frame and it's type
				//************************************************************

				p_traceStream << CoStc_g_catManager.GetInfo(3, 101);
				p_traceStream << l_frameDatas << endl;
				p_traceStream << CoStc_g_catManager.GetInfo(3, 102);
				p_traceStream << CoDatProvorFrame::GetTypeFromAsciiHexDatas(l_frameDatas, l_version);

				//************************************************************
				// Create the frame object
				//************************************************************

				ql_frame = CreateProvorFrame(p_platform, l_frameDatas);

				//************************************************************
				// If the frame object exists,
				//************************************************************

				if (ql_frame != (CoDatProvorFrame*)0)
				{
					//********************************************************
					// Check the CRC
					//********************************************************

					p_traceStream << endl;
					if (ql_frame->IsCrcOk() == TRUE)
					{
						p_traceStream << CoStc_g_catManager.GetInfo(3, 103) << endl;
					}
					else
					{
						p_traceStream << CoStc_g_catManager.GetInfo(3, 104) << endl;
					}

					//********************************************************
					// And trace the frame content
					//********************************************************

					ql_frame->Trace(p_traceStream);
				}

				//************************************************************
				// Else, it means that the frame type was not good, trace the
				// type
				//************************************************************

				else
				{
					p_traceStream << CoStc_g_catManager.GetInfo(3, 105) << endl;
				}
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 494", 102);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : GetApexFrames
// ROL : Extract the Apex datas form the message, create the frames, and
//	 return them
//
//****************************************************************************
void
CoDatArgosMessage::
GetApexFrames(
	const CoDatPlatform& p_platform,		// I : the related platform
	CoDatTechnicalStatus& p_status,			// I : The technical status
	CoDatLocatedRecord*& qp_surfaceRecord,	// O : The surface drift record (if exists)
	RWTPtrSlist<CoDatApexFrame>& p_frames	// O : the apex frames
	) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_version = 0.0;					// the profiler version
	int l_incompleteAscentFrameCount = 0;	// The count of incomplete frames
	int l_incompleteOtherFrameCount = 0;	// The count of incomplete frames
	int l_messageIndex = 0;					// Indexes the message datas
	RWCString l_frameDatas = "";			// Current frame datas
	RWTime l_frameDateTime =				// the frame emitting date/time
		(unsigned long)0;
	int l_frameNumber = 0;					// the frame number
	CoDatApexFrame* ql_frame =				// The current frame object
		(CoDatApexFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the profiler version
		//********************************************************************

		l_version = p_platform.GetAttribute("PR_VERSION").GetDoubleValue();

		//********************************************************************
		// return the surface drift record
		//********************************************************************

		qp_surfaceRecord = _q_surfaceRecord;

		//********************************************************************
		// Reset the parameters
		//********************************************************************

		p_frames.clear();
		if (p_status.HasAttribute("TRANS_APROF_Incompl") == TRUE)
		{
			l_incompleteAscentFrameCount = p_status.GetAttribute("TRANS_APROF_Incompl").GetIntValue();
		}
		if (p_status.HasAttribute("TRANS_OTHER_Incompl") == TRUE)
		{
			l_incompleteOtherFrameCount = p_status.GetAttribute("TRANS_OTHER_Incompl").GetIntValue();
		}

		//********************************************************************
		// While we can extract frame datas in the message body
		//********************************************************************

		while (ExtractNextFrameDatas(l_messageIndex, l_frameDatas, l_frameDateTime) != 0)
		{
			//****************************************************************
			// Check if the length of the frame datas is Ok, if not
			//****************************************************************

			if (CoDatApexFrame::CheckFrameDatasLength(l_frameDatas, l_version) == FALSE)
			{
				l_frameNumber = CoDatApexFrame::GetFrameNumberFromAsciiHexDatas(l_frameDatas, l_version);
				if (l_frameNumber != 1)
				{
					l_incompleteAscentFrameCount++;
				}
				else
				{
					l_incompleteOtherFrameCount++;
				}
			}

			//****************************************************************
			// The length of the frame datas is Ok
			//****************************************************************

			else
			{
				//************************************************************
				// Create the frame object
				//************************************************************

				ql_frame = CreateApexFrame(p_platform, l_frameDatas);

				//************************************************************
				// If the frame object exists, append it to the output list
				//************************************************************

				if (ql_frame != (CoDatApexFrame*)0)
				{
					ql_frame->SetEmittingDateTime(l_frameDateTime);
					ql_frame->SetEmittingSatellite(_satelliteName);
					p_frames.append(ql_frame);
				}
			}
		}

		//********************************************************************
		// Update the status
		//********************************************************************

		p_status.AddAttribute(CoDatAttribute("TRANS_APROF_Incompl", l_incompleteAscentFrameCount));
		p_status.AddAttribute(CoDatAttribute("TRANS_OTHER_Incompl", l_incompleteOtherFrameCount));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 627", 103);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : GetMetoceanFrames
// ROL : Extract the Metocean datas from the message, create the frames, and
//		 return them
//
//****************************************************************************
void
CoDatArgosMessage::
GetMetoceanFrames(
	const CoDatPlatform& p_platform,		// I : the related platform
	CoDatTechnicalStatus& p_status,			// I : The technical status
	CoDatLocatedRecord*& qp_surfaceRecord,	// O : The surface drift record (if exists)
	RWTPtrSlist<CoDatMetoceanFrame>& p_frames	// O : the metocean frames
	) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_version = 0.0;					// the profiler version
	int l_incompleteDriftFrameCount = 0;	// The count of incomplete frames
	int l_incompleteAscentFrameCount = 0;	// The count of incomplete frames
	int l_incompleteOtherFrameCount = 0;	// The count of incomplete frames
	int l_badTypeFrameCount = 0;			// The count of bad type frames
	int l_messageIndex = 0;					// Indexes the message datas
	RWCString l_frameDatas = "";			// Current frame datas
	RWTime l_frameDateTime =				// the frame emitting date/time
		(unsigned long)0;
	int l_frameNumber = 0;					// the frame number
	CoDatMetoceanFrame* ql_frame =			// The current frame object
		(CoDatMetoceanFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the profiler version
		//********************************************************************

		l_version = p_platform.GetAttribute("PR_VERSION").GetDoubleValue();

		//********************************************************************
		// return the surface drift record
		//********************************************************************

		qp_surfaceRecord = _q_surfaceRecord;

		//********************************************************************
		// Reset the parameters
		//********************************************************************

		p_frames.clear();
		if (p_status.HasAttribute("TRANS_IDRIFT_Incompl") == TRUE)
		{
			l_incompleteDriftFrameCount = p_status.GetAttribute("TRANS_IDRIFT_Incompl").GetIntValue();
		}
		if (p_status.HasAttribute("TRANS_APROF_Incompl") == TRUE)
		{
			l_incompleteAscentFrameCount = p_status.GetAttribute("TRANS_APROF_Incompl").GetIntValue();
		}
		if (p_status.HasAttribute("TRANS_OTHER_Incompl") == TRUE)
		{
			l_incompleteOtherFrameCount = p_status.GetAttribute("TRANS_OTHER_Incompl").GetIntValue();
		}
		if (p_status.HasAttribute("TRANS_Bad_type") == TRUE)
		{
			l_badTypeFrameCount = p_status.GetAttribute("TRANS_Bad_type").GetIntValue();
		}

		//********************************************************************
		// While we can extract frame datas in the message body
		//********************************************************************

		while (ExtractNextFrameDatas(l_messageIndex, l_frameDatas, l_frameDateTime) != 0)
		{
			//****************************************************************
			// For new Argos messages (> 1.6) adjust the frame length
			//****************************************************************

			if (l_version == 1.6)
			{
				l_frameDatas = l_frameDatas(2, l_frameDatas.length() - 2);
			}

			//****************************************************************
			// Check if the length of the frame datas is Ok, if not
			//****************************************************************

			if (CoDatMetoceanFrame::CheckFrameDatasLength(l_frameDatas, l_version) == FALSE)
			{
				l_frameNumber = CoDatMetoceanFrame::GetFrameNumberFromAsciiHexDatas(l_frameDatas, l_version);
				if ((l_frameNumber >= 2) && (l_frameNumber <= 15))
				{
					l_incompleteAscentFrameCount++;
				}
				else if ((l_frameNumber >= 225) && (l_frameNumber <= 249))
				{
					l_incompleteDriftFrameCount++;
				}
				else
				{
					l_incompleteOtherFrameCount++;
				}
			}

			//****************************************************************
			// The length of the frame datas is Ok
			//****************************************************************

			else
			{
				//************************************************************
				// Create the frame object
				//************************************************************

				ql_frame = CreateMetoceanFrame(p_platform, l_frameDatas);

				//************************************************************
				// If the frame object exists, append it to the output list
				//************************************************************

				if (ql_frame != (CoDatMetoceanFrame*)0)
				{
					ql_frame->SetEmittingDateTime(l_frameDateTime);
					ql_frame->SetEmittingSatellite(_satelliteName);
					p_frames.append(ql_frame);
				}

				//************************************************************
				// Else, it means that the frame type was not good
				//************************************************************

				else
				{
					l_badTypeFrameCount++;
				}
			}
		}

		//********************************************************************
		// Update the status
		//********************************************************************

		p_status.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Incompl", l_incompleteDriftFrameCount));
		p_status.AddAttribute(CoDatAttribute("TRANS_APROF_Incompl", l_incompleteAscentFrameCount));
		p_status.AddAttribute(CoDatAttribute("TRANS_OTHER_Incompl", l_incompleteOtherFrameCount));
		p_status.AddAttribute(CoDatAttribute("TRANS_Bad_type", l_badTypeFrameCount));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 794", 104);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : GetCmodFrames
// ROL : Extract the Cmod datas form the message, create the frames, and
//	 return them
//
//****************************************************************************
void
CoDatArgosMessage::
GetCmodFrames(
	CoDatSurfaceDriftFrame*& qp_surfaceDriftFrame,	// O : Surface drift frame (if exists)
	RWTPtrSlist<CoDatCmodFrame>& p_cmodFrames,	// O : the cmod frames
	int& p_incompleteFrameCount				// O : number of incomplete frames
	) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// return the surface drift frame
		//********************************************************************

		qp_surfaceDriftFrame = _q_surfaceDriftFrame;

		//********************************************************************
		// Decod and return the provor frames
		//********************************************************************

		DecodCmodFrames(p_cmodFrames,
						p_incompleteFrameCount);
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 847", 105);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : TraceCmodFrames
// ROL : Extract the Cmod datas form the message, create the frames, and
//		 trace their content
//
//****************************************************************************
void
CoDatArgosMessage::
TraceCmodFrames(
    ostream& p_traceStream					// I : the trace stream
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_messageIndex = 0;					// Indexes the message datas
	RWCString l_frameDatas = "";			// Current frame datas
	CoDatCmodFrame* ql_frame =				// The current frame object
		(CoDatCmodFrame*)0;
	int l_frameNumber = 0;					// the frame number
	RWTime l_frameDateTime =				// the frame emitting date/time
		RWTime((unsigned long)0);

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// While we can extract frame datas in the message body
		//********************************************************************

		while (ExtractNextFrameDatas(l_messageIndex, l_frameDatas, l_frameDateTime) != 0)
		{
			//****************************************************************
			// The length of the frame datas is Ok
			//****************************************************************

			if (CoDatCmodFrame::CheckAsciiHexDatasLength(l_frameDatas) != 0)
			{
				//************************************************************
				// Trace the Hexa content of the frame and it's number
				//************************************************************

				p_traceStream << CoStc_g_catManager.GetInfo(3, 101);
				p_traceStream << l_frameDatas << endl;

				//************************************************************
				// Create the frame object
				//************************************************************

				ql_frame = CreateCmodFrame(l_frameDatas);
				ql_frame->SetEmittingDateTime(l_frameDateTime);

				//************************************************************
				// Check the CRC
				//************************************************************

				if (ql_frame->AreChecksumsOk() == TRUE)
				{
					p_traceStream << CoStc_g_catManager.GetInfo(3, 107) << endl;
				}
				else
				{
					p_traceStream << CoStc_g_catManager.GetInfo(3, 108) << endl;
				}

				//************************************************************
				// And trace the frame content
				//************************************************************

				ql_frame->Trace(p_traceStream);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 940", 106);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : GetMoveFrames
// ROL : Extract the Move datas from the message, create the frames, and
//		 return them
//
//****************************************************************************
void
CoDatArgosMessage::
GetMoveFrames(
	const CoDatPlatform& p_platform,		// I : the related platform
	CoDatTechnicalStatus& p_status,			// I : The technical status
	CoDatLocatedRecord*& qp_surfaceRecord,	// O : The surface drift record (if exists)
	RWTPtrSlist<CoDatMoveFrame>& p_frames	// O : the move frames
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_incompleteFrameCount = 0;			// The count of incomplete frames
	int l_messageIndex = 0;					// Indexes the message datas
	RWCString l_frameDatas = "";			// Current frame datas
	RWTime l_frameDateTime =				// the frame emitting date/time
		(unsigned long)0;
	int l_frameNumber = 0;					// the frame number
	CoDatMoveFrame* ql_frame =				// The current frame object
		(CoDatMoveFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// return the surface drift record
		//********************************************************************

		qp_surfaceRecord = _q_surfaceRecord;

		//********************************************************************
		// Reset the parameters
		//********************************************************************

		p_frames.clear();
		if (p_status.HasAttribute("TRANS_Incompl") == TRUE)
		{
			l_incompleteFrameCount = p_status.GetAttribute("TRANS_Incompl").GetIntValue();
		}

		//********************************************************************
		// While we can extract frame datas in the message body
		//********************************************************************

		while (ExtractNextFrameDatas(l_messageIndex, l_frameDatas, l_frameDateTime) != 0)
		{
			//****************************************************************
			// Check if the length of the frame datas is Ok, if not
			//****************************************************************

			if (CoDatMoveFrame::CheckFrameDatasLength(l_frameDatas) == FALSE)
			{
				l_incompleteFrameCount++;
			}

			//****************************************************************
			// The length of the frame datas is Ok
			//****************************************************************

			else
			{
				//************************************************************
				// Create the frame object
				//************************************************************

				ql_frame = CreateMoveFrame(p_platform, l_frameDatas);

				//************************************************************
				// If the frame object exists, append it to the output list
				//************************************************************

				if (ql_frame != (CoDatMoveFrame*)0)
				{
					ql_frame->SetEmittingDateTime(l_frameDateTime);
					ql_frame->SetEmittingSatellite(_satelliteName);
					p_frames.append(ql_frame);
				}
			}
		}

		//********************************************************************
		// Update the status
		//********************************************************************

		p_status.AddAttribute(CoDatAttribute("TRANS_Incompl", l_incompleteFrameCount));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1052", 107);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : CompareDateTime
// ROL : Compare two Argos messages using the header dating and returns
//       -1 if 1st < 2nd
//       0 if 1st = 2nd
//       1 if 1st > 2nd
//
//****************************************************************************
/*static*/ int
CoDatArgosMessage::
CompareDateTime(
	void* qp_1stMessage,					// I : the first message
	void* qp_2ndMessage,					// I : the second message
	void*									// N : not used
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDatSurfaceDriftFrame* ql_1stDrift =	// The 1st message's surface drift frame
		(CoDatSurfaceDriftFrame*)0;
	CoDatSurfaceDriftFrame* ql_2ndDrift =	// The 2nd message's surface drift frame
		(CoDatSurfaceDriftFrame*)0;
	int l_compare = 0;						// The comparison result

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve for both messages their surface drift frame
		//********************************************************************

		ql_1stDrift = ((CoDatArgosMessage*)qp_1stMessage)->_q_surfaceDriftFrame;
		ql_2ndDrift = ((CoDatArgosMessage*)qp_2ndMessage)->_q_surfaceDriftFrame;

		//********************************************************************
		// if the first surface drift frame is defined
		//********************************************************************

		if (ql_1stDrift != (CoDatSurfaceDriftFrame*)0)
		{
			//****************************************************************
			// if the second surface drift frame is defined
			//****************************************************************

			if (ql_2ndDrift != (CoDatSurfaceDriftFrame*)0)
			{
				l_compare = ql_1stDrift->GetDateTime().compareTo(&(ql_2ndDrift->GetDateTime()));
			}

			//****************************************************************
			// if the second surface drift frame is not defined
			//****************************************************************

			else
			{
				l_compare = -1;
			}
		}

		//********************************************************************
		// if the first surface drift frame is not defined
		//********************************************************************

		else
		{
			//****************************************************************
			// if the second surface drift frame is defined
			//****************************************************************

			if (ql_2ndDrift != (CoDatSurfaceDriftFrame*)0)
			{
				l_compare = 1;
			}

			//****************************************************************
			// if the second surface drift frame is not defined
			//****************************************************************

			else
			{
				l_compare = 0;
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1157", 108);
		CO_RETHROW;
	}
	return l_compare;
}




//****************************************************************************
//
// SPG : Read
// ROL : Read itself from a stream
//
//****************************************************************************
void
CoDatArgosMessage::
Read(
	istream& p_stream						// I : the stream to read from
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	char ql_buffer[32];						// Reading buffer
	RWCString l_localization = "";			// the localization string
	CoDatPosition l_position;				// The surface record position
	RWTime l_dateTime =						// The surface record date/time
		(unsigned long)0;
    int l_year = 0;							// The year
    int l_month = 0;						// The month
    int l_day = 0;							// The day in the month
    int l_hours = 0;						// The hours in the day
    int l_minutes = 0;						// The minutes in the hour
    int l_seconds = 0;						// The seconds in the minute
    double l_latitude;						// The latitude
    double l_longitude;						// The longitude
	int l_lineCounter = 0;					// loop counter
	RWCString l_line = "";					// A single line of the message

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// If the reading fails, the instance is not ok
		//********************************************************************

		_isBad = TRUE;

		//********************************************************************
		// Read the program number
		//********************************************************************

		p_stream >> _programNumber;

		//********************************************************************
		// If the program number has not been read, the end of stream has
		// been reached, the instance is not ok but this is not an error
		//********************************************************************

		if (p_stream.fail() == 0)
		{
			//****************************************************************
			// but if the end of stream is reached, this is an error
			//****************************************************************

			if (p_stream.eof() != 0)
			{
				CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1229", 109);
				CO_THROW(CoStc_k_error);
			}

			//****************************************************************
			// Read the beacon code
			//****************************************************************

			p_stream.get(ql_buffer[0]);
			while (ql_buffer[0] == ' ')
			{
				p_stream.get(ql_buffer[0]);
				if ((p_stream.fail() != 0) || (p_stream.eof() != 0))
				{
					CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1243", 109);
					CO_THROW(CoStc_k_error);
				}
			}
			p_stream.get(ql_buffer + 1, 31, ' ');
			_beaconCode = ql_buffer;
			if ((p_stream.fail() != 0) || (p_stream.eof() != 0))
			{
				CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1251", 109);
				CO_THROW(CoStc_k_error);
			}

			//****************************************************************
			// Read the number of lines in the message
			//****************************************************************

			p_stream >> _linesCount;
			if ((p_stream.fail() != 0) || (p_stream.eof() != 0))
			{
				CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1262", 109);
				CO_THROW(CoStc_k_error);
			}
			_linesCount--;

			//****************************************************************
			// Read the number of sensors
			//****************************************************************

			p_stream >> _sensors;
			if ((p_stream.fail() != 0) || (p_stream.eof() != 0))
			{
				CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1274", 109);
				CO_THROW(CoStc_k_error);
			}

			//****************************************************************
			// Read to the end of line
			//****************************************************************

			l_localization.readLine(p_stream, TRUE);
			if ((p_stream.fail() != 0) || (p_stream.eof() != 0))
			{
				CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1285", 109);
				CO_THROW(CoStc_k_error);
			}
			l_localization = l_localization.strip(RWCString::trailing, '\r');

			//****************************************************************
			// Extract the satellite name from the end of line read
			//****************************************************************

			_satelliteName = l_localization[0];
			l_localization.remove(0, 1);
			l_localization = l_localization.strip(RWCString::both);

			//****************************************************************
			// The localization string is not empty, create the surface
			// drift frame
			//****************************************************************

			if (l_localization != "")
			{
				//************************************************************
				// Create the surface drift frame
				//************************************************************

				_q_surfaceDriftFrame = new CoDatSurfaceDriftFrame();
				_q_surfaceDriftFrame->Initialize(l_localization);
				if (_q_surfaceDriftFrame->IsBad() == TRUE)
				{
					delete _q_surfaceDriftFrame;
					_q_surfaceDriftFrame = (CoDatSurfaceDriftFrame*)0;
				}
			}

			//****************************************************************
			// The localization string is not empty, create the surface record
			//****************************************************************

			if (l_localization != "")
			{
				//************************************************************
				// Create the record
				//************************************************************

				_q_surfaceRecord = new CoDatLocatedRecord();

				//************************************************************
				// Read the localization class
				//************************************************************

				l_position.SetClass(l_localization[0]);
				l_localization.remove(0, 1);
				l_localization.strip(RWCString::both);

				//************************************************************
				// Read the date-time and the latitude/longitude
				//************************************************************

				if ((l_localization != "") &&
					(sscanf(l_localization.data(),
							"%d-%d-%d %d:%d:%d %lf %lf",
							&l_year, &l_month, &l_day,
							&l_hours, &l_minutes, &l_seconds,
							&l_latitude, &l_longitude) == 8))
				{
					l_dateTime = RWTime(RWDate(l_day, l_month, l_year),
										l_hours, l_minutes, l_seconds);
					l_position.SetLatitude(l_latitude);
					l_position.SetLongitude(l_longitude);
				}
				else
				{
					l_dateTime = (unsigned long)0;
				}

				//************************************************************
				// If everything is read
				//************************************************************

				if (l_dateTime.isValid() == TRUE)
				{
					_q_surfaceRecord->SetDateTime(l_dateTime);
					_q_surfaceRecord->SetPosition(l_position);
				}
				else
				{
					delete _q_surfaceRecord;
				}
			}

			//****************************************************************
			// Read the content of the message
			//****************************************************************

			for (l_lineCounter = 0; l_lineCounter < _linesCount; l_lineCounter++)
			{
				l_line.readLine(p_stream, FALSE);
				if ((p_stream.fail() != 0) || (p_stream.eof() != 0))
				{
					CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1383", 109);
					CO_THROW(CoStc_k_error);
				}
				l_line = l_line.strip(RWCString::trailing, '\r');
				_message += l_line + "\n";
			}

			//****************************************************************
			// Everything has been read, instance is ok
			//****************************************************************

			_isBad = FALSE;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1404", 110);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : CreateProvorFrame
// ROL : Create a frame object for the frame Datas
//
//****************************************************************************
CoDatProvorFrame*
CoDatArgosMessage::
CreateProvorFrame(
    const CoDatPlatform& p_platform,		// I : the related platform
    const RWCString& p_frameDatas			// I : the extracted datas
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_version = 0.0;					// the profiler version
	int l_type = 0;							// the frame type
	CoDatProvorFrame* ql_frame =			// The created frame
		(CoDatProvorFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Depending of the version and the type
		//********************************************************************

		l_version = p_platform.GetAttribute("PR_VERSION").GetDoubleValue();
		l_type = CoDatProvorFrame::GetTypeFromAsciiHexDatas(p_frameDatas, l_version);
		if ((l_version == 2.2) || (l_version == 2.6) || (l_version == 2.7))
		{
			if ((l_type == 0) || ((l_type >= 1) && (l_type <= 3)))
			{
				ql_frame = new CoDatProvorFrame();
			}
		}
		else if ((l_version == 3.21) || (l_version == 3.5) || (l_version == 3.51) ||
				 (l_version == 3.52) || (l_version == 3.61) || (l_version == 3.8) ||
				 (l_version == 3.81) || (l_version == 4.1) || (l_version == 4.11))
		{
			if ((l_type == 0) || ((l_type >= 4) && (l_type <= 6)))
			{
				ql_frame = new CoDatProvorFrame();
			}
		}
		else if (l_version == 4.0)
		{
			if ((l_type == 0) || ((l_type >= 1) && (l_type <= 3)))
			{
				ql_frame = new CoDatProvorFrame();
			}
		}

		//********************************************************************
		// If the frame has been created, initialize it
		//********************************************************************

		if (ql_frame != (CoDatProvorFrame*)0)
		{
			ql_frame->Initialize(p_frameDatas);
			ql_frame->SetVersion(l_version);
			if (p_platform.GetAttribute("PR_CRC_TYPE").GetIntValue() == 8)
			{
				ql_frame->Check8BitsCrc();
			}
			else
			{
				ql_frame->Check16BitsCrc();
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1495", 111);
		CO_RETHROW;
	}
	return ql_frame;
}




//****************************************************************************
//
// SPG : CreateApexFrame
// ROL : Create a frame object for the frame Datas
//
//****************************************************************************
CoDatApexFrame*
CoDatArgosMessage::
CreateApexFrame(
    const CoDatPlatform& p_platform,		// I : the related platform
    const RWCString& p_frameDatas			// I : the extracted datas
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDatApexFrame* ql_frame =				// The created frame
		(CoDatApexFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Create the frame object and initialize it
		//********************************************************************

		ql_frame = new CoDatApexFrame();
		ql_frame->Initialize(p_frameDatas);
		ql_frame->SetVersion(p_platform.GetAttribute("PR_VERSION").GetDoubleValue());
		ql_frame->Check8BitsCrc();
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1546", 112);
		CO_RETHROW;
	}
	return ql_frame;
}




//****************************************************************************
//
// SPG : CreateMetoceanFrame
// ROL : Create a frame object for the frame Datas
//
//****************************************************************************
CoDatMetoceanFrame*
CoDatArgosMessage::
CreateMetoceanFrame(
    const CoDatPlatform& p_platform,		// I : the related platform
    const RWCString& p_frameDatas			// I : the extracted datas
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_version = 0.0;					// the profiler version
	int l_frameNumber = 0;					// the frame number
	CoDatMetoceanFrame* ql_frame =			// The created frame
		(CoDatMetoceanFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Depending of the version and the type
		//********************************************************************

		l_version = p_platform.GetAttribute("PR_VERSION").GetDoubleValue();
		l_frameNumber = CoDatMetoceanFrame::GetFrameNumberFromAsciiHexDatas(p_frameDatas, l_version);
		if ((l_frameNumber == 1) ||
			((l_frameNumber >= 2) && (l_frameNumber <= 15)) ||
			((l_frameNumber >= 225) && (l_frameNumber <= 249)))
		{
			ql_frame = new CoDatMetoceanFrame();
		}

		//********************************************************************
		// If the frame has been created, initialize it
		//********************************************************************

		if (ql_frame != (CoDatMetoceanFrame*)0)
		{
			ql_frame->Initialize(p_frameDatas);
			ql_frame->SetVersion(l_version);
			ql_frame->CheckCrc();
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1614", 113);
		CO_RETHROW;
	}
	return ql_frame;
}




//****************************************************************************
//
// SPG : DecodCmodFrames
// ROL : Extract the Cmod frames from the message body and return them
//
//****************************************************************************
void
CoDatArgosMessage::
DecodCmodFrames(
    RWTPtrSlist<CoDatCmodFrame>& p_cmodFrames,	// O : the Cmod frames
	int& p_incompleteFrameCount				// O : number of incomplete descending frames
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_messageIndex = 0;					// Indexes the message datas
	RWCString l_frameDatas = "";			// Current frame datas
	CoDatCmodFrame* ql_frame =				// The current frame object
		(CoDatCmodFrame*)0;
	RWTime l_frameDateTime =				// the frame emitting date/time
		RWTime((unsigned long)0);
	RWTValSortedVector<RWTime> l_incompleteFrameDateTime;
											// the date/time for the incomplete frames
	RWTime l_currentDateTime =				// the current frame date/time
		RWTime((unsigned long)0);

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Reset the parameters
		//********************************************************************

		p_cmodFrames.clear();
		p_incompleteFrameCount = 0;

		//********************************************************************
		// While we can extract frame datas in the message body
		//********************************************************************

		while (ExtractNextFrameDatas(l_messageIndex, l_frameDatas, l_frameDateTime) != 0)
		{
			//****************************************************************
			// Check if the length of the frame datas is Ok, if not
			//****************************************************************

			if (CoDatCmodFrame::CheckAsciiHexDatasLength(l_frameDatas) == 0)
			{
				l_incompleteFrameDateTime.append(l_frameDateTime);
			}

			//****************************************************************
			// The length of the frame datas is Ok
			//****************************************************************

			else
			{
				//************************************************************
				// Create the frame object
				//************************************************************

				ql_frame = CreateCmodFrame(l_frameDatas);

				//************************************************************
				// Append it to the output list
				//************************************************************

				ql_frame->SetEmittingDateTime(l_frameDateTime);
				ql_frame->SetEmittingSatellite(_satelliteName);
				p_cmodFrames.append(ql_frame);
			}
		}

		//********************************************************************
		// Remove doubles from incomplete frames
		//********************************************************************

		l_currentDateTime = RWTime();
		while (l_incompleteFrameDateTime.isEmpty() == FALSE)
		{
			l_currentDateTime = l_incompleteFrameDateTime.removeFirst();
			p_incompleteFrameCount++;
			while ((l_incompleteFrameDateTime.isEmpty() == FALSE) &&
				   (l_incompleteFrameDateTime.first() == l_currentDateTime))
			{
				l_incompleteFrameDateTime.removeFirst();
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1724", 114);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : CreateCmodFrame
// ROL : Create a frame object for the frame Datas
//
//****************************************************************************
CoDatCmodFrame*
CoDatArgosMessage::
CreateCmodFrame(
    const RWCString& p_frameDatas			// I : the extracted datas
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDatCmodFrame* ql_frame =				// The created frame
		(CoDatCmodFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Create the frame object
		//********************************************************************

		ql_frame = new CoDatCmodFrame();

		//********************************************************************
		// If the frame has been created, initialize it
		//********************************************************************

		if (ql_frame != (CoDatCmodFrame*)0)
		{
			ql_frame->Initialize(p_frameDatas);
			ql_frame->CheckChecksums();
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1780", 115);
		CO_RETHROW;
	}
	return ql_frame;
}




//****************************************************************************
//
// SPG : CreateMoveFrame
// ROL : Create a frame object for the frame Datas
//
//****************************************************************************
CoDatMoveFrame*
CoDatArgosMessage::
CreateMoveFrame(
    const CoDatPlatform& p_platform,		// I : the related platform
    const RWCString& p_frameDatas			// I : the extracted datas
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDatMoveFrame* ql_frame =				// The created frame
		(CoDatMoveFrame*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Create the frame object and initialize it
		//********************************************************************

		ql_frame = new CoDatMoveFrame();
		ql_frame->Initialize(p_frameDatas);
		ql_frame->SetVersion(p_platform.GetAttribute("PR_VERSION").GetDoubleValue());
		ql_frame->CheckCrc();
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1831", 116);
		CO_RETHROW;
	}
	return ql_frame;
}




//****************************************************************************
//
// SPG : ExtractNextFrameDatas
// ROL : Extract the next Provor Frame Datas, starting at the given position
//
//****************************************************************************
int
CoDatArgosMessage::
ExtractNextFrameDatas(
	int& p_startingPosition,				// IO : starting position for extraction
    RWCString& p_frameDatas,				// O : the extracted datas
	RWTime& p_frameDateTime					// O : the frame's emitting date/time
	) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_success = 0;						// success return flag
	unsigned int l_year = 0;				// year for the emitting date/time
	unsigned int l_month = 0;				// month for the emitting date/time
	unsigned int l_day = 0;					// day for the emitting date/time
	unsigned int l_hour = 0;				// hour for the emitting date/time
	unsigned int l_minute = 0;				// minute for the emitting date/time
	unsigned int l_second = 0;				// second for the emitting date/time

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Reset the output frame datas
		//********************************************************************

		p_frameDatas = "";

		//********************************************************************
		// Skip white chars and check for the end of the message
		//********************************************************************

		if ((SkipWhites(p_startingPosition) == 1) ||
			(p_startingPosition + CoDat_k_frameHeaderLength > _message.length()))
		{
			l_success = 0;
		}

		//********************************************************************
		// If the end of the message is not reached
		//********************************************************************

		else
		{
			//****************************************************************
			// Retrieve the emitting date/time
			//****************************************************************

			l_year = atoi(_message.data() + p_startingPosition);
			l_month = atoi(_message.data() + p_startingPosition + 5);
			l_day = atoi(_message.data() + p_startingPosition + 8);
			l_hour = atoi(_message.data() + p_startingPosition + 11);
			l_minute = atoi(_message.data() + p_startingPosition + 14);
			l_second = atoi(_message.data() + p_startingPosition + 17);
			p_frameDateTime = RWTime(RWDate(l_day, l_month, l_year),
									 l_hour, l_minute, l_second);
			p_startingPosition += CoDat_k_frameHeaderLength;

			//****************************************************************
			// Skip header and white chars, and check for the end of the message
			//****************************************************************

			if (SkipWhites(p_startingPosition) == 1)
			{
				l_success = 0;
			}

			//****************************************************************
			// There is datas to read
			//****************************************************************

			else
			{
				//************************************************************
				// while the end of the message is not reached and the beginning
				// of a new frame is not detected
				//************************************************************

				while ((SkipWhites(p_startingPosition) != 1) &&
					   ((p_startingPosition + 4 >= _message.length()) ||
					    (_message[p_startingPosition + 4] != '-')))
				{
					//********************************************************
					// Add the datas to the frame and advance to next char
					//********************************************************
				
					p_frameDatas.append(_message[p_startingPosition++]);
				}
				l_success = 1;
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 1949", 117);
		CO_RETHROW;
	}
	return l_success;
}




//****************************************************************************
//
// SPG : SkipWhites
// ROL : Increments the given message index for each white char encoutered.
//		 Return 1 if the end of message is reached, 0 else
//
//****************************************************************************
int
CoDatArgosMessage::
SkipWhites(
	int& p_index							// IO : index to increment
	) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_endOfMessage = 0;					// end of message is reached

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Skip white chars
		//********************************************************************

		while ((p_index < _message.length()) &&
			   ((_message[p_index] == ' ') ||
				(_message[p_index] == '\n') ||
				(_message[p_index] == '\r') ||
				(_message[p_index] == '\t')))
		{
			p_index++;
		}

		//********************************************************************
		// Check for the end of the message
		//********************************************************************

		if (p_index < _message.length())
		{
			l_endOfMessage = 0;
		}
		else
		{
			l_endOfMessage = 1;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatArgosMessage.cpp 1.24 2016", 118);
		CO_RETHROW;
	}
	return l_endOfMessage;
}












