//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoDatMetoceanFrame 
// ROL : This class is the base class for all the classes witch implements
//       a Metocean frame. A Metocean frame is initialized with the informations
//       contained in the body of an Argos message. The structure of a
//       Metocean frame is :
//       [0-7] : The crc value
//       [8-15] : The frame number
//       [16-255] : The frame datas (depending of the frame type)
//
// CRE : 21/08/2002
// AUT : ST
// VER : @(#)1.5
//       @(#)04/01/14
//
//****************************************************************************
// 
// HST : 21/08/2002 ST  Creation
//		 11/10/2002 ST  Coriolis V2 compatibility
//		 17/12/2002 ST  Fra 135 : Use the salinity to determine if a record is valid
//		 18/03/2003 ST  New formula to compute the pressure offset
//		 14/01/2004 ST  Manage Metocean 1.6
//
//****************************************************************************
//
// Copyright, IFREMER, 2002
//
//****************************************************************************
static const char CoDatMetoceanFrame_IDENT[] = "@(#) CORIOLIS : CoDatMetoceanFrame.cpp 1.5 04/01/14";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoDatMetoceanFrame_IN_
#include <CoDatMetoceanFrame.h>
#undef  _CoDatMetoceanFrame_IN_
#include <CoDatAttribute.h>
#include <CoDatMeasure.h>
#include <CoDatPhysicalParameterCodes.h>
#include <CoDatRecord.h>
#include <CoDatTechnicalStatus.h>
#include <CoStcError.h>
#include <rw/rwdate.h>
#include <stdlib.h>



//----------------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------------
const int CoDat_k_20bitsArgosIdFrameLength = 256;
const int CoDat_k_28bitsArgosIdFrameLength = 248;
const unsigned int CoDat_k_crc16Polynomial = 69665;




//****************************************************************************
//
// SPG : CoDatMetoceanFrame
// ROL : Constructor
//
//****************************************************************************
CoDatMetoceanFrame::
CoDatMetoceanFrame(
    )
    : CoDatDataFrame()
	, _bitVec()
	, _version(0.0)
	, _isCrcOk(FALSE)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : ~CoDatMetoceanFrame
// ROL : Destructor
//
//****************************************************************************
CoDatMetoceanFrame::
~CoDatMetoceanFrame(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : Initialize
// ROL : Initialize the instance with a 64 bytes string containing the
//		 ASCII hexa values of the 256 bits frame
//
//****************************************************************************
void
CoDatMetoceanFrame::
Initialize(
    const RWCString& p_datas			// I : the data string
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWCString l_dataAdjusted = "";		// The data string, eventually adjusted

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Copy the data string and eventually adjust it's length
		//********************************************************************

		l_dataAdjusted = p_datas;
		if (l_dataAdjusted.length() < CoDatMetoceanFrame_k_maxLength / 4)
		{
			l_dataAdjusted += RWCString('0', CoDatMetoceanFrame_k_maxLength / 4 - l_dataAdjusted.length());
		}

		//********************************************************************
		// Fill the bit vector of the datas
		//********************************************************************

		_bitVec.SetValues(l_dataAdjusted);

		//********************************************************************
		// After initialization, the instance is valid
		//********************************************************************

		_isBad = FALSE;
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 173", 1800);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : CheckCrc
// ROL : Check if the 8 bits CRC of the frame is good
//
//****************************************************************************
RWBoolean
CoDatMetoceanFrame::
CheckCrc(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoStcTBitVec<CoDatMetoceanFrame_k_maxLength> l_copy;	// Copy of the frame's bit vector
	int l_counter = 0;					// loop counter
	unsigned int l_crc = 0;				// the calculated CRC

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Initialize the copy of the frame's bit vector
		//********************************************************************

		l_copy = _bitVec;
		l_copy.ClearBits(0, 16);

		//********************************************************************
		// While the end of the bit vector is not reached
		//********************************************************************

		for (l_counter = 0, l_crc = 0; l_counter < CoDatMetoceanFrame_k_maxLength + 16;)
		{
			//****************************************************************
			// While the end of the bit vector is not reached and the crc is
			// less than the polynomial
			//****************************************************************

			while ((l_counter < CoDatMetoceanFrame_k_maxLength + 16) &&
				   (l_crc < 0x10000))
			{
				l_crc <<= 1;
				if ((l_counter < CoDatMetoceanFrame_k_maxLength) &&
					(l_copy.TestBit(l_counter) == TRUE))
				{
					l_crc++;
				}
				l_counter++;
			}

			//****************************************************************
			// if the crc is greater than the polynomial, substract
			//****************************************************************

			if (l_crc >= 0x10000)
			{
				l_crc ^= CoDat_k_crc16Polynomial;
			}
		}

		//********************************************************************
		// Get the 16 low bits value of the crc
		//********************************************************************

		l_crc &= 0xFFFF;
		l_crc &= 0x00FF;

		//********************************************************************
		// Check if the calculated CRC match the frame's CRC
		//********************************************************************

		if (l_crc == _bitVec.GetUInt(0, 8))
		{
			_isCrcOk = TRUE;
		}
		else
		{
			_isCrcOk = FALSE;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 273", 1801);
		CO_RETHROW;
	}
	return _isCrcOk;
}




//****************************************************************************
//
// SPG : GetFrameNumber
// ROL : Retrieve the number of the frame
//
//****************************************************************************
unsigned int
CoDatMetoceanFrame::
GetFrameNumber(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	unsigned int l_number = 0;			// the frame number

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the frame's type
		//********************************************************************

		if ((_version == 1.0) || (_version == 1.6))
		{
			l_number = _bitVec.GetUInt(8, 8);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 321", 1802);
		CO_RETHROW;
	}
	return l_number;
}




//****************************************************************************
//
// SPG : GetFloatTime
// ROL : Retrieve the float time
//
//****************************************************************************
RWTime
CoDatMetoceanFrame::
GetFloatTime(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWTime l_time = (unsigned long)0;		// the float time

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve the float time
		//********************************************************************

		if (_version == 1.0)
		{
			l_time = RWTime(_bitVec.GetUInt(88, 8),
							_bitVec.GetUInt(96, 8),
							_bitVec.GetUInt(104, 8));
		}
		else if (_version == 1.6)
		{
			l_time = RWTime(_bitVec.GetUInt(88, 5),
							_bitVec.GetUInt(93, 6),
							_bitVec.GetUInt(99, 6));
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 377", 1803);
		CO_RETHROW;
	}
	return l_time;
}




//****************************************************************************
//
// SPG : GetRecords
// ROL : Get the records contained in the frame
//
//****************************************************************************
void
CoDatMetoceanFrame::
GetRecords(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Decod the records
		//********************************************************************

		DecodRecords(p_records);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 421", 1804);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : GetTechnicalStatus
// ROL : Get the data contained in a technical frame and fill the technical status
//
//****************************************************************************
void
CoDatMetoceanFrame::
GetTechnicalStatus(
	int p_timeCorrection,					// I : The time correction to apply
	CoDatTechnicalStatus& p_status			// O : The technical status to fill
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Decod the technical status
		//********************************************************************

		DecodTechnicalStatus(p_status, p_timeCorrection);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 465", 1805);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Recombined
// ROL : Reconstruct the frame by doing statistics on bits of the frames in
//		 the parameter's list
//
//****************************************************************************
void
CoDatMetoceanFrame::
Recombined(
    const RWTPtrSlist<CoDatMetoceanFrame>& p_list	// I : list of frames
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_firstFrame = 0;					// Index of the first frame to use
	int l_bitCounter = 0;					// loop counter
	int l_true = 0;							// bit status counter
	int l_false = 0;						// bit status counter
	int l_frameCounter = 0;					// loop counter

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Find the index of the first frame to use (If there is an even number
		// of frames, the first one is not used
		//********************************************************************

		l_firstFrame = 1 - (p_list.entries() % 2);

		//********************************************************************
		// For each bit in the frame
		//********************************************************************

		for (l_bitCounter = 0; l_bitCounter < CoDatMetoceanFrame_k_maxLength; l_bitCounter++)
		{
			//****************************************************************
			// Reset the counters of bit status
			//****************************************************************

			l_true = 0;
			l_false = 0;

			//****************************************************************
			// For each frame, count the number of true bits and the number
			// of false bits
			//****************************************************************

			for (l_frameCounter = l_firstFrame; l_frameCounter < p_list.entries(); l_frameCounter++)
			{
				if (p_list.at(l_frameCounter)->_bitVec.TestBit(l_bitCounter) == TRUE)
				{
					l_true++;
				}
				else
				{
					l_false++;
				}
			}

			//****************************************************************
			// set the bit status using the statistics result
			//****************************************************************

			if (l_true > l_false)
			{
				_bitVec.SetBit(l_bitCounter);
			}
			else
			{
				_bitVec.ClearBit(l_bitCounter);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 560", 1806);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : GetValidFrameLength
// ROL : Get the valid frame length (number of bits), depending of the version.
//
//****************************************************************************
/*static*/ int
CoDatMetoceanFrame::
GetValidFrameLength(
	double p_version						// I : the profiler version
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_validFrameLength = 0;				// Ther valid frame length

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	//************************************************************************
	// Depending of the version
	//************************************************************************

	if (p_version == 1.0)
	{
		l_validFrameLength = CoDat_k_20bitsArgosIdFrameLength;
	}
	else if (p_version == 1.6)
	{
		l_validFrameLength = CoDat_k_28bitsArgosIdFrameLength;
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    return l_validFrameLength;
}




//****************************************************************************
//
// SPG : CheckFrameDatasLength
// ROL : Check the length of the frame datas (ascii hex values)
//
//****************************************************************************
/*static*/ RWBoolean
CoDatMetoceanFrame::
CheckFrameDatasLength(
    const RWCString& p_datas,				// I : the frame datas
	double p_version						// I : the profiler version
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWBoolean l_isValid = FALSE;			// The frame length is valid ?

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	//************************************************************************
	// Compare to the valid frame length
	//************************************************************************

	if (p_datas.length() == GetValidFrameLength(p_version) / 4)
	{
		l_isValid = TRUE;
	}
	else
	{
		l_isValid = FALSE;
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    return l_isValid;
}




//****************************************************************************
//
// SPG : GetFrameNumberFromAsciiHexDatas
// ROL : Return the frame number as an int from the ascii frame datas
//
//****************************************************************************
/*static*/ int
CoDatMetoceanFrame::
GetFrameNumberFromAsciiHexDatas(
    const RWCString& p_datas,				// I : the data string
	double p_version						// I : the profiler version
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	char ql_buffer[3];					// temp buffer
	int l_number = 0;					// the frame number

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    //************************************************************************
    // Depending of the version
    //************************************************************************

	if ((p_version == 1.0) || (p_version == 1.6))
	{
		ql_buffer[0] = p_datas[2];
		ql_buffer[1] = p_datas[3];
	}
	ql_buffer[2] = '\0';
	l_number = strtol(ql_buffer, (char**)0, 16);

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    return l_number;
}




//****************************************************************************
//
// SPG : CompareFrameNumber
// ROL : Compare two Metocean frames number
//		 -1 if 1st < 2nd
//		 0 if 1st = 2nd
//		 1 if 1st > 2nd
//
//****************************************************************************
/*static*/ int
CoDatMetoceanFrame::
CompareFrameNumber(
    void* qp_1stFrame,						// I : the first frame
    void* qp_2ndFrame,						// I : the second frame
    void*									// N : not used
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_1stFrameNumber = '\0';			// the first frame's number
	int l_2ndFrameNumber = '\0';			// the second frame's number
	int l_compare = 0;						// the comparison result

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Retrieve each frame's number
		//********************************************************************

		l_1stFrameNumber = ((CoDatMetoceanFrame*)qp_1stFrame)->GetFrameNumber();
		l_2ndFrameNumber = ((CoDatMetoceanFrame*)qp_2ndFrame)->GetFrameNumber();

		//********************************************************************
		// And compare them
		//********************************************************************

		if (l_1stFrameNumber < l_2ndFrameNumber)
		{
			l_compare = -1;
		}
		else if (l_1stFrameNumber == l_2ndFrameNumber)
		{
			l_compare = 0;
		}
		else
		{
			l_compare = 1;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 767", 1807);
		CO_RETHROW;
	}
	return l_compare;
}




//****************************************************************************
//
// SPG : DecodRecords
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatMetoceanFrame::
DecodRecords(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if (_version == 1.0)
		{
			DecodRecordsV10(p_records);
		}
		else if (_version == 1.6)
		{
			DecodRecordsV16(p_records);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 818", 1808);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV10
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatMetoceanFrame::
DecodRecordsV10(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	int l_field;							// a bitfield value
	double l_temperatureValue = 0.0;		// the current temperature value
	CoDatMeasure l_temperature;				// the temperature measure;
	double l_salinityValue = 0.0;			// the current salinity value
	CoDatMeasure l_salinity;				// the salinity measure;
	double l_pressureValue = 0.0;			// the current pressure value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit and the number of records
		//********************************************************************

		if (GetFrameNumber() == 1)
		{
			l_position = 208;
			l_recordCount = 1;
		}
		else
		{
			l_position = 16;
			l_recordCount = 5;
		}

		//********************************************************************
		// For each record to decod
		//********************************************************************

		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			//****************************************************************
			// Decod the temperature
			//****************************************************************

			l_temperatureValue = double(_bitVec.GetUInt(l_position, 16)) / 1000.0 - 5.0;
			l_position += 16;
			l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue);

			//****************************************************************
			// Decod the salinity
			//****************************************************************

			l_salinityValue = double(_bitVec.GetUInt(l_position, 16)) / 1000.0 + 25.0;
			l_position += 16;
			l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue);

			//****************************************************************
			// Decod the pressure
			//****************************************************************

			l_pressureValue = double(_bitVec.GetUInt(l_position, 16)) / 10.0;
			l_position += 16;
			l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue);

			//****************************************************************
			// If the salinity is decoded, create the records
			//****************************************************************

			if (l_salinityValue != 25.0)
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_salinity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 933", 1809);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV16
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatMetoceanFrame::
DecodRecordsV16(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	int l_field;							// a bitfield value
	double l_temperatureValue = 0.0;		// the current temperature value
	CoDatMeasure l_temperature;				// the temperature measure;
	double l_salinityValue = 0.0;			// the current salinity value
	CoDatMeasure l_salinity;				// the salinity measure;
	double l_pressureValue = 0.0;			// the current pressure value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit and the number of records
		//********************************************************************

		if (GetFrameNumber() == 1)
		{
			l_position = 201;
			l_recordCount = 1;
		}
		else
		{
			l_position = 16;
			l_recordCount = 5;
		}

		//********************************************************************
		// For each record to decod
		//********************************************************************

		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			//****************************************************************
			// Decod the temperature
			//****************************************************************

			l_temperatureValue = double(_bitVec.GetUInt(l_position, 16)) / 1000.0 - 5.0;
			l_position += 16;
			l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue);

			//****************************************************************
			// Decod the salinity
			//****************************************************************

			l_salinityValue = double(_bitVec.GetUInt(l_position, 15)) / 1000.0 + 25.0;
			l_position += 15;
			l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue);

			//****************************************************************
			// Decod the pressure
			//****************************************************************

			l_pressureValue = double(_bitVec.GetUInt(l_position, 15)) / 10.0;
			l_position += 15;
			l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue);

			//****************************************************************
			// If the salinity is decoded, create the records
			//****************************************************************

			if (l_salinityValue != 25.0)
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_salinity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 1048", 1810);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatus
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatMetoceanFrame::
DecodTechnicalStatus(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if (_version == 1.0)
		{
			DecodTechnicalStatusV10(p_status, p_timeCorrection);
		}
		else if (_version == 1.6)
		{
			DecodTechnicalStatusV16(p_status, p_timeCorrection);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 1099", 1811);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV10
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatMetoceanFrame::
DecodTechnicalStatusV10(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	RWTime l_dateTimeValue =				// A date-time value
		(unsigned long)0;
	double l_doubleValue = 0.0;				// A double value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// DESFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(16, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_Surface_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(24, 8);
		p_status.AddAttribute(CoDatAttribute("DESFC_Surface_valve", l_intValue));

		//********************************************************************
		// DESFC_1st_stab_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(32, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_Depth_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(40, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_valve", l_intValue));

		//********************************************************************
		// DESFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(44, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_pump", l_intValue));

		//********************************************************************
		// DESFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(48, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_End_time", l_dateTimeValue));

		//********************************************************************
		// IMMFC_Depth_correct
		//********************************************************************

		l_intValue = _bitVec.GetUInt(56, 8);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Depth_correct", l_intValue));

		//********************************************************************
		// ASCFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(64, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_End_time", l_dateTimeValue));

		//********************************************************************
		// ASCFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(72, 8);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Depth_pump", l_intValue));

		//********************************************************************
		// ASCFC_Surface_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(80, 8);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Surface_pump", l_intValue));

		//********************************************************************
		// TIMEC_Internal_time
		//********************************************************************

		l_dateTimeValue = RWTime(RWDate(),
								 _bitVec.GetUInt(88, 8),
								 _bitVec.GetUInt(96, 8),
								 _bitVec.GetUInt(104, 8));
		p_status.AddAttribute(CoDatAttribute("TIMEC_Internal_time", l_dateTimeValue));

		//********************************************************************
		// SENSO_Press_offset
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(112, 10)) / 10.0 - 51.2;
		p_status.AddAttribute(CoDatAttribute("SENSO_Press_offset", l_doubleValue));

		//********************************************************************
		// SENSO_Int_vaccum
		//********************************************************************

		l_intValue = int(_bitVec.GetUInt(122, 6)) * 5 + 700;
		p_status.AddAttribute(CoDatAttribute("SENSO_Int_vaccum", l_intValue));

		//********************************************************************
		// TRANS_APROF_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(128, 8);
		p_status.AddAttribute(CoDatAttribute("TRANS_APROF_Emitted", l_intValue));

		//********************************************************************
		// ATEMP_Internal_count
		//********************************************************************

		l_intValue = _bitVec.GetUInt(136, 8);
		p_status.AddAttribute(CoDatAttribute("ATEMP_Internal_count", l_intValue));

		//********************************************************************
		// TRANS_IDRIFT_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(144, 8);
		p_status.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Emitted", l_intValue));

		//********************************************************************
		// IMMFC_Sample_period
		//********************************************************************

		l_intValue = _bitVec.GetUInt(152, 8);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Sample_period", l_intValue));

		//********************************************************************
		// ITEMP_Internal_count
		//********************************************************************

		l_intValue = _bitVec.GetUInt(160, 8);
		p_status.AddAttribute(CoDatAttribute("ITEMP_Internal_count", l_intValue));

		//********************************************************************
		// IMMFC_Start_day
		//********************************************************************

		l_intValue = _bitVec.GetUInt(168, 11);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Start_day", l_intValue));

		//********************************************************************
		// IMMFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(179, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("IMMFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// SENSO_7V_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(187, 6)) * 0.1 + 4;
		p_status.AddAttribute(CoDatAttribute("SENSO_7V_BattVolt", l_doubleValue));

		//********************************************************************
		// SENSO_14V_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(193, 7)) * 0.1 + 8;
		p_status.AddAttribute(CoDatAttribute("SENSO_14V_BattVolt", l_doubleValue));

		//********************************************************************
		// CYCLE_NUMBER
		//********************************************************************

		l_intValue = _bitVec.GetUInt(200, 8);
		p_status.AddAttribute(CoDatAttribute("CYCLE_NUMBER", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 1311", 1812);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV16
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatMetoceanFrame::
DecodTechnicalStatusV16(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	RWTime l_dateTimeValue =				// A date-time value
		(unsigned long)0;
	double l_doubleValue = 0.0;				// A double value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// DESFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(16, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_Surface_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(24, 8);
		p_status.AddAttribute(CoDatAttribute("DESFC_Surface_valve", l_intValue));

		//********************************************************************
		// DESFC_1st_stab_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(32, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_Depth_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(40, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_valve", l_intValue));

		//********************************************************************
		// DESFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(44, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_pump", l_intValue));

		//********************************************************************
		// DESFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(48, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_End_time", l_dateTimeValue));

		//********************************************************************
		// IMMFC_Depth_correct
		//********************************************************************

		l_intValue = _bitVec.GetUInt(56, 8);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Depth_correct", l_intValue));

		//********************************************************************
		// ASCFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(64, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_End_time", l_dateTimeValue));

		//********************************************************************
		// ASCFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(72, 8);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Depth_pump", l_intValue));

		//********************************************************************
		// ASCFC_Surface_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(80, 8);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Surface_pump", l_intValue));

		//********************************************************************
		// TIMEC_Internal_time
		//********************************************************************

		l_dateTimeValue = RWTime(RWDate(),
								 _bitVec.GetUInt(88, 5),
								 _bitVec.GetUInt(93, 6),
								 _bitVec.GetUInt(99, 6));
		p_status.AddAttribute(CoDatAttribute("TIMEC_Internal_time", l_dateTimeValue));

		//********************************************************************
		// SENSO_Press_offset
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(105, 10)) / 10.0 - 51.2;
		p_status.AddAttribute(CoDatAttribute("SENSO_Press_offset", l_doubleValue));

		//********************************************************************
		// SENSO_Int_vaccum
		//********************************************************************

		l_intValue = int(_bitVec.GetUInt(115, 6)) * 5 + 700;
		p_status.AddAttribute(CoDatAttribute("SENSO_Int_vaccum", l_intValue));

		//********************************************************************
		// TRANS_APROF_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(121, 8);
		p_status.AddAttribute(CoDatAttribute("TRANS_APROF_Emitted", l_intValue));

		//********************************************************************
		// ATEMP_Internal_count
		//********************************************************************

		l_intValue = _bitVec.GetUInt(129, 8);
		p_status.AddAttribute(CoDatAttribute("ATEMP_Internal_count", l_intValue));

		//********************************************************************
		// TRANS_IDRIFT_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(137, 8);
		p_status.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Emitted", l_intValue));

		//********************************************************************
		// IMMFC_Sample_period
		//********************************************************************

		l_intValue = _bitVec.GetUInt(145, 8);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Sample_period", l_intValue));

		//********************************************************************
		// ITEMP_Internal_count
		//********************************************************************

		l_intValue = _bitVec.GetUInt(153, 8);
		p_status.AddAttribute(CoDatAttribute("ITEMP_Internal_count", l_intValue));

		//********************************************************************
		// IMMFC_Start_day
		//********************************************************************

		l_intValue = _bitVec.GetUInt(161, 11);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Start_day", l_intValue));

		//********************************************************************
		// IMMFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(172, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("IMMFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// SENSO_7V_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(180, 6)) * 0.1 + 4;
		p_status.AddAttribute(CoDatAttribute("SENSO_7V_BattVolt", l_doubleValue));

		//********************************************************************
		// SENSO_14V_BattVolt
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(186, 7)) * 0.1 + 8;
		p_status.AddAttribute(CoDatAttribute("SENSO_14V_BattVolt", l_doubleValue));

		//********************************************************************
		// CYCLE_NUMBER
		//********************************************************************

		l_intValue = _bitVec.GetUInt(193, 8);
		p_status.AddAttribute(CoDatAttribute("CYCLE_NUMBER", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatMetoceanFrame.cpp 1.5 1523", 1813);
		CO_RETHROW;
	}
}




