//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoDatProvorFrame 
// ROL : This class is the base class for all the classes witch implements
//		 a Provor frame. A Provor frame is initialized with the informations
//		 contained in the body of an Argos message. The structure of a
//		 Provor frame is :
//		 [0-3] : The frame type
//		 [4-19] : The crc value (special crc-8 or crc-ccitt)
//		 [20-255] : The frame datas (depending of the frame type)
//
// CRE : 28/08/1999
// AUT : ST
// VER : @(#)1.12
//       @(#)04/03/02
//
//****************************************************************************
// 
// HST : 28/08/1999 ST  Creation
//		 23/09/1999 ST  FAE-002 : Review of the CRC-16 computation
//		 27/08/2002 ST  Emitting date/time and satellite are managed by DataFrame
//		 11/10/2002 ST  Coriolis V2 compatibility
//		 24/01/2003 ST  FDI 114 : Provor 3.81
//		 22/04/2003 ST  Remove an unexpected dead line in GetRecords
//		 14/10/2003 ST  FDI 145 : Provor 4.1
//		 02/03/2004 ST  Manage Provor 4.11
//
//****************************************************************************
//
// Copyright, IFREMER, 1999
//
//****************************************************************************
static const char CoDatProvorFrame_IDENT[] = "@(#) CORIOLIS : CoDatProvorFrame.cpp 1.12 04/03/02";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoDatProvorFrame_IN_
#include <CoDatProvorFrame.h>
#undef  _CoDatProvorFrame_IN_
#include <CoDatAttribute.h>
#include <CoDatMeasure.h>
#include <CoDatPhysicalParameterCodes.h>
#include <CoDatRecord.h>
#include <CoDatTechnicalStatus.h>
#include <CoStcCatManager.h>
#include <CoStcError.h>
#include <CoStcTools.h>
#include <rw/rwdate.h>
#include <iomanip.h>
#include <iostream.h>



//----------------------------------------------------------------------------
// CONSTANTS
//----------------------------------------------------------------------------
const int CoDat_k_frameDatasLength = 64;
const unsigned int CoDat_k_crc16Polynomial = 69665;




//****************************************************************************
//
// SPG : CoDatProvorFrame
// ROL : Constructor
//
//****************************************************************************
CoDatProvorFrame::
CoDatProvorFrame(
    )
    : CoDatDataFrame()
    , _bitVec()
	, _version(0.0)
    , _isCrcOk()
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : ~CoDatProvorFrame
// ROL : Destructor
//
//****************************************************************************
CoDatProvorFrame::
~CoDatProvorFrame(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}




//****************************************************************************
//
// SPG : Initialize
// ROL : Initialize the instance with the ascii frame datas
//
//****************************************************************************
void
CoDatProvorFrame::
Initialize(
    const RWCString& p_datas			// I : the data string
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Fill the bit vector of the datas
		//********************************************************************

		_bitVec.SetValues(p_datas);

		//********************************************************************
		// After initialization, the instance is valid
		//********************************************************************

		_isBad = FALSE;
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 165", 401);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : Check8BitsCrc
// ROL : Check if the 8 bits CRC of the frame is good
//
//****************************************************************************
RWBoolean
CoDatProvorFrame::
Check8BitsCrc(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoStcTBitVec<CoDatProvorFrame_k_length> l_copy;	// Copy of the frame's bit vector
	int l_counter = 0;						// loop counter
	unsigned char l_value = 0;				// 8-bits-wide value
	unsigned char l_crc = 0;				// the calculated CRC
	unsigned char l_ax = 0;					// counter

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Initialize the copy of the frame's bit vector
		//********************************************************************

		l_copy = _bitVec;
		l_copy.ClearBits(4, 16);

		//********************************************************************
		// For each 8-bits-wide value of the frame
		//********************************************************************

		for (l_counter = 0; l_counter < CoDatProvorFrame_k_length; l_counter += 8)
		{
			//****************************************************************
			// Retrieve the 8-bits-wide value
			//****************************************************************

			l_value = l_copy.GetUInt(l_counter, 8);

			//****************************************************************
			// Compute the CRC
			//****************************************************************

			l_crc = l_crc ^ l_value;
			if (l_crc == 0x00)
			{
				l_crc = 0x7F;
			}
			else
			{
				l_ax = 0;
				if (l_crc & 0x01)
				{
					l_ax++;
				}
				if (l_crc & 0x04)
				{
					l_ax++;
				}
				if (l_crc & 0x08)
				{
					l_ax++;
				}
				if (l_crc & 0x10)
				{
					l_ax++;
				}
				l_crc >>= 1;
				if (l_ax & 0x01)
				{
					l_crc += 0x80;
				}
			}
		}

		//********************************************************************
		// Check if the calculated CRC match the frame's CRC
		//********************************************************************

		if (l_crc == _bitVec.GetUInt(4, 8))
		{
			_isCrcOk = TRUE;
		}
		else
		{
			_isCrcOk = FALSE;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 275", 402);
		CO_RETHROW;
	}
	return _isCrcOk;
}




//****************************************************************************
//
// SPG : Check16BitsCrc
// ROL : Check if the 16 bits CRC of the frame is good
//
//****************************************************************************
RWBoolean
CoDatProvorFrame::
Check16BitsCrc(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_crcPosition = 0;					// the start position of the crc
	CoStcTBitVec<CoDatProvorFrame_k_length> l_copy;	// Copy of the frame's bit vector
	int l_counter = 0;						// loop counter
	unsigned int l_crc = 0;					// the calculated CRC

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Compute the crc start position
		//********************************************************************

		if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
			(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
			(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
			(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
		{
			l_crcPosition = 4;
		}
		else if (_version == 4.0)
		{
			l_crcPosition = 8;
		}

		//********************************************************************
		// Initialize the copy of the frame's bit vector
		//********************************************************************

		l_copy = _bitVec;
		l_copy.ClearBits(l_crcPosition, 16);

		//********************************************************************
		// While the end of the bit vector is not reached
		//********************************************************************

		for (l_counter = 0, l_crc = 0; l_counter < CoDatProvorFrame_k_length + 16;)
		{
			//****************************************************************
			// While the end of the bit vector is not reached and the crc is
			// less than the polynomial
			//****************************************************************

			while ((l_counter < CoDatProvorFrame_k_length + 16) && (l_crc < 0x10000))
			{
				l_crc <<= 1;
				if ((l_counter < CoDatProvorFrame_k_length) &&
					(l_copy.TestBit(l_counter) == TRUE))
				{
					l_crc++;
				}
				l_counter++;
			}

			//****************************************************************
			// if the crc is greater than the polynomial, substract
			//****************************************************************

			if (l_crc >= 0x10000)
			{
				l_crc ^= CoDat_k_crc16Polynomial;
			}
		}

		//********************************************************************
		// Get the 16 low bits value of the crc
		//********************************************************************

		l_crc &= 0xFFFF;

		//********************************************************************
		// Check if the calculated CRC match the frame's CRC
		//********************************************************************

		if (l_crc == _bitVec.GetUInt(l_crcPosition, 16))
		{
			_isCrcOk = TRUE;
		}
		else
		{
			_isCrcOk = FALSE;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 391", 403);
		CO_RETHROW;
	}
	return _isCrcOk;
}




//****************************************************************************
//
// SPG : GetType
// ROL : Retrieve the type of the frame
//
//****************************************************************************
unsigned int
CoDatProvorFrame::
GetType(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	unsigned int l_type = 0;			// the frame's type

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve the frame's type
		//********************************************************************

		if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
			(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
			(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
			(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
		{
			l_type = _bitVec.GetUInt(0, 4);
		}
		else if (_version == 4.0)
		{
			l_type = _bitVec.GetUInt(24, 2);
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 446", 404);
		CO_RETHROW;
    }
    return l_type;
}




//****************************************************************************
//
// SPG : GetRelativeDating
// ROL : Retrieve a relative dating value which can be use for frame's comparison
//
//****************************************************************************
unsigned int
CoDatProvorFrame::
GetRelativeDating(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	unsigned int l_relativeDating = 0;		// The frame relative dating

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
			(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
			(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
			(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
		{
			if ((GetType() == 1) || (GetType() == 3) || (GetType() == 4) || (GetType() == 6))
			{
				l_relativeDating = _bitVec.GetUInt(20, 9);
			}
			else
			{
				l_relativeDating = _bitVec.GetUInt(20, 11);
			}
		}
		else if (_version == 4.0)
		{
			if (GetType() == 2)
			{
				l_relativeDating = _bitVec.GetUInt(33, 11);
			}
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 511", 405);
		CO_RETHROW;
    }
    return l_relativeDating;
}




//****************************************************************************
//
// SPG : SetRelativeDating
// ROL : Change a relative dating value which can be use for frame's comparison
//
//****************************************************************************
void
CoDatProvorFrame::
SetRelativeDating(
	unsigned int p_relativeDating			// I : the new relative dating value
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
			(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
			(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
			(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
		{
			if ((GetType() == 1) || (GetType() == 3) || (GetType() == 4) || (GetType() == 6))
			{
				_bitVec.SetUInt(20, 9, p_relativeDating);
			}
			else
			{
				_bitVec.SetUInt(20, 11, p_relativeDating);
			}
		}
		else if (_version == 4.0)
		{
			if (GetType() == 2)
			{
				_bitVec.SetUInt(33, 11, p_relativeDating);
			}
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 575", 406);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : GetFrameNumber
// ROL : Retrieve the frame number value
//
//****************************************************************************
unsigned int
CoDatProvorFrame::
GetFrameNumber(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	unsigned int l_frameNumber = 0;			// The frame number

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if (_version == 4.0)
		{
			l_frameNumber = _bitVec.GetUInt(26, 7);
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 622", 407);
		CO_RETHROW;
    }
    return l_frameNumber;
}




//****************************************************************************
//
// SPG : GetJoker
// ROL : Retrieve the joker value
//
//****************************************************************************
unsigned int
CoDatProvorFrame::
GetJoker(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	unsigned int l_joker = 0;				// The joker value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if (_version == 4.0)
		{
			if ((GetType() == 1) || (GetType() == 3))
			{
				l_joker = _bitVec.GetUInt(33, 2);
			}
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 673", 408);
		CO_RETHROW;
    }
    return l_joker;
}




//****************************************************************************
//
// SPG : GetId
// ROL : Retrieve a specific value which can be use for frame's comparison
//
//****************************************************************************
unsigned int
CoDatProvorFrame::
GetId(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	unsigned int l_id = 0;				// The id value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// For drift frames, the id is the relative dating value
		//********************************************************************

		if (GetType() == 2)
		{
			if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
				(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
				(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
				(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
			{
				l_id = _bitVec.GetUInt(20, 11);
			}
			else if (_version == 4.0)
			{
				l_id = _bitVec.GetUInt(33, 11);
			}
		}

		//********************************************************************
		// For vertical profiles
		//********************************************************************

		else
		{
			//****************************************************************
			// For versions 2.2 to 3.8, the id is the relative dating and the
			// first pressure
			//****************************************************************
			
			if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
				(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
				(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
				(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
			{
				l_id = _bitVec.GetUInt(20, 20);
			}

			//****************************************************************
			// For version 4.0, the id is the frame number and the joker
			//****************************************************************

			else
			{
				l_id = _bitVec.GetUInt(26, 9);
			}
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 760", 409);
		CO_RETHROW;
    }
    return l_id;
}




//****************************************************************************
//
// SPG : GetFirstPressure
// ROL : Retrieve the first pressure of the frame
//
//****************************************************************************
double
CoDatProvorFrame::
GetFirstPressure(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_firstPressure = 0.0;			// The frame first pressure

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
			(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
			(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
			(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
		{
			if ((GetType() == 1) || (GetType() == 3) || (GetType() == 4) || (GetType() == 6))
			{
				l_firstPressure = double(_bitVec.GetUInt(29, 11));
			}
			else
			{
				l_firstPressure = double(_bitVec.GetUInt(31, 11));
			}
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 818", 410);
		CO_RETHROW;
    }
    return l_firstPressure;
}




//****************************************************************************
//
// SPG : GetFloatTime
// ROL : Retrieve the float time
//
//****************************************************************************
RWTime
CoDatProvorFrame::
GetFloatTime(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWTime l_floatTime = (unsigned long)0;	// The float time

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if ((_version == 2.2) || (_version == 3.21))
		{
			l_floatTime = RWTime(RWDate(),
								 _bitVec.GetUInt(213, 5),
								 _bitVec.GetUInt(218, 6),
								 _bitVec.GetUInt(224, 6));
		}
		else if ((_version == 2.6) || (_version == 2.7) || (_version == 3.5) ||
				 (_version == 3.51) || (_version == 3.52) || (_version == 3.61))
		{
			l_floatTime = RWTime(RWDate(),
								 _bitVec.GetUInt(182, 5),
								 _bitVec.GetUInt(187, 6),
								 _bitVec.GetUInt(193, 6));
		}
		else if ((_version == 3.8) || (_version == 3.81) || (_version == 4.1) ||
				 (_version == 4.11))
		{
			l_floatTime = RWTime(RWDate(),
								 _bitVec.GetUInt(162, 5),
								 _bitVec.GetUInt(167, 6),
								 _bitVec.GetUInt(173, 6));
		}
		else if (_version == 4.0)
		{
			l_floatTime = RWTime(RWDate(),
								 _bitVec.GetUInt(177, 5),
								 _bitVec.GetUInt(182, 6),
								 _bitVec.GetUInt(188, 6));
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 892", 411);
		CO_RETHROW;
    }
    return l_floatTime;
}




//****************************************************************************
//
// SPG : GetSurfaceTemperatureValue
// ROL : Retrieve the surface temperature value
//
//****************************************************************************
double
CoDatProvorFrame::
GetSurfaceTemperatureValue(
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	double l_value = 0.0;					// The surface temperature value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if ((_version == 2.2) || (_version == 3.21))
		{
			l_value = double(short((unsigned short)_bitVec.GetUInt(239, 16))) / 1000.0;
		}
		else if ((_version == 2.6) || (_version == 2.7) || (_version == 3.5) ||
				 (_version == 3.51) || (_version == 3.52) || (_version == 3.61))
		{
			l_value = double(short((unsigned short)_bitVec.GetUInt(208, 16))) / 1000.0;
		}
		else if ((_version == 3.8) || (_version == 3.81) || (_version == 4.1) ||
				 (_version == 4.11))
		{
			l_value = double(short((unsigned short)_bitVec.GetUInt(188, 16))) / 1000.0;
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 950", 412);
		CO_RETHROW;
    }
    return l_value;
}




//****************************************************************************
//
// SPG : GetRecords
// ROL : Get the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
GetRecords(
	const RWTime& p_profileDateTime,		// I : Date/time of the relative profile
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDatRecord l_previousRecord;			// the previous record
	int l_recordCount = 0;					// number of records
	int l_recordCounter = 0;				// loop counter
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	CoDatMeasure l_measure;					// the current measure
	double l_value = 0.0;					// the current measure value
	RWTime l_dateTime = (unsigned long)0;	// the first record date-time

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Decod the records
		//********************************************************************

		DecodRecords(p_records);

		//********************************************************************
		// The list of records contains absolute and relative records. They
		// all should be absolute
		//********************************************************************

		l_recordCount = p_records.entries();
		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			//****************************************************************
			// Retrieve the current record
			//****************************************************************

			ql_record = p_records.at(l_recordCounter);

			//****************************************************************
			// Process the pressure
			//****************************************************************

			if (ql_record->HasMeasure(CoDat_k_ppPressure) == TRUE)
			{
				l_measure = ql_record->GetMeasure(CoDat_k_ppPressure);
				if (l_measure.GetQuality() == '5')
				{
					l_value = l_measure.GetValue();
					l_value += l_previousRecord.GetMeasure(CoDat_k_ppPressure).GetValue();
					l_measure.SetValue(l_value);
					ql_record->AddMeasure(l_measure);
				}
				l_previousRecord.AddMeasure(l_measure);
			}
			
			//****************************************************************
			// Process the temperature
			//****************************************************************

			if (ql_record->HasMeasure(CoDat_k_ppTemperature) == TRUE)
			{
				l_measure = ql_record->GetMeasure(CoDat_k_ppTemperature);
				if (l_measure.GetQuality() == '5')
				{
					l_value = l_measure.GetValue();
					l_value += l_previousRecord.GetMeasure(CoDat_k_ppTemperature).GetValue();
					l_measure.SetValue(l_value);
					ql_record->AddMeasure(l_measure);
				}
				l_previousRecord.AddMeasure(l_measure);
			}
			
			//****************************************************************
			// Process the conductivity
			//****************************************************************

			if (ql_record->HasMeasure(CoDat_k_ppConductivity) == TRUE)
			{
				l_measure = ql_record->GetMeasure(CoDat_k_ppConductivity);
				if (l_measure.GetQuality() == '5')
				{
					l_value = l_measure.GetValue();
					l_value += l_previousRecord.GetMeasure(CoDat_k_ppConductivity).GetValue();
					l_measure.SetValue(l_value);
					ql_record->AddMeasure(l_measure);
				}
				l_previousRecord.AddMeasure(l_measure);
			}
			
			//****************************************************************
			// Process the salinity
			//****************************************************************

			if (ql_record->HasMeasure(CoDat_k_ppSalinity) == TRUE)
			{
				l_measure = ql_record->GetMeasure(CoDat_k_ppSalinity);
				if (l_measure.GetQuality() == '5')
				{
					l_value = l_measure.GetValue();
					l_value += l_previousRecord.GetMeasure(CoDat_k_ppSalinity).GetValue();
					l_measure.SetValue(l_value);
					ql_record->AddMeasure(l_measure);
				}
				l_previousRecord.AddMeasure(l_measure);
			}
		}

		//********************************************************************
		// Set the first record date-time (depends of the frame type)
		//********************************************************************

		if ((GetType() == 1) || (GetType() == 3) || (GetType() == 4) || (GetType() == 6))
		{
			l_dateTime = p_profileDateTime + 60 * GetRelativeDating();
		}
		else
		{
			l_dateTime = RWTime(RWDate(p_profileDateTime),
								GetRelativeDating() % 32, 0, 0);
			l_dateTime = l_dateTime + 86400 * (GetRelativeDating() / 32);
		}
	    p_records.first()->SetDateTime(l_dateTime);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 1103", 413);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : GetTechnicalStatus
// ROL : Get the data contained in a technical frame and fill the technical status
//
//****************************************************************************
void
CoDatProvorFrame::
GetTechnicalStatus(
	int p_timeCorrection,					// I : The time correction to apply
	CoDatTechnicalStatus& p_status			// O : The technical status to fill
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Decod the technical status
		//********************************************************************

		DecodTechnicalStatus(p_status, p_timeCorrection);
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 1147", 414);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Recombined
// ROL : Reconstruct the frame by doing statistics on bits of the frames in
//		 the parameter's list
//
//****************************************************************************
void
CoDatProvorFrame::
Recombined(
    const RWTPtrSlist<CoDatProvorFrame>& p_list	// I : list of frames
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

    int l_firstFrame = 0;					// Index of the first frame to use
    int l_bitCounter = 0;					// loop counter
    int l_true = 0;							// bit status counter
    int l_false = 0;						// bit status counter
    int l_frameCounter = 0;					// loop counter

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Find the index of the first frame to use (If there is an even number
		// of frames, the first one is not used
		//********************************************************************

		l_firstFrame = 1 - (p_list.entries() % 2);

		//********************************************************************
		// For each bit in the frame
		//********************************************************************

		for (l_bitCounter = 0; l_bitCounter < CoDatProvorFrame_k_length; l_bitCounter++)
		{
			//****************************************************************
			// Reset the counters of bit status
			//****************************************************************

			l_true = 0;
			l_false = 0;

			//****************************************************************
			// For each frame, count the number of true bits and the number
			// of false bits
			//****************************************************************

			for (l_frameCounter = l_firstFrame; l_frameCounter < p_list.entries(); l_frameCounter++)
			{
				if (p_list.at(l_frameCounter)->_bitVec.TestBit(l_bitCounter) == TRUE)
				{
					l_true++;
				}
				else
				{
					l_false++;
				}
			}

			//****************************************************************
			// set the bit status using the statistics result
			//****************************************************************

			if (l_true > l_false)
			{
				_bitVec.SetBit(l_bitCounter);
			}
			else
			{
				_bitVec.ClearBit(l_bitCounter);
			}
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 1242", 415);
		CO_RETHROW;
    }
}




//****************************************************************************
//
// SPG : Trace
// ROL : Trace the frame content into the given stream
//
//****************************************************************************
void
CoDatProvorFrame::
Trace(
	ostream& p_traceStream			// I : the trace stream
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWTPtrSlist<CoDatRecord> l_records;		// the list of records
	CoDatTechnicalStatus l_technicalStatus;	// the technical status

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// If the frame is a data frame
		//********************************************************************

		if (GetType() != 0)
		{
			//****************************************************************
			// Trace the frame relative dating or the frame number
			//****************************************************************

			if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
				(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
				(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
				(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 402);
				p_traceStream << GetRelativeDating() << endl;
			}
			else if (_version == 4.0)
			{
				if (GetType() == 2)
				{
					p_traceStream << CoStc_g_catManager.GetInfo(3, 415);
					p_traceStream << GetFrameNumber() << endl;
					p_traceStream << CoStc_g_catManager.GetInfo(3, 402);
					p_traceStream << GetRelativeDating() << endl;
				}
				else
				{
					p_traceStream << CoStc_g_catManager.GetInfo(3, 415);
					p_traceStream << GetFrameNumber() << endl;
					p_traceStream << CoStc_g_catManager.GetInfo(3, 416);
					p_traceStream << GetJoker() << endl;
				}
			}

			//****************************************************************
			// If the frame has a good CRC
			//****************************************************************

			if (IsCrcOk() == TRUE)
			{
			    //****************************************************************
			    // Decod and trace the records
			    //****************************************************************

				DecodRecords(l_records);
				TraceRecords(p_traceStream, l_records);
			}

			//****************************************************************
			// If the frame has a bad CRC
			//****************************************************************

			else
			{
				//************************************************************
				// Trace the first pressure
				//************************************************************

				if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
					(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
					(_version == 3.52) || (_version == 3.61) || (_version == 3.8) ||
					(_version == 3.81) || (_version == 4.1) || (_version == 4.11))
				{
					p_traceStream << CoStc_g_catManager.GetInfo(3, 403);
					p_traceStream << int(GetFirstPressure()) << endl;
				}
			}
		}

		//********************************************************************
		// If the frame is a technical frame
		//********************************************************************

		else
		{
			//****************************************************************
			// If the frame has a good CRC
			//****************************************************************

			if (IsCrcOk() == TRUE)
			{
			    //****************************************************************
			    // Decod and trace the technical status
			    //****************************************************************

				DecodTechnicalStatus(l_technicalStatus);
				TraceTechnicalStatus(p_traceStream, l_technicalStatus);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 1374", 416);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : CheckFrameDatasLength
// ROL : Check the length of the frame datas (ascii hex values)
//
//****************************************************************************
/*static*/ RWBoolean
CoDatProvorFrame::
CheckFrameDatasLength(
    const RWCString& p_datas,				// I : the data string
	double p_version						// I : the profiler version
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	RWBoolean l_isValid = FALSE;			// The frmae length is valid ?

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	//************************************************************************
	// Depending of the version
	//************************************************************************

	if ((p_version == 2.2) || (p_version == 2.6) || (p_version == 2.7) ||
		(p_version == 3.21) || (p_version == 3.5) || (p_version == 3.51) ||
		(p_version == 3.52) || (p_version == 3.61) || (p_version == 3.8) ||
		(p_version == 3.81) || (p_version == 4.0) || (p_version == 4.1) ||
		(p_version == 4.11))
	{
		if (p_datas.length() == CoDat_k_frameDatasLength)
		{
			l_isValid = TRUE;
		}
		else
		{
			l_isValid = FALSE;
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    return l_isValid;
}




//****************************************************************************
//
// SPG : GetTypeFromAsciiHexDatas
// ROL : Return the 4-bits frame type as an int from the ascii frame datas
//
//****************************************************************************
/*static*/ int
CoDatProvorFrame::
GetTypeFromAsciiHexDatas(
    const RWCString& p_datas,				// I : the data string
	double p_version						// I : the profileur version
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	char l_charType = '\0';					// the frame type as a char
	int l_intType = -1;						// the frame type as an int

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    //************************************************************************
    // Retrieve the char containning the frame version
    //************************************************************************

	if ((p_version == 2.2) || (p_version == 2.6) || (p_version == 2.7) ||
		(p_version == 3.21) || (p_version == 3.5) || (p_version == 3.51) ||
		(p_version == 3.52) || (p_version == 3.61) || (p_version == 3.8) ||
		(p_version == 3.81) || (p_version == 4.1) || (p_version == 4.11))
	{
		l_charType = p_datas[0];
	}
	else if (p_version == 4.0)
	{
		l_charType = p_datas[6];
	}

	//************************************************************************
	// Convert it into a int value
	//************************************************************************

    if ((l_charType >= '0') && (l_charType <= '9'))
    {
		l_intType = int(l_charType - '0');
    }
    else if ((l_charType >= 'A') && (l_charType <= 'F'))
    {
		l_intType = int(l_charType - 'A' + 10);
    }
    else if ((l_charType >= 'a') && (l_charType <= 'f'))
    {
		l_intType = int(l_charType - 'a' + 10);
    }

	//************************************************************************
	// And eventually mask it
	//************************************************************************

	if (p_version == 4.0)
	{
		l_intType = l_intType / 4;
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    return l_intType;
}




//****************************************************************************
//
// SPG : CompareRelativeDating
// ROL : Compare two Provor frames relative dating and returns :
//		 -1 if 1st < 2nd
//		 0 if 1st = 2nd
//		 1 if 1st > 2nd
//
//****************************************************************************
/*static*/ int
CoDatProvorFrame::
CompareRelativeDating(
	void* qp_1stFrame,						// I : the first frame
	void* qp_2ndFrame,						// I : the second frame
	void* qp_period							// I : the period as void*
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

    unsigned int l_1stFrameRelativeDating = 0;	// the first frame's relative dating
    unsigned int l_2ndFrameRelativeDating = 0;	// the second frame's relative dating
    int l_compare = 0;						// the comparison result

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve each frame's relative dating
		//********************************************************************

		l_1stFrameRelativeDating = ((CoDatProvorFrame*)qp_1stFrame)->GetRelativeDating();
		l_2ndFrameRelativeDating = ((CoDatProvorFrame*)qp_2ndFrame)->GetRelativeDating();

		//********************************************************************
		// And compare the relative dating
		//********************************************************************

		if (l_1stFrameRelativeDating < l_2ndFrameRelativeDating)
		{
			l_compare = -1;
		}
		else if (l_1stFrameRelativeDating == l_2ndFrameRelativeDating)
		{
			l_compare = 0;
		}
		else
		{
			l_compare = 1;
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 1573", 417);
		CO_RETHROW;
    }
    return l_compare;
}




//****************************************************************************
//
// SPG : CompareId
// ROL : Compare two Provor frames id and returns :
//		 -1 if 1st < 2nd
//		 0 if 1st = 2nd
//		 1 if 1st > 2nd
//
//****************************************************************************
/*static*/ int
CoDatProvorFrame::
CompareId(
	void* qp_1stFrame,						// I : the first frame
	void* qp_2ndFrame,						// I : the second frame
	void* qp_period							// I : the period as void*
	)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

    unsigned int l_1stFrameId = 0;			// the first frame's id
    unsigned int l_2ndFrameId = 0;			// the second frame's id
    int l_compare = 0;						// the comparison result

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
		//********************************************************************
		// Retrieve each frame's id
		//********************************************************************

		l_1stFrameId = ((CoDatProvorFrame*)qp_1stFrame)->GetId();
		l_2ndFrameId = ((CoDatProvorFrame*)qp_2ndFrame)->GetId();

		//********************************************************************
		// And compare the relative dating
		//********************************************************************

		if (l_1stFrameId < l_2ndFrameId)
		{
			l_compare = -1;
		}
		else if (l_1stFrameId == l_2ndFrameId)
		{
			l_compare = 0;
		}
		else
		{
			l_compare = 1;
		}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 1644", 418);
		CO_RETHROW;
    }
    return l_compare;
}




//****************************************************************************
//
// SPG : DecodRecords
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecords(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if (_version == 2.2)
		{
			DecodRecordsV22(p_records);
		}
		else if ((_version == 2.6) || (_version == 2.7))
		{
			DecodRecordsV26(p_records);
		}
		else if (_version == 3.21)
		{
			DecodRecordsV321(p_records);
		}
		else if ((_version == 3.5) || (_version == 3.61))
		{
			DecodRecordsV35(p_records);
		}
		else if ((_version == 3.51) || (_version == 3.52))
		{
			DecodRecordsV351(p_records);
		}
		else if ((_version == 3.8) || (_version == 3.81))
		{
			DecodRecordsV38(p_records);
		}
		else if (_version == 4.0)
		{
			DecodRecordsV40(p_records);
		}
		else if ((_version == 4.1) || (_version == 4.11))
		{
			DecodRecordsV41(p_records);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 1719", 419);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV22
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecordsV22(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	double l_pressureValue = 0.0;			// the current pressure value
	double l_temperatureValue = 0.0;		// the current temperature value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatMeasure l_temperature;				// the temperature measure;
	int l_field;							// a bitfield value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit
		//********************************************************************

		if ((GetType() == 1) || (GetType() == 3))
		{
			l_position = 29;
		}
		else
		{
			l_position = 31;
		}

		//********************************************************************
		// Decod the first record
		//********************************************************************

		l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
		l_position += 11;
		l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
		l_position += 12;
		ql_record = new CoDatRecord();
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
		p_records.append(ql_record);

		//********************************************************************
		// While there is records to decod
		//********************************************************************

		while (l_position < CoDatProvorFrame_k_length)
		{
			//****************************************************************
			// Should we decod a relative or an absolute pressure ?
			//****************************************************************

			if (_bitVec.TestBit(l_position++) == FALSE)
			{
				//************************************************************
				// Absolute pressure
				//************************************************************
				
				if (l_position <= CoDatProvorFrame_k_length - 11)
				{
					l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
					l_position += 11;
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative pressure
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_pressureValue = double(int(l_field | 0xFFFFFFC0));
					}
					else
					{
						l_pressureValue = l_field;
					}
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '5');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute temperature ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 12)
				{
					l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
					l_position += 12;
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_temperatureValue = double(int(l_field | 0xFFFFFFC0)) / 100.0;
					}
					else
					{
						l_temperatureValue = l_field / 100.0;
					}
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// If the pressure and the temperature are decoded, create the records
			//****************************************************************

			if ((l_pressureValue != 0.0) || (l_temperatureValue != -5.0))
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 1915", 420);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV26
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecordsV26(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	double l_pressureValue = 0.0;			// the current pressure value
	double l_temperatureValue = 0.0;		// the current temperature value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatMeasure l_temperature;				// the temperature measure;
	int l_field;							// a bitfield value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit
		//********************************************************************

		if ((GetType() == 1) || (GetType() == 3))
		{
			l_position = 29;
		}
		else
		{
			l_position = 31;
		}

		//********************************************************************
		// Decod the first record
		//********************************************************************

		l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
		l_position += 11;
		l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
		l_position += 12;
		ql_record = new CoDatRecord();
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
		p_records.append(ql_record);

		//********************************************************************
		// While there is records to decod
		//********************************************************************

		while (l_position < CoDatProvorFrame_k_length)
		{
			//****************************************************************
			// Should we decod a relative or an absolute pressure ?
			//****************************************************************

			if (_bitVec.TestBit(l_position++) == FALSE)
			{
				//************************************************************
				// Absolute pressure
				//************************************************************
				
				if (l_position <= CoDatProvorFrame_k_length - 11)
				{
					l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
					l_position += 11;
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative pressure (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 5)
				{
					l_field = int(_bitVec.GetUInt(l_position, 5));
					l_position += 5;
					if (GetType() == 1)
					{
						l_pressureValue = l_field;
					}
					else if (GetType() == 3)
					{
						l_pressureValue = -l_field;
					}
					else
					{
						if (l_field >= 16)
						{
							l_pressureValue = double(int(l_field | 0xFFFFFFE0));
						}
						else
						{
							l_pressureValue = l_field;
						}
					}
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '5');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute temperature ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 12)
				{
					l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
					l_position += 12;
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_temperatureValue = double(int(l_field | 0xFFFFFFC0)) / 100.0;
					}
					else
					{
						l_temperatureValue = l_field / 100.0;
					}
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// If the pressure and the temperature are decoded, create the records
			//****************************************************************

			if ((l_pressureValue != 0.0) || (l_temperatureValue != -5.0))
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 2122", 421);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV321
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecordsV321(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	double l_pressureValue = 0.0;			// the current pressure value
	double l_temperatureValue = 0.0;		// the current temperature value
	double l_conductivityValue = 0.0;		// the current conductivity value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatMeasure l_temperature;				// the temperature measure;
	CoDatMeasure l_conductivity;			// the conductivity measure;
	int l_field;							// a bitfield value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit
		//********************************************************************

		if ((GetType() == 4) || (GetType() == 6))
		{
			l_position = 29;
		}
		else
		{
			l_position = 31;
		}

		//********************************************************************
		// Decod the first record
		//********************************************************************

		l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
		l_position += 11;
		l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
		l_position += 12;
		l_conductivityValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 + 25.0;
		l_position += 12;
		ql_record = new CoDatRecord();
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '0'));
		p_records.append(ql_record);

		//********************************************************************
		// While there is records to decod
		//********************************************************************

		while (l_position < CoDatProvorFrame_k_length)
		{
			//****************************************************************
			// Should we decod a relative or an absolute pressure ?
			//****************************************************************

			if (_bitVec.TestBit(l_position++) == FALSE)
			{
				//************************************************************
				// Absolute pressure
				//************************************************************
				
				if (l_position <= CoDatProvorFrame_k_length - 11)
				{
					l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
					l_position += 11;
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative pressure
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_pressureValue = double(int(l_field | 0xFFFFFFC0));
					}
					else
					{
						l_pressureValue = l_field;
					}
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '5');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute temperature ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 12)
				{
					l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
					l_position += 12;
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_temperatureValue = double(int(l_field | 0xFFFFFFC0)) / 100.0;
					}
					else
					{
						l_temperatureValue = l_field / 100.0;
					}
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute conductivity ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute conductivity
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 12)
				{
					l_conductivityValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 + 25.0;
					l_position += 12;
					l_conductivity = CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '0');
				}
				else
				{
					l_conductivityValue = 25.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative conductivity
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_conductivityValue = double(int(l_field | 0xFFFFFFC0)) / 100.0;
					}
					else
					{
						l_conductivityValue = l_field / 100.0;
					}
					l_conductivity = CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '5');
				}
				else
				{
					l_conductivityValue = 25.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// If the pressure, the temperature and the conductivity
			// are decoded, create the records
			//****************************************************************

			if ((l_pressureValue != 0.0) ||
				(l_temperatureValue != -5.0) ||
				(l_conductivityValue != 25.0))
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_conductivity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 2377", 422);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV35
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecordsV35(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	double l_pressureValue = 0.0;			// the current pressure value
	double l_temperatureValue = 0.0;		// the current temperature value
	double l_conductivityValue = 0.0;		// the current conductivity value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatMeasure l_temperature;				// the temperature measure;
	CoDatMeasure l_conductivity;			// the conductivity measure;
	int l_field;							// a bitfield value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit
		//********************************************************************

		if ((GetType() == 4) || (GetType() == 6))
		{
			l_position = 29;
		}
		else
		{
			l_position = 31;
		}

		//********************************************************************
		// Decod the first record
		//********************************************************************

		l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
		l_position += 11;
		l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
		l_position += 12;
		l_conductivityValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 + 25.0;
		l_position += 12;
		ql_record = new CoDatRecord();
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '0'));
		p_records.append(ql_record);

		//********************************************************************
		// While there is records to decod
		//********************************************************************

		while (l_position < CoDatProvorFrame_k_length)
		{
			//****************************************************************
			// Should we decod a relative or an absolute pressure ?
			//****************************************************************

			if (_bitVec.TestBit(l_position++) == FALSE)
			{
				//************************************************************
				// Absolute pressure
				//************************************************************
				
				if (l_position <= CoDatProvorFrame_k_length - 11)
				{
					l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
					l_position += 11;
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative pressure (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 5)
				{
					l_field = int(_bitVec.GetUInt(l_position, 5));
					l_position += 5;
					if (GetType() == 4)
					{
						l_pressureValue = l_field;
					}
					else if (GetType() == 6)
					{
						l_pressureValue = -l_field;
					}
					else
					{
						if (l_field >= 16)
						{
							l_pressureValue = double(int(l_field | 0xFFFFFFE0));
						}
						else
						{
							l_pressureValue = l_field;
						}
					}
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '5');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute temperature ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 12)
				{
					l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
					l_position += 12;
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_temperatureValue = double(int(l_field | 0xFFFFFFC0)) / 100.0;
					}
					else
					{
						l_temperatureValue = l_field / 100.0;
					}
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute conductivity ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute conductivity
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 12)
				{
					l_conductivityValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 + 25.0;
					l_position += 12;
					l_conductivity = CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '0');
				}
				else
				{
					l_conductivityValue = 25.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative conductivity
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_conductivityValue = double(int(l_field | 0xFFFFFFC0)) / 100.0;
					}
					else
					{
						l_conductivityValue = l_field / 100.0;
					}
					l_conductivity = CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '5');
				}
				else
				{
					l_conductivityValue = 25.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// If the pressure, the temperature and the conductivity
			// are decoded, create the records
			//****************************************************************

			if ((l_pressureValue != 0.0) ||
				(l_temperatureValue != -5.0) ||
				(l_conductivityValue != 25.0))
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_conductivity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 2643", 423);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV351
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecordsV351(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	double l_pressureValue = 0.0;			// the current pressure value
	double l_temperatureValue = 0.0;		// the current temperature value
	double l_salinityValue = 0.0;			// the current salinity value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatMeasure l_temperature;				// the temperature measure;
	CoDatMeasure l_salinity;				// the salinity measure;
	int l_field;							// a bitfield value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit
		//********************************************************************

		if ((GetType() == 4) || (GetType() == 6))
		{
			l_position = 29;
		}
		else
		{
			l_position = 31;
		}

		//********************************************************************
		// Decod the first record
		//********************************************************************

		l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
		l_position += 11;
		l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
		l_position += 12;
		l_salinityValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 + 5.0;
		l_position += 12;
		ql_record = new CoDatRecord();
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0'));
		p_records.append(ql_record);

		//********************************************************************
		// While there is records to decod
		//********************************************************************

		while (l_position < CoDatProvorFrame_k_length)
		{
			//****************************************************************
			// Should we decod a relative or an absolute pressure ?
			//****************************************************************

			if (_bitVec.TestBit(l_position++) == FALSE)
			{
				//************************************************************
				// Absolute pressure
				//************************************************************
				
				if (l_position <= CoDatProvorFrame_k_length - 11)
				{
					l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
					l_position += 11;
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative pressure (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 5)
				{
					l_field = int(_bitVec.GetUInt(l_position, 5));
					l_position += 5;
					if (GetType() == 4)
					{
						l_pressureValue = l_field;
					}
					else if (GetType() == 6)
					{
						l_pressureValue = -l_field;
					}
					else
					{
						if (l_field >= 16)
						{
							l_pressureValue = double(int(l_field | 0xFFFFFFE0));
						}
						else
						{
							l_pressureValue = l_field;
						}
					}
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '5');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute temperature ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 12)
				{
					l_temperatureValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 - 5.0;
					l_position += 12;
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_temperatureValue = double(int(l_field | 0xFFFFFFC0)) / 100.0;
					}
					else
					{
						l_temperatureValue = l_field / 100.0;
					}
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');
				}
				else
				{
					l_temperatureValue = -5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute salinity ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute salinity
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 12)
				{
					l_salinityValue = double(_bitVec.GetUInt(l_position, 12)) / 100.0 + 5.0;
					l_position += 12;
					l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0');
				}
				else
				{
					l_salinityValue = 5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative salinity
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (l_field >= 32)
					{
						l_salinityValue = double(int(l_field | 0xFFFFFFC0)) / 100.0;
					}
					else
					{
						l_salinityValue = l_field / 100.0;
					}
					l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '5');
				}
				else
				{
					l_salinityValue = 5.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// If the pressure, the temperature and the salinity
			// are decoded, create the records
			//****************************************************************

			if ((l_pressureValue != 0.0) ||
				(l_temperatureValue != -5.0) ||
				(l_salinityValue != 5.0))
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_salinity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 2909", 424);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV38
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecordsV38(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	double l_pressureValue = 0.0;			// the current pressure value
	double l_temperatureValue = 0.0;		// the current temperature value
	double l_conductivityValue = 0.0;		// the current conductivity value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatMeasure l_temperature;				// the temperature measure;
	CoDatMeasure l_conductivity;			// the conductivity measure;
	int l_field;							// a bitfield value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit
		//********************************************************************

		if ((GetType() == 4) || (GetType() == 6))
		{
			l_position = 29;
		}
		else
		{
			l_position = 31;
		}

		//********************************************************************
		// Decod the first record
		//********************************************************************

		l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
		l_position += 11;
		l_temperatureValue = double(_bitVec.GetUInt(l_position, 15)) / 1000.0 - 2.0;
		l_position += 15;
		l_conductivityValue = double(_bitVec.GetUInt(l_position, 16)) / 1000.0;
		l_position += 16;
		ql_record = new CoDatRecord();
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '0'));
		p_records.append(ql_record);

		//********************************************************************
		// While there is records to decod
		//********************************************************************

		while (l_position < CoDatProvorFrame_k_length)
		{
			//****************************************************************
			// Should we decod a relative or an absolute pressure ?
			//****************************************************************

			if (_bitVec.TestBit(l_position++) == FALSE)
			{
				//************************************************************
				// Absolute pressure
				//************************************************************
				
				if (l_position <= CoDatProvorFrame_k_length - 11)
				{
					l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
					l_position += 11;
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative pressure (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (GetType() == 4)
					{
						l_pressureValue = l_field;
					}
					else if (GetType() == 6)
					{
						l_pressureValue = -l_field;
					}
					else
					{
						if (l_field >= 32)
						{
							l_pressureValue = double(int(l_field | 0xFFFFFFC0));
						}
						else
						{
							l_pressureValue = l_field;
						}
					}
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '5');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute temperature ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 15)
				{
					l_temperatureValue = double(_bitVec.GetUInt(l_position, 15)) / 1000.0 - 2.0;
					l_position += 15;
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');
				}
				else
				{
					l_temperatureValue = -2.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative temperature (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 10)
				{
					l_field = int(_bitVec.GetUInt(l_position, 10));
					l_position += 10;
					if (GetType() == 4)
					{
						l_temperatureValue = -l_field / 1000.0 + 0.1;
					}
					else if (GetType() == 6)
					{
						l_temperatureValue = l_field / 1000.0 - 0.1;
					}
					else
					{
						if (l_field >= 512)
						{
							l_temperatureValue = double(int(l_field | 0xFFFFFC00)) / 1000.0;
						}
						else
						{
							l_temperatureValue = l_field / 1000.0;
						}
					}
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');
				}
				else
				{
					l_temperatureValue = -2.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute conductivity ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute conductivity
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 16)
				{
					l_conductivityValue = double(_bitVec.GetUInt(l_position, 16)) / 1000.0;
					l_position += 16;
					l_conductivity = CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '0');
				}
				else
				{
					l_conductivityValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative conductivity (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 10)
				{
					l_field = int(_bitVec.GetUInt(l_position, 10));
					l_position += 10;
					if (GetType() == 4)
					{
						l_conductivityValue = -l_field / 1000.0 + 0.1;
					}
					else if (GetType() == 6)
					{
						l_conductivityValue = l_field / 1000.0 - 0.1;
					}
					else
					{
						if (l_field >= 512)
						{
							l_conductivityValue = double(int(l_field | 0xFFFFFC00)) / 1000.0;
						}
						else
						{
							l_conductivityValue = l_field / 1000.0;
						}
					}
					l_conductivity = CoDatMeasure(CoDat_k_ppConductivity, l_conductivityValue, '5');
				}
				else
				{
					l_conductivityValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// If the pressure, the temperature and the conductivity
			// are decoded, create the records
			//****************************************************************

			if ((l_pressureValue != 0.0) ||
				(l_temperatureValue != -2.0) ||
				(l_conductivityValue != 0.0))
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_conductivity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 3197", 425);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV40
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecordsV40(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	double l_pressureValue = 0.0;			// the current pressure value
	double l_temperatureValue = 0.0;		// the current temperature value
	double l_salinityValue = 0.0;			// the current salinity value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatMeasure l_temperature;				// the temperature measure;
	CoDatMeasure l_salinity;				// the salinity measure;
	int l_field;							// a bitfield value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// For the drift records
		//********************************************************************

		if (GetType() == 2)
		{
			//****************************************************************
			// Compute the start bit
			//****************************************************************

			l_position = 44;

			//****************************************************************
			// Decod the first record
			//****************************************************************

			l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
			l_position += 11;
			l_temperatureValue = double(_bitVec.GetUInt(l_position, 14)) * 0.002 - 2.0;
			l_position += 14;
			l_salinityValue = double(_bitVec.GetUInt(l_position, 13)) * 0.0025 + 25.0;
			l_position += 13;
			ql_record = new CoDatRecord();
			ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0'));
			ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
			ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0'));
			p_records.append(ql_record);

			//****************************************************************
			// While there is records to decod
			//****************************************************************

			while (l_position < CoDatProvorFrame_k_length)
			{
				//************************************************************
				// Should we decod a relative or an absolute pressure ?
				//************************************************************

				if (_bitVec.TestBit(l_position++) == FALSE)
				{
					//********************************************************
					// Absolute pressure
					//********************************************************
				
					if (l_position <= CoDatProvorFrame_k_length - 11)
					{
						l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
						l_position += 11;
						l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0');
					}
					else
					{
						l_pressureValue = 0.0;
						l_position = CoDatProvorFrame_k_length;
					}
				}
				else
				{
					//********************************************************
					// Relative pressure
					//********************************************************

					if (l_position <= CoDatProvorFrame_k_length - 6)
					{
						l_field = int(_bitVec.GetUInt(l_position, 6));
						l_position += 6;
						if (l_field >= 32)
						{
							l_pressureValue = double(int(l_field | 0xFFFFFFC0));
						}
						else
						{
							l_pressureValue = l_field;
						}
						l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '5');
					}
					else
					{
						l_pressureValue = 0.0;
						l_position = CoDatProvorFrame_k_length;
					}
				}

				//************************************************************
				// Should we decod a relative or an absolute temperature ?
				//************************************************************

				if ((l_position >= CoDatProvorFrame_k_length) ||
					(_bitVec.TestBit(l_position++) == FALSE))
				{
					//********************************************************
					// Absolute temperature
					//********************************************************
				
					if (l_position <= CoDatProvorFrame_k_length - 14)
					{
						l_temperatureValue = double(_bitVec.GetUInt(l_position, 14)) * 0.002 - 2.0;
						l_position += 14;
						l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');
					}
					else
					{
						l_temperatureValue = -2.0;
						l_position = CoDatProvorFrame_k_length;
					}
				}
				else
				{
					//********************************************************
					// Relative temperature
					//********************************************************

					if (l_position <= CoDatProvorFrame_k_length - 8)
					{
						l_field = int(_bitVec.GetUInt(l_position, 8));
						l_position += 8;
						if (l_field >= 128)
						{
							l_temperatureValue = double(int(l_field | 0xFFFFFF00)) * 0.002;
						}
						else
						{
							l_temperatureValue = double(l_field) * 0.002;
						}
						l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');
					}
					else
					{
						l_temperatureValue = -2.0;
						l_position = CoDatProvorFrame_k_length;
					}
				}

				//************************************************************
				// Should we decod a relative or an absolute salinity ?
				//************************************************************

				if ((l_position >= CoDatProvorFrame_k_length) ||
					(_bitVec.TestBit(l_position++) == FALSE))
				{
					//********************************************************
					// Absolute salinity
					//********************************************************
				
					if (l_position <= CoDatProvorFrame_k_length - 13)
					{
						l_salinityValue = double(_bitVec.GetUInt(l_position, 13)) * 0.0025 + 25.0;
						l_position += 13;
						l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0');
					}
					else
					{
						l_salinityValue = 25.0;
						l_position = CoDatProvorFrame_k_length;
					}
				}
				else
				{
					//********************************************************
					// Relative salinity
					//********************************************************

					if (l_position <= CoDatProvorFrame_k_length - 7)
					{
						l_field = int(_bitVec.GetUInt(l_position, 7));
						l_position += 7;
						if (l_field >= 64)
						{
							l_salinityValue = double(int(l_field | 0xFFFFFF80)) * 0.0025;
						}
						else
						{
							l_salinityValue = double(l_field) * 0.0025;
						}
						l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '5');
					}
					else
					{
						l_salinityValue = 25.0;
						l_position = CoDatProvorFrame_k_length;
					}
				}

				//************************************************************
				// If the pressure, the temperature and the salinity
				// are decoded, create the records
				//************************************************************

				if ((l_pressureValue != 0.0) ||
					(l_temperatureValue != -2.0) ||
					(l_salinityValue != 25.0))
				{
					ql_record = new CoDatRecord();
					ql_record->AddMeasure(l_pressure);
					ql_record->AddMeasure(l_temperature);
					ql_record->AddMeasure(l_salinity);
					p_records.append(ql_record);
				}
			}
		}

		//********************************************************************
		// For profile records without joker
		//********************************************************************

		else if (((GetType() == 1) || (GetType() == 3)) && (GetJoker() == 0))
		{
			//****************************************************************
			// Compute the start bit
			//****************************************************************

			l_position = 35;

			//****************************************************************
			// Decod the first record
			//****************************************************************

			l_temperatureValue = double(_bitVec.GetUInt(l_position, 14)) * 0.002 - 2.0;
			l_position += 14;
			l_salinityValue = double(_bitVec.GetUInt(l_position, 13)) * 0.0025 + 25.0;
			l_position += 13;
			ql_record = new CoDatRecord();
			ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
			ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0'));
			p_records.append(ql_record);

			//****************************************************************
			// While there is records to decod
			//****************************************************************

			while (l_position < CoDatProvorFrame_k_length - 21)
			{
				//************************************************************
				// Decod a relative temperature
				//************************************************************

				l_field = int(_bitVec.GetUInt(l_position, 11));
				l_position += 11;
				if (l_field >= 1024)
				{
					l_temperatureValue = double(int(l_field | 0xFFFFF800)) * 0.002;
				}
				else
				{
					l_temperatureValue = double(l_field) * 0.002;
				}
				l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');

				//************************************************************
				// Decod a relative salinity
				//************************************************************

				l_field = int(_bitVec.GetUInt(l_position, 10));
				l_position += 10;
				if (l_field >= 512)
				{
					l_salinityValue = double(int(l_field | 0xFFFFFC00)) * 0.0025;
				}
				else
				{
					l_salinityValue = double(l_field) * 0.0025;
				}
				l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '5');

				//************************************************************
				// Create the record
				//************************************************************

				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_salinity);
				p_records.append(ql_record);
			}

			//****************************************************************
			// Remove trailing records where relative temperature and
			// relative salinity are 0
			//****************************************************************

			ql_record = p_records.last();
			while ((p_records.entries() > 0) &&
				   (ql_record->GetMeasure(CoDat_k_ppTemperature).GetValue() == 0.0) &&
				   (ql_record->GetMeasure(CoDat_k_ppSalinity).GetValue() == 0.0))
			{
				ql_record = p_records.removeLast();
				delete ql_record;
				if (p_records.entries() > 0)
				{
					ql_record = p_records.last();
				}
			}
		}

		//********************************************************************
		// For profile records with joker
		//********************************************************************

		else if (((GetType() == 1) || (GetType() == 3)) && (GetJoker() != 0))
		{
			//****************************************************************
			// Compute the start bit
			//****************************************************************

			l_position = 35;

			//****************************************************************
			// Decod the first record
			//****************************************************************

			l_temperatureValue = double(_bitVec.GetUInt(l_position, 14)) * 0.002 - 2.0;
			l_position += 14;
			l_salinityValue = double(_bitVec.GetUInt(l_position, 13)) * 0.0025 + 25.0;
			l_position += 13;
			ql_record = new CoDatRecord();
			ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
			ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0'));
			p_records.append(ql_record);

			//****************************************************************
			// While there is records to decod
			//****************************************************************

			while (l_position < CoDatProvorFrame_k_length - 27)
			{
				//************************************************************
				// Decod an absolute temperature
				//************************************************************

				l_temperatureValue = double(_bitVec.GetUInt(l_position, 14)) * 0.002 - 2.0;
				l_position += 14;
				l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');

				//************************************************************
				// Decod an absolute salinity
				//************************************************************

				l_salinityValue = double(_bitVec.GetUInt(l_position, 13)) * 0.0025 + 25.0;
				l_position += 13;
				l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0');

				//************************************************************
				// If the temperature and the salinity are decoded, create
				// the record
				//************************************************************

				if ((l_temperatureValue != -2.0) ||
					(l_salinityValue != 25.0))
				{
					ql_record = new CoDatRecord();
					ql_record->AddMeasure(l_temperature);
					ql_record->AddMeasure(l_salinity);
					p_records.append(ql_record);
				}
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 3607", 426);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodRecordsV41
// ROL : Decod the records contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodRecordsV41(
	RWTPtrSlist<CoDatRecord>& p_records		// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_position = 0;						// current bitfield position
	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	double l_pressureValue = 0.0;			// the current pressure value
	double l_temperatureValue = 0.0;		// the current temperature value
	double l_salinityValue = 0.0;			// the current salinity value
	CoDatMeasure l_pressure;				// the pressure measure;
	CoDatMeasure l_temperature;				// the temperature measure;
	CoDatMeasure l_salinity;				// the salinity measure;
	int l_field;							// a bitfield value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Clear the output list
		//********************************************************************

		p_records.clear();

		//********************************************************************
		// Compute the start bit
		//********************************************************************

		if ((GetType() == 4) || (GetType() == 6))
		{
			l_position = 29;
		}
		else
		{
			l_position = 31;
		}

		//********************************************************************
		// Decod the first record
		//********************************************************************

		l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
		l_position += 11;
		l_temperatureValue = double(_bitVec.GetUInt(l_position, 15)) / 1000.0 - 2.0;
		l_position += 15;
		l_salinityValue = double(_bitVec.GetUInt(l_position, 15)) / 1000.0 + 10.0;
		l_position += 15;
		ql_record = new CoDatRecord();
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0'));
		ql_record->AddMeasure(CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0'));
		p_records.append(ql_record);

		//********************************************************************
		// While there is records to decod
		//********************************************************************

		while (l_position < CoDatProvorFrame_k_length)
		{
			//****************************************************************
			// Should we decod a relative or an absolute pressure ?
			//****************************************************************

			if (_bitVec.TestBit(l_position++) == FALSE)
			{
				//************************************************************
				// Absolute pressure
				//************************************************************
				
				if (l_position <= CoDatProvorFrame_k_length - 11)
				{
					l_pressureValue = double(_bitVec.GetUInt(l_position, 11));
					l_position += 11;
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '0');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative pressure (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 6)
				{
					l_field = int(_bitVec.GetUInt(l_position, 6));
					l_position += 6;
					if (GetType() == 4)
					{
						l_pressureValue = l_field;
					}
					else if (GetType() == 6)
					{
						l_pressureValue = -l_field;
					}
					else
					{
						if (l_field >= 32)
						{
							l_pressureValue = double(int(l_field | 0xFFFFFFC0));
						}
						else
						{
							l_pressureValue = l_field;
						}
					}
					l_pressure = CoDatMeasure(CoDat_k_ppPressure, l_pressureValue, '5');
				}
				else
				{
					l_pressureValue = 0.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute temperature ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute temperature
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 15)
				{
					l_temperatureValue = double(_bitVec.GetUInt(l_position, 15)) / 1000.0 - 2.0;
					l_position += 15;
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '0');
				}
				else
				{
					l_temperatureValue = -2.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative temperature (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 10)
				{
					l_field = int(_bitVec.GetUInt(l_position, 10));
					l_position += 10;
					if (GetType() == 4)
					{
						l_temperatureValue = -l_field / 1000.0 + 0.1;
					}
					else if (GetType() == 6)
					{
						l_temperatureValue = l_field / 1000.0 - 0.1;
					}
					else
					{
						if (l_field >= 512)
						{
							l_temperatureValue = double(int(l_field | 0xFFFFFC00)) / 1000.0;
						}
						else
						{
							l_temperatureValue = l_field / 1000.0;
						}
					}
					l_temperature = CoDatMeasure(CoDat_k_ppTemperature, l_temperatureValue, '5');
				}
				else
				{
					l_temperatureValue = -2.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// Should we decod a relative or an absolute salinity ?
			//****************************************************************

			if ((l_position >= CoDatProvorFrame_k_length) ||
				(_bitVec.TestBit(l_position++) == FALSE))
			{
				//************************************************************
				// Absolute salinity
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 15)
				{
					l_salinityValue = double(_bitVec.GetUInt(l_position, 15)) / 1000.0 + 10.0;
					l_position += 15;
					l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '0');
				}
				else
				{
					l_salinityValue = 10.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}
			else
			{
				//************************************************************
				// Relative salinity (depends of the frame type)
				//************************************************************

				if (l_position <= CoDatProvorFrame_k_length - 8)
				{
					l_field = int(_bitVec.GetUInt(l_position, 8));
					l_position += 8;
					if (GetType() == 4)
					{
						l_salinityValue = -l_field / 1000.0 + 0.025;
					}
					else if (GetType() == 6)
					{
						l_salinityValue = l_field / 1000.0 - 0.025;
					}
					else
					{
						if (l_field >= 128)
						{
							l_salinityValue = double(int(l_field | 0xFFFFFF00)) / 1000.0;
						}
						else
						{
							l_salinityValue = l_field / 1000.0;
						}
					}
					l_salinity = CoDatMeasure(CoDat_k_ppSalinity, l_salinityValue, '5');
				}
				else
				{
					l_salinityValue = 10.0;
					l_position = CoDatProvorFrame_k_length;
				}
			}

			//****************************************************************
			// If the pressure, the temperature and the conductivity
			// are decoded, create the records
			//****************************************************************

			if ((l_pressureValue != 0.0) ||
				(l_temperatureValue != -2.0) ||
				(l_salinityValue != 10.0))
			{
				ql_record = new CoDatRecord();
				ql_record->AddMeasure(l_pressure);
				ql_record->AddMeasure(l_temperature);
				ql_record->AddMeasure(l_salinity);
				p_records.append(ql_record);
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 3895", 435);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatus
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodTechnicalStatus(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Depending of the frame version
		//********************************************************************

		if ((_version == 2.2) || (_version == 3.21))
		{
			DecodTechnicalStatusV22(p_status, p_timeCorrection);
		}
		else if ((_version == 2.6) || (_version == 2.7) || (_version == 3.5) ||
				 (_version == 3.51) || (_version == 3.52))
		{
			DecodTechnicalStatusV26(p_status, p_timeCorrection);
		}
		else if (_version == 3.61)
		{
			DecodTechnicalStatusV361(p_status, p_timeCorrection);
		}
		else if ((_version == 3.8) || (_version == 3.81) || (_version == 4.1) ||
				 (_version == 4.11))
		{
			DecodTechnicalStatusV38(p_status, p_timeCorrection);
		}
		else if (_version == 4.0)
		{
			DecodTechnicalStatusV40(p_status, p_timeCorrection);
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 3960", 427);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV22
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodTechnicalStatusV22(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	RWTime l_dateTimeValue =				// A date-time value
		(unsigned long)0;
	double l_doubleValue = 0.0;				// A double value
	RWCString l_stringValue = "";			// A string value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// DESFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(20, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_Surface_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(28, 7);
		p_status.AddAttribute(CoDatAttribute("DESFC_Surface_valve", l_intValue));

		//********************************************************************
		// DESFC_1st_stab_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(35, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_1st_stab_press
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(43, 8)) * 10.0;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_press", l_doubleValue));

		//********************************************************************
		// DESFC_Depth_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(51, 3);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_valve", l_intValue));

		//********************************************************************
		// DESFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(54, 3);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_pump", l_intValue));

		//********************************************************************
		// DESFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(57, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_End_time", l_dateTimeValue));

		//********************************************************************
		// IMMFC_Depth_correct
		//********************************************************************

		l_intValue = _bitVec.GetUInt(65, 4);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Depth_correct", l_intValue));

		//********************************************************************
		// ASCFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(69, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_End_time", l_dateTimeValue));

		//********************************************************************
		// ASCFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(77, 5);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Depth_pump", l_intValue));

		//********************************************************************
		// ASCFC_Surface_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(82, 5);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Surface_pump", l_intValue));

		//********************************************************************
		// DESPR_PRSEP
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(87, 11));
		p_status.AddAttribute(CoDatAttribute("DESPR_PRSEP", l_doubleValue));

		//********************************************************************
		// DESPR_NTS
		//********************************************************************

		l_intValue = _bitVec.GetUInt(98, 6);
		p_status.AddAttribute(CoDatAttribute("DESPR_NTS", l_intValue));

		//********************************************************************
		// DESPR_ETS
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(104, 5)) / 100.0;
		p_status.AddAttribute(CoDatAttribute("DESPR_ETS", l_doubleValue));

		//********************************************************************
		// DESPR_NTF
		//********************************************************************

		l_intValue = _bitVec.GetUInt(120, 8);
		p_status.AddAttribute(CoDatAttribute("DESPR_NTF", l_intValue));

		//********************************************************************
		// DESPR_ETF
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(128, 5)) / 100.0;
		p_status.AddAttribute(CoDatAttribute("DESPR_ETF", l_doubleValue));

		//********************************************************************
		// ASCPR_PRSEP
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(146, 11));
		p_status.AddAttribute(CoDatAttribute("ASCPR_PRSEP", l_doubleValue));

		//********************************************************************
		// ASCPR_NTS
		//********************************************************************

		l_intValue = _bitVec.GetUInt(157, 6);
		p_status.AddAttribute(CoDatAttribute("ASCPR_NTS", l_intValue));

		//********************************************************************
		// ASCPR_ETS
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(163, 5)) / 100.0;
		p_status.AddAttribute(CoDatAttribute("ASCPR_ETS", l_doubleValue));

		//********************************************************************
		// ASCPR_NTF
		//********************************************************************

		l_intValue = _bitVec.GetUInt(179, 8);
		p_status.AddAttribute(CoDatAttribute("ASCPR_NTF", l_intValue));

		//********************************************************************
		// ASCPR_ETF
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(187, 5)) / 100.0;
		p_status.AddAttribute(CoDatAttribute("ASCPR_ETF", l_doubleValue));

		//********************************************************************
		// Ixxxx_Internal_count
		//********************************************************************

		l_intValue = _bitVec.GetUInt(205, 8);
		p_status.AddAttribute(CoDatAttribute("ITEMP_Internal_count", l_intValue));
		p_status.AddAttribute(CoDatAttribute("ICOND_Internal_count", l_intValue));

		//********************************************************************
		// TIMEC_Internal_time
		//********************************************************************

		l_dateTimeValue = RWTime(RWDate(),
								 _bitVec.GetUInt(213, 5),
								 _bitVec.GetUInt(218, 6),
								 _bitVec.GetUInt(224, 6));
		p_status.AddAttribute(CoDatAttribute("TIMEC_Internal_time", l_dateTimeValue));

		//********************************************************************
		// SENSO_Press_offset
		//********************************************************************

		l_intValue = _bitVec.GetUInt(230, 6);
		if (l_intValue >= 32)
		{
			l_doubleValue = double(int(l_intValue | 0xFFFFFFC0));
		}
		else
		{
			l_doubleValue = double(l_intValue);
		}
		p_status.AddAttribute(CoDatAttribute("SENSO_Press_offset", l_doubleValue));

		//********************************************************************
		// SENSO_Internal_press
		//********************************************************************

		l_intValue = _bitVec.GetUInt(236, 3);
		l_stringValue = CoStc_g_catManager.GetInfo(3, 420 + l_intValue);
		p_status.AddAttribute(CoDatAttribute("SENSO_Internal_press", l_stringValue));

		//********************************************************************
		// SURFC_Temperature
		//********************************************************************

		l_doubleValue = double(short((unsigned short)_bitVec.GetUInt(239, 16))) / 1000.0;
		p_status.AddAttribute(CoDatAttribute("SURFC_Temperature", l_doubleValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 4203", 428);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV26
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodTechnicalStatusV26(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	RWTime l_dateTimeValue =				// A date-time value
		(unsigned long)0;
	double l_doubleValue = 0.0;				// A double value
	RWCString l_stringValue = "";			// A string value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// DESFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(20, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_Surface_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(28, 7);
		p_status.AddAttribute(CoDatAttribute("DESFC_Surface_valve", l_intValue));

		//********************************************************************
		// DESFC_1st_stab_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(35, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_1st_stab_press
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(43, 8)) * 10.0;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_press", l_doubleValue));

		//********************************************************************
		// DESFC_Depth_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(51, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_valve", l_intValue));

		//********************************************************************
		// DESFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(55, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_pump", l_intValue));

		//********************************************************************
		// DESFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(59, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_End_time", l_dateTimeValue));

		//********************************************************************
		// IMMFC_Depth_correct
		//********************************************************************

		l_intValue = _bitVec.GetUInt(67, 4);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Depth_correct", l_intValue));

		//********************************************************************
		// ASCFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(71, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_End_time", l_dateTimeValue));

		//********************************************************************
		// ASCFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(79, 5);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Depth_pump", l_intValue));

		//********************************************************************
		// ASCFC_Surface_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(84, 5);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Surface_pump", l_intValue));

		//********************************************************************
		// TRANS_DPROF_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(89, 5);
		p_status.AddAttribute(CoDatAttribute("TRANS_DPROF_Emitted", l_intValue));

		//********************************************************************
		// TRANS_IDRIFT_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(94, 5);
		p_status.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Emitted", l_intValue));

		//********************************************************************
		// TRANS_APROF_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(99, 5);
		p_status.AddAttribute(CoDatAttribute("TRANS_APROF_Emitted", l_intValue));

		//********************************************************************
		// DESPR_PRSEP
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(104, 11));
		p_status.AddAttribute(CoDatAttribute("DESPR_PRSEP", l_doubleValue));

		//********************************************************************
		// DESPR_NTS
		//********************************************************************

		l_intValue = _bitVec.GetUInt(115, 6);
		p_status.AddAttribute(CoDatAttribute("DESPR_NTS", l_intValue));

		//********************************************************************
		// DESPR_ETS
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(121, 5)) / 100.0;
		p_status.AddAttribute(CoDatAttribute("DESPR_ETS", l_doubleValue));

		//********************************************************************
		// DESPR_NTF
		//********************************************************************

		l_intValue = _bitVec.GetUInt(126, 8);
		p_status.AddAttribute(CoDatAttribute("DESPR_NTF", l_intValue));

		//********************************************************************
		// DESPR_ETF
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(134, 5)) / 100.0;
		p_status.AddAttribute(CoDatAttribute("DESPR_ETF", l_doubleValue));

		//********************************************************************
		// ASCPR_PRSEP
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(139, 11));
		p_status.AddAttribute(CoDatAttribute("ASCPR_PRSEP", l_doubleValue));

		//********************************************************************
		// ASCPR_NTS
		//********************************************************************

		l_intValue = _bitVec.GetUInt(150, 6);
		p_status.AddAttribute(CoDatAttribute("ASCPR_NTS", l_intValue));

		//********************************************************************
		// ASCPR_ETS
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(156, 5)) / 100.0;
		p_status.AddAttribute(CoDatAttribute("ASCPR_ETS", l_doubleValue));

		//********************************************************************
		// ASCPR_NTF
		//********************************************************************

		l_intValue = _bitVec.GetUInt(161, 8);
		p_status.AddAttribute(CoDatAttribute("ASCPR_NTF", l_intValue));

		//********************************************************************
		// ASCPR_ETF
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(169, 5)) / 100.0;
		p_status.AddAttribute(CoDatAttribute("ASCPR_ETF", l_doubleValue));

		//********************************************************************
		// Ixxxx_Internal_count
		//********************************************************************

		l_intValue = _bitVec.GetUInt(174, 8);
		p_status.AddAttribute(CoDatAttribute("ITEMP_Internal_count", l_intValue));
		p_status.AddAttribute(CoDatAttribute("ICOND_Internal_count", l_intValue));

		//********************************************************************
		// TIMEC_Internal_time
		//********************************************************************

		l_dateTimeValue = RWTime(RWDate(),
								 _bitVec.GetUInt(182, 5),
								 _bitVec.GetUInt(187, 6),
								 _bitVec.GetUInt(193, 6));
		p_status.AddAttribute(CoDatAttribute("TIMEC_Internal_time", l_dateTimeValue));

		//********************************************************************
		// SENSO_Press_offset
		//********************************************************************

		l_intValue = _bitVec.GetUInt(199, 6);
		if (l_intValue >= 32)
		{
			l_doubleValue = double(int(l_intValue | 0xFFFFFFC0));
		}
		else
		{
			l_doubleValue = double(l_intValue);
		}
		p_status.AddAttribute(CoDatAttribute("SENSO_Press_offset", l_doubleValue));

		//********************************************************************
		// SENSO_Internal_press
		//********************************************************************

		l_intValue = _bitVec.GetUInt(205, 3);
		l_stringValue = CoStc_g_catManager.GetInfo(3, 420 + l_intValue);
		p_status.AddAttribute(CoDatAttribute("SENSO_Internal_press", l_stringValue));

		//********************************************************************
		// SURFC_Temperature
		//********************************************************************

		l_doubleValue = double(short((unsigned short)_bitVec.GetUInt(208, 16))) / 1000.0;
		p_status.AddAttribute(CoDatAttribute("SURFC_Temperature", l_doubleValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 4467", 429);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV361
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodTechnicalStatusV361(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	RWTime l_dateTimeValue =				// A date-time value
		(unsigned long)0;
	double l_doubleValue = 0.0;				// A double value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// The v3.61 technical status is identical to the v2.6 technical
		// status with 4 new field at the end
		//********************************************************************

		DecodTechnicalStatusV26(p_status, p_timeCorrection);

		//********************************************************************
		// ASCFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(224, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// IMMFC_Gap_order_ent
		//********************************************************************

		l_intValue = _bitVec.GetUInt(232, 3);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Gap_order_ent", l_intValue));

		//********************************************************************
		// IMMFC_Min_pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(235, 8)) * 10.0;
		p_status.AddAttribute(CoDatAttribute("IMMFC_Min_pressure", l_doubleValue));

		//********************************************************************
		// IMMFC_Max_pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(243, 8)) * 10.0;
		p_status.AddAttribute(CoDatAttribute("IMMFC_Max_pressure", l_doubleValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 4546", 430);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV38
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodTechnicalStatusV38(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	RWTime l_dateTimeValue =				// A date-time value
		(unsigned long)0;
	double l_doubleValue = 0.0;				// A double value
	RWCString l_stringValue = "";			// A string value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// DESFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(20, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_Surface_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(28, 7);
		p_status.AddAttribute(CoDatAttribute("DESFC_Surface_valve", l_intValue));

		//********************************************************************
		// DESFC_1st_stab_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(35, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_1st_stab_press
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(43, 8)) * 10.0;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_press", l_doubleValue));

		//********************************************************************
		// DESFC_Depth_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(51, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_valve", l_intValue));

		//********************************************************************
		// DESFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(55, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_pump", l_intValue));

		//********************************************************************
		// DESFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(59, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_End_time", l_dateTimeValue));

		//********************************************************************
		// IMMFC_Depth_correct
		//********************************************************************

		l_intValue = _bitVec.GetUInt(67, 4);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Depth_correct", l_intValue));

		//********************************************************************
		// ASCFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(71, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_End_time", l_dateTimeValue));

		//********************************************************************
		// ASCFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(79, 5);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Depth_pump", l_intValue));

		//********************************************************************
		// ASCFC_Surface_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(84, 5);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Surface_pump", l_intValue));

		//********************************************************************
		// TRANS_DPROF_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(89, 5);
		p_status.AddAttribute(CoDatAttribute("TRANS_DPROF_Emitted", l_intValue));

		//********************************************************************
		// TRANS_IDRIFT_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(94, 5);
		p_status.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Emitted", l_intValue));

		//********************************************************************
		// TRANS_APROF_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(99, 5);
		p_status.AddAttribute(CoDatAttribute("TRANS_APROF_Emitted", l_intValue));

		//********************************************************************
		// DESPR_PRSEP
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(104, 11));
		p_status.AddAttribute(CoDatAttribute("DESPR_PRSEP", l_doubleValue));

		//********************************************************************
		// DESPR_NTS
		//********************************************************************

		l_intValue = _bitVec.GetUInt(115, 6);
		p_status.AddAttribute(CoDatAttribute("DESPR_NTS", l_intValue));

		//********************************************************************
		// DESPR_NTF
		//********************************************************************

		l_intValue = _bitVec.GetUInt(121, 8);
		p_status.AddAttribute(CoDatAttribute("DESPR_NTF", l_intValue));

		//********************************************************************
		// ASCPR_PRSEP
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(129, 11));
		p_status.AddAttribute(CoDatAttribute("ASCPR_PRSEP", l_doubleValue));

		//********************************************************************
		// ASCPR_NTS
		//********************************************************************

		l_intValue = _bitVec.GetUInt(140, 6);
		p_status.AddAttribute(CoDatAttribute("ASCPR_NTS", l_intValue));

		//********************************************************************
		// ASCPR_NTF
		//********************************************************************

		l_intValue = _bitVec.GetUInt(146, 8);
		p_status.AddAttribute(CoDatAttribute("ASCPR_NTF", l_intValue));

		//********************************************************************
		// Ixxxx_Internal_count
		//********************************************************************

		l_intValue = _bitVec.GetUInt(154, 8);
		p_status.AddAttribute(CoDatAttribute("ITEMP_Internal_count", l_intValue));
		p_status.AddAttribute(CoDatAttribute("ICOND_Internal_count", l_intValue));

		//********************************************************************
		// TIMEC_Internal_time
		//********************************************************************

		l_dateTimeValue = RWTime(RWDate(),
								 _bitVec.GetUInt(162, 5),
								 _bitVec.GetUInt(167, 6),
								 _bitVec.GetUInt(173, 6));
		p_status.AddAttribute(CoDatAttribute("TIMEC_Internal_time", l_dateTimeValue));

		//********************************************************************
		// SENSO_Press_offset
		//********************************************************************

		l_intValue = _bitVec.GetUInt(179, 6);
		if (l_intValue >= 32)
		{
			l_doubleValue = double(int(l_intValue | 0xFFFFFFC0));
		}
		else
		{
			l_doubleValue = double(l_intValue);
		}
		p_status.AddAttribute(CoDatAttribute("SENSO_Press_offset", l_doubleValue));

		//********************************************************************
		// SENSO_Internal_press
		//********************************************************************

		l_intValue = _bitVec.GetUInt(185, 3);
		l_stringValue = CoStc_g_catManager.GetInfo(3, 420 + l_intValue);
		p_status.AddAttribute(CoDatAttribute("SENSO_Internal_press", l_stringValue));

		//********************************************************************
		// SURFC_Temperature
		//********************************************************************

		l_doubleValue = double(short((unsigned short)_bitVec.GetUInt(188, 16))) / 1000.0;
		p_status.AddAttribute(CoDatAttribute("SURFC_Temperature", l_doubleValue));

		//********************************************************************
		// ASCFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(204, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// IMMFC_Gap_order_ent
		//********************************************************************

		l_intValue = _bitVec.GetUInt(212, 3);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Gap_order_ent", l_intValue));

		//********************************************************************
		// IMMFC_Min_pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(215, 8)) * 10.0;
		p_status.AddAttribute(CoDatAttribute("IMMFC_Min_pressure", l_doubleValue));

		//********************************************************************
		// IMMFC_Max_pressure
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(223, 8)) * 10.0;
		p_status.AddAttribute(CoDatAttribute("IMMFC_Max_pressure", l_doubleValue));
	
		//********************************************************************
		// IMMFC_Grounded
		//********************************************************************

		l_intValue = _bitVec.GetUInt(231, 1);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Grounded", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 4818", 431);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : DecodTechnicalStatusV40
// ROL : Decod the technical status contained in the frame
//
//****************************************************************************
void
CoDatProvorFrame::
DecodTechnicalStatusV40(
	CoDatTechnicalStatus& p_status,			// O : the technical status
	int p_timeCorrection					// I : The time correction to apply
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_intValue = 0;						// An int value
	RWTime l_dateTimeValue =				// A date-time value
		(unsigned long)0;
	double l_doubleValue = 0.0;				// A double value
	RWCString l_stringValue = "";			// A string value

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// TRANS_DPROF_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(33, 4);
		p_status.AddAttribute(CoDatAttribute("TRANS_DPROF_Emitted", l_intValue));

		//********************************************************************
		// TRANS_IDRIFT_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(37, 6);
		p_status.AddAttribute(CoDatAttribute("TRANS_IDRIFT_Emitted", l_intValue));

		//********************************************************************
		// TRANS_APROF_Emitted
		//********************************************************************

		l_intValue = _bitVec.GetUInt(43, 6);
		p_status.AddAttribute(CoDatAttribute("TRANS_APROF_Emitted", l_intValue));

		//********************************************************************
		// DESFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(49, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_Surface_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(57, 7);
		p_status.AddAttribute(CoDatAttribute("DESFC_Surface_valve", l_intValue));

		//********************************************************************
		// DESFC_1st_stab_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(64, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_time", l_dateTimeValue));

		//********************************************************************
		// DESFC_1st_stab_press
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(72, 8)) * 10.0;
		p_status.AddAttribute(CoDatAttribute("DESFC_1st_stab_press", l_doubleValue));

		//********************************************************************
		// DESFC_Depth_valve
		//********************************************************************

		l_intValue = _bitVec.GetUInt(80, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_valve", l_intValue));

		//********************************************************************
		// DESFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(84, 4);
		p_status.AddAttribute(CoDatAttribute("DESFC_Depth_pump", l_intValue));

		//********************************************************************
		// DESFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(88, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("DESFC_End_time", l_dateTimeValue));

		//********************************************************************
		// IMMFC_Depth_correct
		//********************************************************************

		l_intValue = _bitVec.GetUInt(96, 4);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Depth_correct", l_intValue));

		//********************************************************************
		// ASCFC_End_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(100, 8) * 6;
		l_dateTimeValue = RWTime(RWDate(), l_intValue / 60, l_intValue % 60) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_End_time", l_dateTimeValue));

		//********************************************************************
		// ASCFC_Depth_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(108, 5);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Depth_pump", l_intValue));

		//********************************************************************
		// ASCFC_Surface_pump
		//********************************************************************

		l_intValue = _bitVec.GetUInt(113, 5);
		p_status.AddAttribute(CoDatAttribute("ASCFC_Surface_pump", l_intValue));

		//********************************************************************
		// PRCFG_Profile_option
		//********************************************************************

		l_intValue = _bitVec.GetUInt(118, 5);
		p_status.AddAttribute(CoDatAttribute("PRCFG_Profile_option", l_intValue));

		//********************************************************************
		// PRCFG_Drift_option
		//********************************************************************

		l_intValue = _bitVec.GetUInt(123, 3);
		p_status.AddAttribute(CoDatAttribute("PRCFG_Drift_option", l_intValue));

		//********************************************************************
		// PRCFG_Cycle_period
		//********************************************************************

		l_intValue = _bitVec.GetUInt(126, 6);
		p_status.AddAttribute(CoDatAttribute("PRCFG_Cycle_period", l_intValue));

		//********************************************************************
		// PRCFG_Reference_day
		//********************************************************************

		l_intValue = _bitVec.GetUInt(132, 6);
		p_status.AddAttribute(CoDatAttribute("PRCFG_Reference_day", l_intValue));

		//********************************************************************
		// ASCFC_Start_time
		//********************************************************************

		l_intValue = _bitVec.GetUInt(138, 5);
		l_dateTimeValue = RWTime(RWDate(), l_intValue, 0) - p_timeCorrection;
		p_status.AddAttribute(CoDatAttribute("ASCFC_Start_time", l_dateTimeValue));

		//********************************************************************
		// PRCFG_Drift_depth
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(143, 11));
		p_status.AddAttribute(CoDatAttribute("PRCFG_Drift_depth", l_doubleValue));

		//********************************************************************
		// IMMFC_Grounded
		//********************************************************************

		l_intValue = _bitVec.GetUInt(154, 1);
		p_status.AddAttribute(CoDatAttribute("IMMFC_Grounded", l_intValue));

		//********************************************************************
		// IMMFC_Grounded_press
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(155, 11));
		p_status.AddAttribute(CoDatAttribute("IMMFC_Grounded_press", l_doubleValue));

		//********************************************************************
		// PRCFG_Profile_depth
		//********************************************************************

		l_doubleValue = double(_bitVec.GetUInt(166, 11));
		p_status.AddAttribute(CoDatAttribute("PRCFG_Profile_depth", l_doubleValue));

		//********************************************************************
		// TIMEC_Internal_time
		//********************************************************************

		l_dateTimeValue = RWTime(RWDate(),
								 _bitVec.GetUInt(177, 5),
								 _bitVec.GetUInt(182, 6),
								 _bitVec.GetUInt(188, 6));
		p_status.AddAttribute(CoDatAttribute("TIMEC_Internal_time", l_dateTimeValue));

		//********************************************************************
		// SENSO_Press_offset
		//********************************************************************

		l_intValue = _bitVec.GetUInt(194, 6);
		if (l_intValue >= 32)
		{
			l_doubleValue = double(int(l_intValue | 0xFFFFFFC0));
		}
		else
		{
			l_doubleValue = double(l_intValue);
		}
		p_status.AddAttribute(CoDatAttribute("SENSO_Press_offset", l_doubleValue));

		//********************************************************************
		// SENSO_Internal_press
		//********************************************************************

		l_intValue = _bitVec.GetUInt(200, 3);
		l_stringValue = CoStc_g_catManager.GetInfo(3, 420 + l_intValue);
		p_status.AddAttribute(CoDatAttribute("SENSO_Internal_press", l_stringValue));

		//********************************************************************
		// PRCFG_Soft_version
		//********************************************************************

		l_intValue = _bitVec.GetUInt(251, 5);
		p_status.AddAttribute(CoDatAttribute("PRCFG_Soft_version", l_intValue));
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 5068", 432);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : TraceRecords
// ROL : Trace the content of a list of records
//
//****************************************************************************
void
CoDatProvorFrame::
TraceRecords(
    ostream& p_traceStream,					// I : the trace stream
	const RWTPtrSlist<CoDatRecord>& p_records	// O : the list of records
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	CoDatRecord* ql_record =				// the current record
		(CoDatRecord*)0;
	CoDatRecord l_recombined;				// the recombined record
	int l_recordCount = 0;					// loop counter
	int l_recordCounter = 0;				// the number of records
	CoDatMeasure l_measure;					// the current measure
	double l_delta = 0.0;					// the delta (relative measure)
	int l_measureCount = 0;					// number of measures
	int l_measureCounter = 0;				// loop counter

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// Adjust to the left
		//********************************************************************

		p_traceStream.setf(ios::left, ios::adjustfield);

		//********************************************************************
		// If the list of records contains at least one record
		//********************************************************************

		if (p_records.entries() > 0)
		{
			//****************************************************************
			// Depending of the measure of the first record, sorts the header
			//****************************************************************

			ql_record = p_records.first();
			if (ql_record->HasMeasure(CoDat_k_ppPressure) == TRUE)
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 404);
			}
			if (ql_record->HasMeasure(CoDat_k_ppTemperature) == TRUE)
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 405);
			}
			if (ql_record->HasMeasure(CoDat_k_ppConductivity) == TRUE)
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 406);
			}
			if (ql_record->HasMeasure(CoDat_k_ppSalinity) == TRUE)
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 407);
			}
			p_traceStream << endl;
			l_measureCount = ql_record->GetMeasureCount();
			for (l_measureCounter = 0; l_measureCounter < l_measureCount; l_measureCounter++)
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 410);
			}
			p_traceStream << endl;
			for (l_measureCounter = 0; l_measureCounter < l_measureCount; l_measureCounter++)
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 411);
			}
			p_traceStream << endl;
		}

		//********************************************************************
		// For each record of the list
		//********************************************************************

		l_recordCount = p_records.entries();
		for (l_recordCounter = 0; l_recordCounter < l_recordCount; l_recordCounter++)
		{
			//****************************************************************
			// Retrieve the current record
			//****************************************************************

			ql_record = p_records.at(l_recordCounter);

			//****************************************************************
			// Trace the pressure ?
			//****************************************************************

			if (ql_record->HasMeasure(CoDat_k_ppPressure) == TRUE)
			{
				//************************************************************
				// Set the output format
				//************************************************************

				p_traceStream.setf(ios::fixed, ios::floatfield);
				p_traceStream.precision(0);

				//************************************************************
				// Retrieve the measure
				//************************************************************

				l_measure = ql_record->GetMeasure(CoDat_k_ppPressure);

				//************************************************************
				// If the measure is absolute
				//************************************************************

				if (l_measure.GetQuality() == '0')
				{
					p_traceStream << RWCString(' ', 10);
					p_traceStream << setw(10) << l_measure.GetValue();
					l_recombined.AddMeasure(l_measure);
				}

				//************************************************************
				// If the measure is relative
				//************************************************************

				else
				{
					l_delta = l_measure.GetValue();
					p_traceStream << setw(10) << l_delta;
					l_measure = l_recombined.GetMeasure(CoDat_k_ppPressure);
					l_measure.SetValue(l_measure.GetValue() + l_delta);
					p_traceStream << setw(10) << l_measure.GetValue();
					l_recombined.AddMeasure(l_measure);
				}

				//************************************************************
				// Reset the output format
				//************************************************************

				p_traceStream.setf(0, ios::floatfield);
				p_traceStream.precision();
			}

			//****************************************************************
			// Trace the temperature ?
			//****************************************************************

			if (ql_record->HasMeasure(CoDat_k_ppTemperature) == TRUE)
			{
				//************************************************************
				// Set the output format
				//************************************************************

				p_traceStream.setf(ios::fixed, ios::floatfield);
				if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
					(_version == 3.21) || (_version == 3.5) || (_version == 3.51) ||
					(_version == 3.52) || (_version == 3.61))
				{
					p_traceStream.precision(2);
				}
				else if ((_version == 3.8) || (_version == 3.81) || (_version == 4.0) ||
						 (_version == 4.1) || (_version == 4.11))
				{
					p_traceStream.precision(3);
				}

				//************************************************************
				// Retrieve the measure
				//************************************************************

				l_measure = ql_record->GetMeasure(CoDat_k_ppTemperature);

				//************************************************************
				// If the measure is absolute
				//************************************************************

				if (l_measure.GetQuality() == '0')
				{
					p_traceStream << RWCString(' ', 10);
					p_traceStream << setw(10) << l_measure.GetValue();
					l_recombined.AddMeasure(l_measure);
				}

				//************************************************************
				// If the measure is relative
				//************************************************************

				else
				{
					l_delta = l_measure.GetValue();
					p_traceStream << setw(10) << l_delta;
					l_measure = l_recombined.GetMeasure(CoDat_k_ppTemperature);
					l_measure.SetValue(l_measure.GetValue() + l_delta);
					p_traceStream << setw(10) << l_measure.GetValue();
					l_recombined.AddMeasure(l_measure);
				}

				//************************************************************
				// Reset the output format
				//************************************************************

				p_traceStream.precision();
				p_traceStream.setf(0, ios::floatfield);
			}

			//****************************************************************
			// Trace the conductivity ?
			//****************************************************************

			if (ql_record->HasMeasure(CoDat_k_ppConductivity) == TRUE)
			{
				//************************************************************
				// Set the output format
				//************************************************************

				p_traceStream.setf(ios::fixed, ios::floatfield);
				if ((_version == 2.2) || (_version == 2.6) || (_version == 2.7) ||
					(_version == 3.21) || (_version == 3.5) || (_version == 3.61))
				{
					p_traceStream.precision(2);
				}
				else if ((_version == 3.8) || (_version == 3.81))
				{
					p_traceStream.precision(3);
				}

				//************************************************************
				// Retrieve the measure
				//************************************************************

				l_measure = ql_record->GetMeasure(CoDat_k_ppConductivity);

				//************************************************************
				// If the measure is absolute
				//************************************************************

				if (l_measure.GetQuality() == '0')
				{
					p_traceStream << RWCString(' ', 10);
					p_traceStream << setw(10) << l_measure.GetValue();
					l_recombined.AddMeasure(l_measure);
				}

				//************************************************************
				// If the measure is relative
				//************************************************************

				else
				{
					l_delta = l_measure.GetValue();
					p_traceStream << setw(10) << l_delta;
					l_measure = l_recombined.GetMeasure(CoDat_k_ppConductivity);
					l_measure.SetValue(l_measure.GetValue() + l_delta);
					p_traceStream << setw(10) << l_measure.GetValue();
					l_recombined.AddMeasure(l_measure);
				}

				//************************************************************
				// Reset the output format
				//************************************************************

				p_traceStream.precision();
				p_traceStream.setf(0, ios::floatfield);
			}

			//****************************************************************
			// Trace the salinity ?
			//****************************************************************

			if (ql_record->HasMeasure(CoDat_k_ppSalinity) == TRUE)
			{
				//************************************************************
				// Set the output format
				//************************************************************

				p_traceStream.setf(ios::fixed, ios::floatfield);
				if ((_version == 3.51) || (_version == 3.52))
				{
					p_traceStream.precision(2);
				}
				else if ((_version == 4.1) || (_version == 4.11))
				{
					p_traceStream.precision(3);
				}
				else if (_version == 4.0)
				{
					p_traceStream.precision(4);
				}

				//************************************************************
				// Retrieve the measure
				//************************************************************

				l_measure = ql_record->GetMeasure(CoDat_k_ppSalinity);

				//************************************************************
				// If the measure is absolute
				//************************************************************

				if (l_measure.GetQuality() == '0')
				{
					p_traceStream << RWCString(' ', 10);
					p_traceStream << setw(10) << l_measure.GetValue();
					l_recombined.AddMeasure(l_measure);
				}

				//************************************************************
				// If the measure is relative
				//************************************************************

				else
				{
					l_delta = l_measure.GetValue();
					p_traceStream << setw(10) << l_delta;
					l_measure = l_recombined.GetMeasure(CoDat_k_ppSalinity);
					l_measure.SetValue(l_measure.GetValue() + l_delta);
					p_traceStream << setw(10) << l_measure.GetValue();
					l_recombined.AddMeasure(l_measure);
				}

				//************************************************************
				// Reset the output format
				//************************************************************

				p_traceStream.precision();
				p_traceStream.setf(0, ios::floatfield);
			}

			//****************************************************************
			// Finish the line
			//****************************************************************

			p_traceStream << endl;
		}

		//********************************************************************
		// Adjust to the right
		//********************************************************************

		p_traceStream.setf(ios::right, ios::adjustfield);
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 5426", 433);
		CO_RETHROW;
	}
}




//****************************************************************************
//
// SPG : TraceTechnicalStatus
// ROL : Trace the content of a technical status
//
//****************************************************************************
void
CoDatProvorFrame::
TraceTechnicalStatus(
    ostream& p_traceStream,					// I : the trace stream
	const CoDatTechnicalStatus& p_status	// O : the technical status
    ) const
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

	int l_attributeCount = 0;				// Number of entries in the technical status
	int l_attributeCounter = 0;				// Loop counter
	CoDatAttribute l_attribute;				// The current attribute
	RWCString l_attributeName = "";			// The name of the current attribute

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

	CO_TRY
	{
		//********************************************************************
		// For each attribute of the technical status
		//********************************************************************

		l_attributeCount = p_status.GetAttributeCount();
		for (l_attributeCounter = 0; l_attributeCounter < l_attributeCount; l_attributeCounter++)
		{
			//****************************************************************
			// Retrieve the current attribute and it's name
			//****************************************************************

			l_attribute = p_status.GetAttribute(l_attributeCounter);
			l_attributeName = l_attribute.GetName();

			//****************************************************************
			// DESFC_Start_time
			//****************************************************************

			if (l_attributeName == "DESFC_Start_time")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 430);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDateTimeValue(), "%H:%M:%S") << endl;
			}

			//****************************************************************
			// DESFC_Surface_valve
			//****************************************************************

			else if (l_attributeName == "DESFC_Surface_valve")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 431);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// DESFC_1st_stab_time
			//****************************************************************

			else if (l_attributeName == "DESFC_1st_stab_time")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 432);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDateTimeValue(), "%H:%M:%S") << endl;
			}

			//****************************************************************
			// DESFC_1st_stab_press
			//****************************************************************

			else if (l_attributeName == "DESFC_1st_stab_press")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 433);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.0lf") << endl;
			}

			//****************************************************************
			// DESFC_Depth_valve
			//****************************************************************

			else if (l_attributeName == "DESFC_Depth_valve")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 434);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// DESFC_Depth_pump
			//****************************************************************

			else if (l_attributeName == "DESFC_Depth_pump")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 435);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// DESFC_End_time
			//****************************************************************

			else if (l_attributeName == "DESFC_End_time")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 436);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDateTimeValue(), "%H:%M:%S") << endl;
			}

			//****************************************************************
			// IMMFC_Depth_correct
			//****************************************************************

			else if (l_attributeName == "IMMFC_Depth_correct")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 437);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// ASCFC_End_time
			//****************************************************************

			else if (l_attributeName == "ASCFC_End_time")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 438);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDateTimeValue(), "%H:%M:%S") << endl;
			}

			//****************************************************************
			// ASCFC_Depth_pump
			//****************************************************************

			else if (l_attributeName == "ASCFC_Depth_pump")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 439);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// ASCFC_Surface_pump
			//****************************************************************

			else if (l_attributeName == "ASCFC_Surface_pump")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 440);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// DESPR_PRSEP
			//****************************************************************

			else if (l_attributeName == "DESPR_PRSEP")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 441);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.0lf") << endl;
			}

			//****************************************************************
			// DESPR_NTS
			//****************************************************************

			else if (l_attributeName == "DESPR_NTS")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 442);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// DESPR_ETS
			//****************************************************************

			else if (l_attributeName == "DESPR_ETS")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 443);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.2lf") << endl;
			}

			//****************************************************************
			// DESPR_NTF
			//****************************************************************

			else if (l_attributeName == "DESPR_NTF")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 446);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// DESPR_ETF
			//****************************************************************

			else if (l_attributeName == "DESPR_ETF")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 447);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.2lf") << endl;
			}

			//****************************************************************
			// ASCPR_PRSEP
			//****************************************************************

			else if (l_attributeName == "ASCPR_PRSEP")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 450);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.0lf") << endl;
			}

			//****************************************************************
			// ASCPR_NTS
			//****************************************************************

			else if (l_attributeName == "ASCPR_NTS")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 451);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// ASCPR_ETS
			//****************************************************************

			else if (l_attributeName == "ASCPR_ETS")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 452);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.2lf") << endl;
			}

			//****************************************************************
			// ASCPR_NTF
			//****************************************************************

			else if (l_attributeName == "ASCPR_NTF")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 455);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// ASCPR_ETF
			//****************************************************************

			else if (l_attributeName == "ASCPR_ETF")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 456);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.2lf") << endl;
			}

			//****************************************************************
			// ITEMP_Internal_count
			//****************************************************************

			else if (l_attributeName == "ITEMP_Internal_count")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 459);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// TIMEC_Internal_time
			//****************************************************************

			else if (l_attributeName == "TIMEC_Internal_time")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 460);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDateTimeValue(), "%H:%M:%S") << endl;
			}

			//****************************************************************
			// SENSO_Press_offset
			//****************************************************************

			else if (l_attributeName == "SENSO_Press_offset")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 461);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.0lf") << endl;
			}

			//****************************************************************
			// SENSO_Internal_press
			//****************************************************************

			else if (l_attributeName == "SENSO_Internal_press")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 462);
				p_traceStream << l_attribute.GetStringValue() << endl;
			}

			//****************************************************************
			// SURFC_Temperature
			//****************************************************************

			else if (l_attributeName == "SURFC_Temperature")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 463);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.3lf") << endl;
			}

			//****************************************************************
			// TRANS_DPROF_Emitted
			//****************************************************************

			else if (l_attributeName == "TRANS_DPROF_Emitted")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 464);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// TRANS_IDRIFT_Emitted
			//****************************************************************

			else if (l_attributeName == "TRANS_IDRIFT_Emitted")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 465);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// TRANS_APROF_Emitted
			//****************************************************************

			else if (l_attributeName == "TRANS_APROF_Emitted")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 466);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// ASCFC_Start_time
			//****************************************************************

			else if (l_attributeName == "ASCFC_Start_time")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 467);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDateTimeValue(), "%H:%M:%S") << endl;
			}

			//****************************************************************
			// IMMFC_Gap_order_ent
			//****************************************************************

			else if (l_attributeName == "IMMFC_Gap_order_ent")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 468);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// IMMFC_Min_pressure
			//****************************************************************

			else if (l_attributeName == "IMMFC_Min_pressure")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 469);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.0lf") << endl;
			}

			//****************************************************************
			// IMMFC_Max_pressure
			//****************************************************************

			else if (l_attributeName == "IMMFC_Max_pressure")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 470);
				p_traceStream << CoStcTools_ToString(l_attribute.GetDoubleValue(), "%.0lf") << endl;
			}

			//****************************************************************
			// IMMFC_Grounded
			//****************************************************************

			else if (l_attributeName == "IMMFC_Grounded")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 471);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// PRCFG_Profile_option
			//****************************************************************

			else if (l_attributeName == "PRCFG_Profile_option")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 472);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// PRCFG_Drift_option
			//****************************************************************

			else if (l_attributeName == "PRCFG_Drift_option")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 473);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// PRCFG_Cycle_period
			//****************************************************************

			else if (l_attributeName == "PRCFG_Cycle_period")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 474);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// PRCFG_Reference_day
			//****************************************************************

			else if (l_attributeName == "PRCFG_Reference_day")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 475);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}

			//****************************************************************
			// PRCFG_Drift_depth
			//****************************************************************

			else if (l_attributeName == "PRCFG_Drift_depth")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 476);
				p_traceStream << l_attribute.GetDoubleValue() << endl;
			}

			//****************************************************************
			// IMMFC_Grounded_press
			//****************************************************************

			else if (l_attributeName == "IMMFC_Grounded_press")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 477);
				p_traceStream << l_attribute.GetDoubleValue() << endl;
			}

			//****************************************************************
			// PRCFG_Profile_depth
			//****************************************************************

			else if (l_attributeName == "PRCFG_Profile_depth")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 478);
				p_traceStream << l_attribute.GetDoubleValue() << endl;
			}

			//****************************************************************
			// PRCFG_Soft_version
			//****************************************************************

			else if (l_attributeName == "PRCFG_Soft_version")
			{
				p_traceStream << CoStc_g_catManager.GetInfo(3, 479);
				p_traceStream << l_attribute.GetIntValue() << endl;
			}
		}
	}

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

	CO_CATCH(CoStc_e_Error)
	{
		CoStc_g_error.Send(3, "CoDatProvorFrame.cpp 1.12 5904", 434);
		CO_RETHROW;
	}
}








