//****************************************************************************
//
// PRO : CORIOLIS
// MOD : CoDatSurfaceDriftFrame 
// ROL : This class implements a surface drift frame. It is initialized with
//       the informations contained in the header of an Argos message.
//
// CRE : 28/08/1999
// AUT : ST
// VER : @(#)1.4
//       @(#)02/02/26
//****************************************************************************
// 
// HST : 28/08/1999 ST  Creation
//       04/10/1999 ST  Manage temperature, conductivity and salinity
//		 26/02/2002 ST  Use CoDatPosition instead of CoStcLocalization
//
//****************************************************************************
//
// Copyright, IFREMER, 1999
//
//****************************************************************************
static const char CoDatSurfaceDriftFrame_IDENT[] = "@(#) CORIOLIS : CoDatSurfaceDriftFrame.cpp 1.4 02/02/26";


//----------------------------------------------------------------------------
// INTERFACE
//----------------------------------------------------------------------------
#define _CoDatSurfaceDriftFrame_IN_
#include <CoDatSurfaceDriftFrame.h>
#undef  _CoDatSurfaceDriftFrame_IN_
#include <limits.h>
#include <stdio.h>
#include <rw/rwdate.h>
#include <CoStcError.h>




//****************************************************************************
//
// SPG : CoDatSurfaceDriftFrame
// ROL : Constructor
//
//****************************************************************************
CoDatSurfaceDriftFrame::
CoDatSurfaceDriftFrame(
    )
    : CoDatDataFrame()
    , _qualityClass('/0')
    , _dateTime((unsigned long)0)
    , _position()
    , _temperature(DBL_MAX)
    , _conductivity(DBL_MAX)
    , _salinity(DBL_MAX)
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
	CoStc_g_error.Send(3, "CoDatSurfaceDriftFrame.cpp 1.4 75", 301);
	_isBad = 1;
    }
}




//****************************************************************************
//
// SPG : ~CoDatSurfaceDriftFrame
// ROL : Destructor
//
//****************************************************************************
CoDatSurfaceDriftFrame::
~CoDatSurfaceDriftFrame(
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

}



//$##
//****************************************************************************
//
// SPG : Initialize
//
// ROL : Initialize the instance with the informations extracted from the
//	 header of the Argos message
//
//****************************************************************************
void
CoDatSurfaceDriftFrame::
Initialize(
    const RWCString& p_datas			// I : the data string
    )
{
//----------------------------------------------------------------------------
// DONNEES LOCALES
//----------------------------------------------------------------------------

    int l_year = 0;							// The year
    int l_month = 0;						// The month
    int l_day = 0;							// The day in the month
    int l_hours = 0;						// The hours in the day
    int l_minutes = 0;						// The minutes in the hour
    int l_seconds = 0;						// The seconds in the minute
    double l_latitude;						// The latitude
    double l_longitude;						// The longitude

//----------------------------------------------------------------------------
// TRAITEMENT
//----------------------------------------------------------------------------

    CO_TRY
    {
	//********************************************************************
	// The instance is supposed to be valid 
	//********************************************************************

	_isBad = FALSE;

	//********************************************************************
	// Retrieve the quality class
	//********************************************************************

	if (p_datas.length() > 1)
	{
	    _qualityClass = p_datas[0];
	    if ((_qualityClass < '1') || (_qualityClass > '3'))
	    {
		_isBad = TRUE;
	    }
	}
	else
	{
	    _qualityClass = '/0';
	    _isBad = TRUE;
	}

	//********************************************************************
	// Retrieve the date-time
	//********************************************************************

	if (_isBad == FALSE)
	{
	    if ((p_datas.length() > 3) &&
		(sscanf(p_datas.data() + 2, "%d-%d-%d %d:%d:%d",
			&l_year, &l_month, &l_day,
			&l_hours, &l_minutes, &l_seconds) == 6))
	    {
		_dateTime = RWTime(RWDate(l_day, l_month, l_year),
				   l_hours, l_minutes, l_seconds);
	    }
	    else
	    {
		_dateTime = RWTime((unsigned long)0);
		_isBad = TRUE;
	    }
	}

	//********************************************************************
	// Retrieve the position
	//********************************************************************

	if (_isBad == FALSE)
	{
	    if ((p_datas.length() > 22) &&
		(sscanf(p_datas.data() + 21, "%lf %lf",
			&l_latitude, &l_longitude) == 2))
	    {
		_position.SetLatitude(l_latitude);
		_position.SetLongitude(l_longitude);
	    }
	    else
	    {
		_position = CoDatPosition();
		_isBad = TRUE;
	    }
	}
    }

//----------------------------------------------------------------------------
// ERREURS ET RETOUR
//----------------------------------------------------------------------------

    CO_CATCH(CoStc_e_Error)
    {
	CoStc_g_error.Send(3, "CoDatSurfaceDriftFrame.cpp 1.4 215", 302);
	CO_RETHROW;
    }
}











